@@ -56,6 +56,7 @@ public:
                                     MAP_PRIVATE | MAP_SHARED | MAP_STACK |
                                     MAP_GROWSDOWN;
   static const int checkable_flags_mask = MAP_PRIVATE | MAP_SHARED;
+  static const int checkable_prot_mask = PROT_READ | PROT_WRITE | PROT_EXEC;
   static const dev_t NO_DEVICE = 0;
   static const ino_t NO_INODE = 0;
 
@@ -151,7 +152,11 @@ public:
   bool is_heap() const { return fsname() == "[heap]"; }
   bool is_stack() const { return fsname().find("[stack") == 0; }
   bool is_vvar() const { return fsname() == "[vvar]"; }
+  bool is_vvar_vclock() const { return fsname() == "[vvar_vclock]"; }
   bool is_vsyscall() const { return fsname() == "[vsyscall]"; }
+  bool is_named_anonymous() const {
+    return fsname().find("[anon:") == 0 || fsname().find("[anon_shmem:") == 0;
+  }
 
   struct stat fake_stat() const {
     struct stat fake_stat;
@@ -278,7 +283,7 @@ public:
       return *this;
     }
     Mapping subrange(MemoryRange range,
-                     std::function<KernelMapping(const KernelMapping&)> f);
+                     std::function<KernelMapping(const KernelMapping&)> f) const;
 
     const KernelMapping map;
     // The corresponding KernelMapping in the recording. During recording,
@@ -910,6 +915,14 @@ public:
   // Returns true if the range is completely covered by private mappings
   bool range_is_private_mapping(const MemoryRange& range) const;
 
+  /**
+   * When two processes share an address space (e.g. with vfork(2) or
+   * clone(2) CLONE_VM), and one process calls execve(2), we need to unmap
+   * that process's syscallbuf. This list is checked the next time a task
+   * in that address space runs to perform the unmapping
+   */
+  std::vector<MemoryRange> regions_pending_unmap;
+
 private:
   struct Breakpoint;
   typedef std::map<remote_code_ptr, Breakpoint> BreakpointMap;
