@@ -20,6 +20,7 @@
 #include "kernel_abi.h"
 #include "kernel_metadata.h"
 #include "log.h"
+#include "record_signal.h"
 #include "util.h"
 
 using namespace std;
@@ -97,7 +98,8 @@ AutoRemoteSyscalls::AutoRemoteSyscalls(Task* t,
       use_singlestep_path(false),
       enable_mem_params_(enable_mem_params),
       restore_sigmask(false),
-      need_sigpending_renable(false) {
+      need_sigpending_renable(false),
+      need_desched_event_reenable(false) {
   if (initial_at_seccomp) {
     // This should only ever happen during recording - we don't use the
     // seccomp traps during replay.
@@ -142,15 +144,28 @@ AutoRemoteSyscalls::AutoRemoteSyscalls(Task* t,
   }
   if (t->session().is_recording()) {
     RecordTask *rt = static_cast<RecordTask*>(t);
+    sig_set_t signals_to_block = 0;
+
     if (rt->schedule_frozen) {
       // If we're explicitly controlling the schedule, make sure not to accidentally run
       // any signals that we were not meant to be able to see.
+      memset(&signals_to_block, 0xff, sizeof(sig_set_t));
+    }
+    if (desched_event_armed(rt)) {
+      // If the desched event is enabled, we need to disable it, so that we don't get
+      // the desched signal interrupting the syscall we're trying to make. We also
+      // need to mask it, so that if there's a pending desched signal from before
+      // we disable it, we don't accidently steal it.
+      signals_to_block |= signal_bit(rt->session().syscallbuf_desched_sig());
+      need_desched_event_reenable = true;
+      disarm_desched_event(rt);
+    }
+
+    if (signals_to_block) {
       restore_sigmask = true;
       sigmask_to_restore = rt->get_sigmask();
-      sig_set_t all_blocked;
-      memset(&all_blocked, 0xff, sizeof(all_blocked));
       // Ignore the process dying here - we'll notice later.
-      (void)rt->set_sigmask(all_blocked);
+      (void)rt->set_sigmask(signals_to_block | sigmask_to_restore);
     }
   }
 }
@@ -325,6 +340,9 @@ void AutoRemoteSyscalls::restore_state_to(Task* t) {
   if (restore_sigmask) {
     static_cast<RecordTask*>(t)->set_sigmask(sigmask_to_restore);
   }
+  if (need_desched_event_reenable) {
+    arm_desched_event(static_cast<RecordTask*>(t));
+  }
   if (need_sigpending_renable) {
     // The purpose of this PTRACE_INTERRUPT is to re-enable TIF_SIGPENDING on
     // the tracee, without forcing any actual signals on it. Since PTRACE_INTERRUPT
@@ -601,6 +619,31 @@ static long child_recvmsg(AutoRemoteSyscalls& remote, int child_sock) {
     LOG(debug) << "Failed to recvmsg " << ret;
     return ret;
   }
+
+  typename Arch::msghdr msghdr =
+      remote.task()->read_mem(msg.remote_msg(), &ok);
+  if (!ok) {
+    ASSERT(remote.task(), errno == ESRCH || errno == EIO);
+    LOG(debug) << "Failed to read msghdr";
+    return -ESRCH;
+  }
+  ASSERT(remote.task(), !(msghdr.msg_flags & MSG_CTRUNC))
+      << "Control message was truncated; error in receiving fd in "
+         "AutoRemoteSyscalls::child_recvmsg(). msghdr.msg_flags: "
+      << HEX(msghdr.msg_flags) << "\n"
+      << "This error has been most likely caused by a process\n"
+      << "exceeding the max allowed open files limit set by\n"
+      << "Linux. Please consult `man 1 ulimit' and `man 1 prlimit' to\n"
+      << "learn how the max open files limit may be changed/checked.\n"
+      << "As usual, always carefully think through all implications of\n"
+      << "changing the process limits on your programs before making any\n"
+      << "changes.\n\n"
+      << "If the above Assertion still fails, then (a) The limit you set was\n"
+      << "not high enough, or (b) the program could be opening files in an\n"
+      << "unbounded fashion, or (c) there is some other reason why socket\n"
+      << "control messages are being truncated and file descriptors cannot be\n"
+      << "received via SCM_RIGHTS.";
+
   int their_fd = remote.task()->read_mem(msg.remote_cmsgdata(), &ok);
   if (!ok) {
     ASSERT(remote.task(), errno == ESRCH || errno == EIO);
@@ -837,13 +880,13 @@ int64_t AutoRemoteSyscalls::infallible_lseek_syscall(int fd, int64_t offset,
   }
 }
 
-void AutoRemoteSyscalls::check_syscall_result(long ret, int syscallno, bool allow_death) {
+bool AutoRemoteSyscalls::check_syscall_result(long ret, int syscallno, bool allow_death) {
   if (word_size(t->arch()) == 4) {
     // Sign-extend ret because it can be a 32-bit negative errno
     ret = (int)ret;
   }
   if (ret == -ESRCH && allow_death && !t->session().is_replaying()) {
-    return;
+    return true;
   }
   if (-4096 < ret && ret < 0) {
     string extra_msg;
@@ -861,6 +904,7 @@ void AutoRemoteSyscalls::check_syscall_result(long ret, int syscallno, bool allo
                      << " arg3=0x" << hex << t->regs().arg3() << " arg4=0x" << t->regs().arg4()
                      << " arg5=0x" << hex << t->regs().arg5() << " arg6=0x" << t->regs().arg6();
   }
+  return false;
 }
 
 void AutoRemoteSyscalls::finish_direct_mmap(
