@@ -12,11 +12,12 @@
 
 namespace rr {
 
+class AddressSpace;
 class RecordTask;
 class ReplayTask;
 class Task;
 
-class FdTable : public HasTaskSet {
+class FdTable final : public HasTaskSet {
 public:
   typedef std::shared_ptr<FdTable> shr_ptr;
 
@@ -32,18 +33,17 @@ public:
   Switchable will_write(Task* t, int fd);
   void did_write(Task* t, int fd, const std::vector<FileMonitor::Range>& ranges,
                  FileMonitor::LazyOffset& offset);
-  void did_dup(int from, int to);
+  void did_dup(int from, int to) {
+    did_dup(this, from, to);
+  }
+  void did_dup(FdTable* table, int from, int to);
   void did_close(int fd);
 
   shr_ptr clone() const {
     return shr_ptr(new FdTable(*this));
   }
 
-  static shr_ptr create(Task* t) {
-    shr_ptr fds(new FdTable());
-    fds->insert_task(t);
-    return fds;
-  }
+  static shr_ptr create(Task* t);
 
   bool is_monitoring(int fd) const { return fds.count(fd) > 0; }
   uint32_t count_beyond_limit() const { return fd_count_beyond_limit; }
@@ -70,16 +70,39 @@ public:
    */
   void close_after_exec(ReplayTask* t, const std::vector<int>& fds_to_close);
 
+  // Used to optimize ReplayTask's find_free_file_descriptor
+  int last_free_fd() const { return last_free_fd_; }
+  void set_last_free_fd(int last_free_fd) { last_free_fd_ = last_free_fd; }
+
+  void insert_task(Task* t) override;
+  void erase_task(Task* t) override;
+
 private:
-  FdTable() : fd_count_beyond_limit(0) {}
+  explicit FdTable(uint32_t syscallbuf_fds_disabled_size)
+    : syscallbuf_fds_disabled_size(syscallbuf_fds_disabled_size),
+      fd_count_beyond_limit(0), last_free_fd_(0) {}
+  // Does not call the base-class copy constructor because
+  // we don't want to copy the task set; the new FdTable will
+  // be for new tasks.
   FdTable(const FdTable& other) : fds(other.fds),
-    fd_count_beyond_limit(other.fd_count_beyond_limit) {}
+    syscallbuf_fds_disabled_size(other.syscallbuf_fds_disabled_size),
+    fd_count_beyond_limit(other.fd_count_beyond_limit),
+    last_free_fd_(other.last_free_fd_) {}
 
   void update_syscallbuf_fds_disabled(int fd);
 
   std::unordered_map<int, FileMonitor::shr_ptr> fds;
-  // Number of elements of `fds` that are >= SYSCALLBUF_FDS_DISABLED_SIZE
+  std::unordered_map<AddressSpace*, int> vms;
+  // Currently this is only used during recording, so we could use
+  // SYSCALLBUF_FDS_DISABLED_SIZE directly and not bother tracking it in
+  // the trace header, but to make things less fragile in case we ever need to
+  // know it during replay, we track it here.
+  int syscallbuf_fds_disabled_size;
+  // Number of elements of `fds` that are >= syscallbuf_fds_disabled_size.
+  // Only used during recording.
   uint32_t fd_count_beyond_limit;
+  // Only used during recording.
+  int last_free_fd_;
 };
 
 } // namespace rr
