@@ -20,7 +20,7 @@
 #include "BreakpointCondition.h"
 #include "ElfReader.h"
 #include "Event.h"
-#include "GdbCommandHandler.h"
+#include "DebuggerExtensionCommandHandler.h"
 #include "GdbServerExpression.h"
 #include "ReplaySession.h"
 #include "ReplayTask.h"
@@ -88,7 +88,7 @@ GdbServer::GdbServer(std::unique_ptr<GdbServerConnection>& connection,
  * value that can be named by |regname|.
  */
 static size_t get_reg(const Registers& regs, const ExtraRegisters& extra_regs,
-                      uint8_t* buf, GdbRegister regname, bool* defined) {
+                      uint8_t* buf, GdbServerRegister regname, bool* defined) {
   size_t num_bytes = regs.read_register(buf, regname, defined);
   if (!*defined) {
     num_bytes = extra_regs.read_register(buf, regname, defined);
@@ -96,7 +96,7 @@ static size_t get_reg(const Registers& regs, const ExtraRegisters& extra_regs,
   return num_bytes;
 }
 
-static bool set_reg(Task* target, const GdbRegisterValue& reg) {
+static bool set_reg(Task* target, const GdbServerRegisterValue& reg) {
   if (!reg.defined) {
     return false;
   }
@@ -122,10 +122,10 @@ static bool set_reg(Task* target, const GdbRegisterValue& reg) {
 /**
  * Return the register |which|, which may not have a defined value.
  */
-GdbRegisterValue GdbServer::get_reg(const Registers& regs,
+GdbServerRegisterValue GdbServer::get_reg(const Registers& regs,
                                     const ExtraRegisters& extra_regs,
-                                    GdbRegister which) {
-  GdbRegisterValue reg;
+                                    GdbServerRegister which) {
+  GdbServerRegisterValue reg;
   memset(&reg, 0, sizeof(reg));
   reg.name = which;
   reg.size = rr::get_reg(regs, extra_regs, &reg.value[0], which, &reg.defined);
@@ -186,7 +186,7 @@ static void maybe_singlestep_for_event(Task* t, GdbRequest* req) {
 
 void GdbServer::dispatch_regs_request(const Registers& regs,
                                       const ExtraRegisters& extra_regs) {
-  GdbRegister end;
+  GdbServerRegister end;
   // Send values for all the registers we sent XML register descriptions for.
   // Those descriptions are controlled by GdbServerConnection::cpu_features().
   bool have_PKU = dbg->cpu_features() & GdbServerConnection::CPU_PKU;
@@ -205,8 +205,9 @@ void GdbServer::dispatch_regs_request(const Registers& regs,
       FATAL() << "Unknown architecture";
       return;
   }
-  vector<GdbRegisterValue> rs;
-  for (GdbRegister r = GdbRegister(0); r <= end; r = GdbRegister(r + 1)) {
+  vector<GdbServerRegisterValue> rs;
+  rs.reserve(end);
+  for (GdbServerRegister r = GdbServerRegister(0); r <= end; r = GdbServerRegister(r + 1)) {
     rs.push_back(get_reg(regs, extra_regs, r));
   }
   dbg->reply_get_regs(rs);
@@ -313,7 +314,7 @@ static vector<GdbServerConnection::ThreadInfo> thread_info(const Session& sessio
 
 void GdbServer::notify_stop_internal(const Session& session,
                                      ExtendedTaskId which, int sig,
-                                     const char *reason) {
+                                     const string& reason) {
   dbg->notify_stop(which, sig, thread_info(session), reason);
 }
 
@@ -638,7 +639,7 @@ void GdbServer::dispatch_debugger_request(Session& session,
       return;
     }
     case DREQ_GET_REG: {
-      GdbRegisterValue reg =
+      GdbServerRegisterValue reg =
           get_reg(target->regs(), *target->extra_regs_fallible(), req.reg().name);
       dbg->reply_get_reg(reg);
       return;
@@ -794,7 +795,7 @@ void GdbServer::dispatch_debugger_request(Session& session,
     }
     case DREQ_RR_CMD:
       dbg->reply_rr_cmd(
-          GdbCommandHandler::process_command(*this, target, req.rr_cmd()));
+          DebuggerExtensionCommandHandler::process_command(*this, target, req.rr_cmd()));
       return;
 #ifdef PROC_SERVICE_H
     case DREQ_QSYMBOL: {
@@ -961,10 +962,10 @@ bool GdbServer::diverter_process_debugger_requests(
         Task* task = diversion_session.find_task(last_continue_task.tuid);
         if (task) {
           std::string reply =
-              GdbCommandHandler::process_command(*this, task, req->rr_cmd());
+              DebuggerExtensionCommandHandler::process_command(*this, task, req->rr_cmd());
           // Certain commands cause the diversion to end immediately
           // while other commands must work within a diversion.
-          if (reply == GdbCommandHandler::cmd_end_diversion()) {
+          if (reply == DebuggerExtensionCommandHandler::cmd_end_diversion()) {
             diversion_refcount = 0;
             return false;
           }
@@ -1007,14 +1008,12 @@ void GdbServer::maybe_notify_stop(const Session& session,
                                   const GdbRequest& req,
                                   const BreakStatus& break_status) {
   bool do_stop = false;
-  remote_ptr<void> watch_addr;
-  char watch[1024];
-  watch[0] = '\0';
+  stringstream reason;
   if (!break_status.watchpoints_hit.empty()) {
     do_stop = true;
     memset(&stop_siginfo, 0, sizeof(stop_siginfo));
     stop_siginfo.si_signo = SIGTRAP;
-    watch_addr = break_status.watchpoints_hit[0].addr;
+    remote_ptr<void> watch_addr = break_status.watchpoints_hit[0].addr;
     bool any_hw_break = false;
     for (const auto& w : break_status.watchpoints_hit) {
       if (w.type == WATCH_EXEC) {
@@ -1022,9 +1021,9 @@ void GdbServer::maybe_notify_stop(const Session& session,
       }
     }
     if (dbg->hwbreak_supported() && any_hw_break) {
-      snprintf(watch, sizeof(watch) - 1, "hwbreak:;");
+      reason << "hwbreak:;";
     } else if (watch_addr) {
-      snprintf(watch, sizeof(watch) - 1, "watch:%" PRIxPTR ";", watch_addr.as_int());
+      reason << "watch:" << std::hex << watch_addr.as_int() << ";";
     }
     LOG(debug) << "Stopping for watchpoint at " << watch_addr;
   }
@@ -1034,7 +1033,7 @@ void GdbServer::maybe_notify_stop(const Session& session,
     stop_siginfo.si_signo = SIGTRAP;
     if (break_status.breakpoint_hit) {
       if (dbg->swbreak_supported()) {
-        snprintf(watch, sizeof(watch) - 1, "swbreak:;");
+        reason << "swbreak:;";
       }
       LOG(debug) << "Stopping for breakpoint";
     } else {
@@ -1071,6 +1070,9 @@ void GdbServer::maybe_notify_stop(const Session& session,
       } else {
         // The start of the debuggee task-group should trigger a silent stop.
         stop_siginfo.si_signo = 0;
+        // Documented as "has reached the end (or the beginning when executing
+        // backward) of the log"
+        reason << "replaylog:begin;";
         LOG(debug) << "Stopping at start of execution while running backwards";
       }
     }
@@ -1087,7 +1089,7 @@ void GdbServer::maybe_notify_stop(const Session& session,
     /* Notify the debugger and process any new requests
      * that might have triggered before resuming. */
     notify_stop_internal(session, extended_task_id(t), stop_siginfo.si_signo,
-                         watch);
+                         reason.str());
     last_query_task = last_continue_task = extended_task_id(t);
   }
 }
@@ -1369,14 +1371,21 @@ GdbServer::ContinueOrStop GdbServer::handle_exited_state(
   if (timeline_) {
     final_event = timeline_->current_session().trace_reader().time();
   }
-  GdbRequest req = process_debugger_requests(REPORT_THREADS_DEAD);
-  ContinueOrStop s;
-  if (detach_or_restart(req, &s)) {
-    last_resume_request = GdbRequest();
-    return s;
+  while (true) {
+    GdbRequest req = process_debugger_requests(REPORT_THREADS_DEAD);
+    ContinueOrStop s;
+    if (detach_or_restart(req, &s)) {
+      last_resume_request = GdbRequest();
+      return s;
+    }
+    if (req.type == DREQ_INTERRUPT) {
+      // Ignore this. Sometimes LLDB seems to send it automatically
+      // after the task has exited, before we detach I guess.
+      continue;
+    }
+    FATAL() << "Received continue/interrupt request after end-of-trace: "
+            << req.type;
   }
-  FATAL() << "Received continue/interrupt request after end-of-trace.";
-  return STOP_DEBUGGING;
 }
 
 ReplayTask* GdbServer::require_timeline_current_task() {
@@ -1793,6 +1802,7 @@ void GdbServer::restart_session(const GdbRequest& req) {
 void GdbServer::serve_replay(std::shared_ptr<ReplaySession> session,
                              const Target& target,
                              volatile bool* stop_replaying_to_target,
+                             DebuggerType debugger_type,
                              const ConnectionFlags& flags) {
   ReplayTimeline timeline(std::move(session));
 
@@ -1828,8 +1838,8 @@ void GdbServer::serve_replay(std::shared_ptr<ReplaySession> session,
     DEBUG_ASSERT(nwritten == sizeof(params));
   } else {
     vector<string> cmd = debugger_launch_command(t, listen_socket.domain,
-        listen_socket.host, listen_socket.port,
-        flags.serve_files, flags.debugger_name);
+        listen_socket.host, listen_socket.port, flags.serve_files,
+        flags.debugger_name, debugger_type);
     fprintf(stderr, "Launch debugger with\n  %s\n", to_shell_string(cmd).c_str());
   }
 
@@ -1845,7 +1855,8 @@ void GdbServer::serve_replay(std::shared_ptr<ReplaySession> session,
 
   do {
     LOG(debug) << "initializing debugger connection";
-    auto connection = GdbServerConnection::await_connection(t, listen_socket.fd);
+    auto connection = GdbServerConnection::await_connection(t, listen_socket.fd,
+      debugger_type);
 
     GdbServer server(connection, timeline.current_session().current_task(),
                      &timeline, target);
@@ -1946,6 +1957,7 @@ static bool is_likely_interp(string fsname) {
 #endif
 }
 
+#ifndef __BIONIC__
 static remote_ptr<void> base_addr_from_rendezvous(Task* t, string fname)
 {
   remote_ptr<void> interpreter_base = t->vm()->saved_interpreter_base();
@@ -1987,6 +1999,7 @@ static remote_ptr<void> base_addr_from_rendezvous(Task* t, string fname)
   }
   return nullptr;
 }
+#endif
 
 int GdbServer::open_file(Session& session, Task* continue_task, const std::string& file_name) {
   // XXX should we require file_scope_pid == 0 here?
@@ -2062,6 +2075,7 @@ int GdbServer::open_file(Session& session, Task* continue_task, const std::strin
     // Last ditch attempt: Dig through the tracee's libc rendezvous struct to
     // see if we can find this file by a different name (e.g. if it was opened
     // via symlink)
+#ifndef __BIONIC__
     remote_ptr<void> base = base_addr_from_rendezvous(continue_task, file_name);
     if (base != nullptr && continue_task->vm()->has_mapping(base)) {
       int ret_fd = 0;
@@ -2072,6 +2086,7 @@ int GdbServer::open_file(Session& session, Task* continue_task, const std::strin
       memory_files.insert(make_pair(ret_fd, FileId(continue_task->vm()->mapping_of(base).recorded_map)));
       return ret_fd;
     }
+#endif
     LOG(debug) << "... not found";
     return -1;
    }
