@@ -12,13 +12,15 @@
 #include <unordered_set>
 #include <vector>
 
-#include "GdbRegister.h"
+#include "GdbServerRegister.h"
 #include "Registers.h"
 #include "ReplaySession.h"
 #include "ReplayTimeline.h"
 #include "ScopedFd.h"
+#include "TargetDescription.h"
 #include "TaskishUid.h"
 #include "core.h"
+#include "launch_debugger.h"
 
 namespace rr {
 
@@ -80,9 +82,9 @@ inline std::ostream& operator<<(std::ostream& o, const ExtendedTaskId& t) {
  * Represents a possibly-undefined register |name|.  |size| indicates how
  * many bytes of |value| are valid, if any.
  */
-struct GdbRegisterValue {
+struct GdbServerRegisterValue {
   enum { MAX_SIZE = Registers::MAX_SIZE };
-  GdbRegister name;
+  GdbServerRegister name;
   union {
     uint8_t value[MAX_SIZE];
     uint8_t value1;
@@ -274,7 +276,7 @@ struct GdbRequest {
     int kind = 0;
     std::vector<std::vector<uint8_t>> conditions;
   } watch_;
-  GdbRegisterValue reg_;
+  GdbServerRegisterValue reg_;
   struct Restart {
     int64_t param = 0;
     std::string param_str;
@@ -342,11 +344,11 @@ struct GdbRequest {
     DEBUG_ASSERT(type >= DREQ_WATCH_FIRST && type <= DREQ_WATCH_LAST);
     return watch_;
   }
-  GdbRegisterValue& reg() {
+  GdbServerRegisterValue& reg() {
     DEBUG_ASSERT(type >= DREQ_REG_FIRST && type <= DREQ_REG_LAST);
     return reg_;
   }
-  const GdbRegisterValue& reg() const {
+  const GdbServerRegisterValue& reg() const {
     DEBUG_ASSERT(type >= DREQ_REG_FIRST && type <= DREQ_REG_LAST);
     return reg_;
   }
@@ -486,7 +488,8 @@ public:
    * debugging context, or it won't return.
    */
   static std::unique_ptr<GdbServerConnection> await_connection(
-    Task* t, ScopedFd& listen_fd, const Features& features = Features());
+    Task* t, ScopedFd& listen_fd, DebuggerType debugger_type,
+    const Features& features = Features());
 
   /**
    * Call this when the target of |req| is needed to fulfill the
@@ -536,7 +539,7 @@ public:
    */
   void notify_stop(ExtendedTaskId which, int sig,
                    const std::vector<ThreadInfo>& threads,
-                   const char *reason);
+                   const std::string& reason);
 
   /** Notify the debugger that a restart request failed. */
   void notify_restart_failed();
@@ -617,13 +620,13 @@ public:
   /**
    * Send |value| back to the debugger host.  |value| may be undefined.
    */
-  void reply_get_reg(const GdbRegisterValue& value);
+  void reply_get_reg(const GdbServerRegisterValue& value);
 
   /**
    * Send |file| back to the debugger host.  |file| may contain
    * undefined register values.
    */
-  void reply_get_regs(const std::vector<GdbRegisterValue>& file);
+  void reply_get_regs(const std::vector<GdbServerRegisterValue>& file);
 
   /**
    * Pass |ok = true| iff the requested register was successfully set.
@@ -750,10 +753,12 @@ public:
     CPU_AARCH64 = 0x4,
     CPU_PKU = 0x8
   };
-  void set_cpu_features(uint32_t features) { cpu_features_ = features; }
+
+  void set_cpu_features(SupportedArch arch);
   uint32_t cpu_features() const { return cpu_features_; }
 
-  GdbServerConnection(ThreadGroupUid tguid, const Features& features);
+  GdbServerConnection(ThreadGroupUid tguid, DebuggerType debugger_type,
+    const Features& features);
 
   /**
    * Wait for a debugger client to connect to |dbg|'s socket.  Blocks
@@ -792,6 +797,7 @@ private:
   void write_hex_bytes_packet(const uint8_t* bytes, size_t len);
   void write_xfer_response(const void* data, size_t size, uint64_t offset,
                            uint64_t len);
+  void parse_thread_suffix_threadid(char* payload, GdbThreadId* out);
   /**
    * Consume bytes in the input buffer until start-of-packet ('$') or
    * the interrupt character is seen.  Does not block.  Return true if
@@ -845,7 +851,7 @@ private:
   void consume_request();
   void send_stop_reply_packet(ExtendedTaskId thread, int sig,
                               const std::vector<ThreadInfo>& threads,
-                              const char *reason);
+                              const std::string& reason);
   void send_file_error_reply(int system_errno);
   std::string format_thread_id(ExtendedTaskId thread);
 
@@ -860,6 +866,7 @@ private:
   // this thread group exists when interfacing with gdb
   ThreadGroupUid tguid;
   uint32_t cpu_features_;
+  DebuggerType debugger_type;
   // true when "no-ack mode" enabled, in which we don't have
   // to send ack packets back to gdb.  This is a huge perf win.
   bool no_ack;
@@ -877,6 +884,7 @@ private:
   bool hwbreak_supported_; // client supports hwbreak extension
   bool swbreak_supported_; // client supports swbreak extension
   bool list_threads_in_stop_reply_; // client requested threads: and thread-pcs: in stop replies
+  std::unique_ptr<TargetDescription> target_description;
 };
 
 } // namespace rr
