@@ -18,12 +18,21 @@ void HasTaskSet::erase_task(Task* t) {
 }
 
 Task* HasTaskSet::first_running_task() const {
-  for (auto t : task_set()) {
-    if (!t->already_exited() && !t->is_dying()) {
+  for (Task* t : task_set()) {
+    if (!t->already_exited() && !t->seen_ptrace_exit_event()) {
       return t;
     }
   }
   return nullptr;
 }
 
+Task* HasTaskSet::find_other_thread_group(Task* t) const {
+  for (Task* tt : task_set()) {
+    if (tt->thread_group() != t->thread_group()) {
+      return tt;
+    }
+  }
+  return nullptr;
+}
+
 } // namespace rr
