@@ -14,15 +14,10 @@ class MagicSaveDataMonitor : public FileMonitor {
 public:
   MagicSaveDataMonitor() {}
 
-  virtual Type type() { return MagicSaveData; }
+  virtual Type type() override { return MagicSaveData; }
 
-  /**
-   * During recording, record the written data.
-   * During replay, check that the written data matches what was recorded.
-   */
-  virtual bool needs_offset(Task*, bool) { return false; }
   virtual void did_write(Task* t, const std::vector<Range>& ranges,
-                         LazyOffset& offset);
+                         LazyOffset& offset) override;
 };
 
 } // namespace rr
