@@ -528,7 +528,6 @@ static bool patch_syscall_with_hook_x86ish(Monkeypatcher& patcher,
                                            size_t instruction_length,
                                            uint32_t fake_syscall_number) {
   size_t patch_region_size = instruction_length + hook.patch_region_length;
-  uint8_t jump_patch[patch_region_size];
   // We're patching in a relative jump, so we need to compute the offset from
   // the end of the jump to our actual destination.
   remote_ptr<uint8_t> jump_patch_start = ip_of_instruction.to_data_ptr<uint8_t>();
@@ -580,7 +579,8 @@ static bool patch_syscall_with_hook_x86ish(Monkeypatcher& patcher,
 
   // pad with NOPs to the next instruction
   static const uint8_t NOP = 0x90;
-  memset(jump_patch, NOP, sizeof(jump_patch));
+  vector<uint8_t> jump_patch;
+  jump_patch.resize(patch_region_size, NOP);
   if (hook_can_ignore_interfering_branches(hook, JumpPatch::size)) {
     // If the preceding instruction is long enough to contain the entire jump,
     // and is a nop, replace the original instruction by a jump back to the
@@ -590,9 +590,9 @@ static bool patch_syscall_with_hook_x86ish(Monkeypatcher& patcher,
     jump_patch[patch_region_size-2] = 0xeb; // jmp rel
     jump_patch[patch_region_size-1] = (int8_t)-patch_region_size;
   }
-  JumpPatch::substitute(jump_patch, jump_offset32);
+  JumpPatch::substitute(jump_patch.data(), jump_offset32);
   bool ok = true;
-  write_and_record_bytes(t, jump_patch_start, sizeof(jump_patch), jump_patch, &ok);
+  write_and_record_bytes(t, jump_patch_start, jump_patch.size(), jump_patch.data(), &ok);
   if (!ok) {
     LOG(warn) << "Couldn't write patch; errno=" << errno;
   }
@@ -981,7 +981,9 @@ static uint64_t jump_patch_size(SupportedArch arch)
 const syscall_patch_hook* Monkeypatcher::find_syscall_hook(RecordTask* t,
                                                            remote_code_ptr ip,
                                                            bool entering_syscall,
-                                                           size_t instruction_length) {
+                                                           size_t instruction_length,
+                                                           bool &should_retry,
+                                                           bool &transient_failure) {
   /* we need to inspect this many bytes before the start of the instruction,
      to find every short jump that might land after it. Conservative. */
   static const intptr_t LOOK_BACK = 0x80;
@@ -1035,9 +1037,9 @@ const syscall_patch_hook* Monkeypatcher::find_syscall_hook(RecordTask* t,
         // A patch that uses bytes before the syscall can't be done when
         // entering the syscall, it must be done when exiting. So set a flag on
         // the Task that tells us to come back later.
-        t->retry_syscall_patching = true;
+        should_retry = true;
         LOG(debug) << "Deferring syscall patching at " << ip << " in " << t
-                   << " until syscall exit.";
+                  << " until syscall exit.";
         return nullptr;
       }
       matches_hook = true;
@@ -1111,6 +1113,7 @@ const syscall_patch_hook* Monkeypatcher::find_syscall_hook(RecordTask* t,
       end_range = ip + instruction_length + hook.patch_region_length;
     }
     if (!safe_for_syscall_patching(start_range, end_range, t)) {
+      transient_failure = true;
       LOG(debug)
           << "Temporarily declining to patch syscall at " << ip
           << " because a different task has its ip in the patched range";
@@ -1141,18 +1144,18 @@ const syscall_patch_hook* Monkeypatcher::find_syscall_hook(RecordTask* t,
 // The `entering_syscall` flag tells us whether or not we're at syscall entry.
 // If we are, and we find a pattern that can only be patched at exit, we'll
 // set a flag on the RecordTask telling it to try again after syscall exit.
-bool Monkeypatcher::try_patch_syscall_x86ish(RecordTask* t, bool entering_syscall,
-                                             SupportedArch arch) {
-  Registers r = t->regs();
-  remote_code_ptr ip = r.ip();
-
+bool Monkeypatcher::try_patch_syscall_x86ish(RecordTask* t, remote_code_ptr ip, bool entering_syscall,
+                                             SupportedArch arch, bool &should_retry) {
   ASSERT(t, is_x86ish(arch)) << "Unsupported architecture";
 
   size_t instruction_length = rr::syscall_instruction_length(arch);
+  bool transient_failure = false;
   const syscall_patch_hook* hook_ptr = find_syscall_hook(t, ip - instruction_length,
-      entering_syscall, instruction_length);
+      entering_syscall, instruction_length, should_retry, transient_failure);
   bool success = false;
-  intptr_t syscallno = r.original_syscallno();
+  // `syscallno` isn't necessarily correct here (in the extremely rare corner case that we
+  // deferred a patch and the signal handler changed it), but we only use it for logging.
+  intptr_t syscallno = t->regs().original_syscallno();
   if (hook_ptr) {
     // Get out of executing the current syscall before we patch it.
     if (entering_syscall && !t->exit_syscall_and_prepare_restart()) {
@@ -1170,7 +1173,7 @@ bool Monkeypatcher::try_patch_syscall_x86ish(RecordTask* t, bool entering_syscal
   }
 
   if (!success) {
-    if (!t->retry_syscall_patching) {
+    if (!should_retry && !transient_failure) {
       LOG(debug) << "Failed to patch syscall at " << ip << " syscall "
                  << syscall_name(syscallno, t->arch()) << " tid " << t->tid;
       tried_to_patch_syscall_addresses.insert(ip);
@@ -1181,15 +1184,12 @@ bool Monkeypatcher::try_patch_syscall_x86ish(RecordTask* t, bool entering_syscal
   return true;
 }
 
-bool Monkeypatcher::try_patch_syscall_aarch64(RecordTask* t, bool entering_syscall) {
-  Registers r = t->regs();
-  remote_code_ptr ip = r.ip() - 4;
-
+bool Monkeypatcher::try_patch_syscall_aarch64(RecordTask* t, remote_code_ptr ip, bool entering_syscall) {
   uint32_t inst[2] = {0, 0};
-  size_t bytes_count = t->read_bytes_fallible(ip.to_data_ptr<uint8_t>() - 4, 8, &inst);
+  size_t bytes_count = t->read_bytes_fallible(ip.to_data_ptr<uint8_t>() - 8, 8, &inst);
   if (bytes_count < sizeof(inst) || inst[1] != 0xd4000001) {
     LOG(debug) << "Declining to patch syscall at "
-               << ip << " for unexpected instruction";
+               << ip - 4 << " for unexpected instruction";
     tried_to_patch_syscall_addresses.insert(ip);
     return false;
   }
@@ -1201,7 +1201,7 @@ bool Monkeypatcher::try_patch_syscall_aarch64(RecordTask* t, bool entering_sysca
     // Our syscall hook cannot do that so this would have to be a raw syscall.
     // We can handle this at runtime but if we know the call is definitely
     // a clone we can avoid patching it here.
-    LOG(debug) << "Declining to patch clone syscall at " << ip;
+    LOG(debug) << "Declining to patch clone syscall at " << ip - 4;
     tried_to_patch_syscall_addresses.insert(ip);
     return false;
   }
@@ -1210,9 +1210,9 @@ bool Monkeypatcher::try_patch_syscall_aarch64(RecordTask* t, bool entering_sysca
              memcmp(syscall_hooks[0].patch_region_bytes, &inst[1], 4) == 0))
     << "Unknown syscall hook";
 
-  if (!safe_for_syscall_patching(ip, ip + 4, t)) {
+  if (!safe_for_syscall_patching(ip - 4, ip, t)) {
     LOG(debug)
-      << "Temporarily declining to patch syscall at " << ip
+      << "Temporarily declining to patch syscall at " << ip - 4
       << " because a different task has its ip in the patched range";
     return false;
   }
@@ -1222,10 +1222,10 @@ bool Monkeypatcher::try_patch_syscall_aarch64(RecordTask* t, bool entering_sysca
     return false;
   }
 
-  LOG(debug) << "Patching syscall at " << ip << " syscall "
-             << syscall_name(r.original_syscallno(), aarch64) << " tid " << t->tid;
+  LOG(debug) << "Patching syscall at " << ip - 4 << " syscall "
+             << syscall_name(t->regs().original_syscallno(), aarch64) << " tid " << t->tid;
 
-  auto success = patch_syscall_with_hook(*this, t, syscall_hooks[0], ip, 4, 0);
+  auto success = patch_syscall_with_hook(*this, t, syscall_hooks[0], ip - 4, 4, 0);
   if (!success && entering_syscall) {
     // Need to reenter the syscall to undo exit_syscall_and_prepare_restart
     if (!t->enter_syscall()) {
@@ -1234,8 +1234,8 @@ bool Monkeypatcher::try_patch_syscall_aarch64(RecordTask* t, bool entering_sysca
   }
 
   if (!success) {
-    LOG(debug) << "Failed to patch syscall at " << ip << " syscall "
-               << syscall_name(r.original_syscallno(), aarch64) << " tid " << t->tid;
+    LOG(debug) << "Failed to patch syscall at " << ip - 4 << " syscall "
+               << syscall_name(t->regs().original_syscallno(), aarch64) << " tid " << t->tid;
     tried_to_patch_syscall_addresses.insert(ip);
     return false;
   }
@@ -1243,7 +1243,14 @@ bool Monkeypatcher::try_patch_syscall_aarch64(RecordTask* t, bool entering_sysca
   return true;
 }
 
-bool Monkeypatcher::try_patch_syscall(RecordTask* t, bool entering_syscall) {
+
+bool Monkeypatcher::try_patch_syscall(RecordTask* t, bool entering_syscall, bool &should_retry) {
+  Registers r = t->regs();
+  remote_code_ptr ip = r.ip();
+  return try_patch_syscall(t, entering_syscall, should_retry, ip);
+}
+
+bool Monkeypatcher::try_patch_syscall(RecordTask* t, bool entering_syscall, bool &should_retry, remote_code_ptr ip) {
   if (syscall_hooks.empty()) {
     // Syscall hooks not set up yet. Don't spew warnings, and don't
     // fill tried_to_patch_syscall_addresses with addresses that we might be
@@ -1261,14 +1268,12 @@ bool Monkeypatcher::try_patch_syscall(RecordTask* t, bool entering_syscall) {
     return false;
   }
 
-  Registers r = t->regs();
-  remote_code_ptr ip = r.ip();
   // We should not get here for untraced syscalls or anything else from the rr page.
   // These should be normally prevented by our seccomp filter
   // and in the case of syscalls interrupted by signals,
   // the check for the syscall restart should prevent us from reaching here.
-  DEBUG_ASSERT(ip.to_data_ptr<void>() < AddressSpace::rr_page_start() ||
-               ip.to_data_ptr<void>() >= AddressSpace::rr_page_end());
+  ASSERT(t, ip.to_data_ptr<void>() < AddressSpace::rr_page_start() ||
+            ip.to_data_ptr<void>() >= AddressSpace::rr_page_end());
   if (tried_to_patch_syscall_addresses.count(ip) || is_jump_stub_instruction(ip, true)) {
     return false;
   }
@@ -1303,13 +1308,13 @@ bool Monkeypatcher::try_patch_syscall(RecordTask* t, bool entering_syscall) {
   }
 
   if (arch == aarch64) {
-    return try_patch_syscall_aarch64(t, entering_syscall);
+    return try_patch_syscall_aarch64(t, ip, entering_syscall);
   }
-  return try_patch_syscall_x86ish(t, entering_syscall, arch);
+  return try_patch_syscall_x86ish(t, ip, entering_syscall, arch, should_retry);
 }
 
 bool Monkeypatcher::try_patch_trapping_instruction(RecordTask* t, size_t instruction_length,
-                                                   bool before_instruction) {
+                                                   bool before_instruction, bool &should_retry) {
   if (syscall_hooks.empty()) {
     // Syscall hooks not set up yet. Don't spew warnings, and don't
     // fill tried_to_patch_syscall_addresses with addresses that we might be
@@ -1332,8 +1337,9 @@ bool Monkeypatcher::try_patch_trapping_instruction(RecordTask* t, size_t instruc
   // event, not a FLUSH_SYSCALLBUF event.
   t->maybe_flush_syscallbuf();
 
+  bool transient_failure = false;
   const syscall_patch_hook* hook_ptr =
-    find_syscall_hook(t, ip_of_instruction, before_instruction, instruction_length);
+    find_syscall_hook(t, ip_of_instruction, before_instruction, instruction_length, should_retry, transient_failure);
   bool success = false;
   if (hook_ptr) {
     LOG(debug) << "Patching trapping instruction at " << ip_of_instruction << " tid " << t->tid;
@@ -1343,7 +1349,7 @@ bool Monkeypatcher::try_patch_trapping_instruction(RecordTask* t, size_t instruc
   }
 
   if (!success) {
-    if (!t->retry_syscall_patching) {
+    if (!should_retry && !transient_failure) {
       LOG(debug) << "Failed to patch trapping instruction at " << ip_of_instruction << " tid " << t->tid;
       tried_to_patch_syscall_addresses.insert(ip_of_instruction + instruction_length);
     }
@@ -1587,6 +1593,80 @@ void Monkeypatcher::unpatch_dl_runtime_resolves(RecordTask* t) {
   saved_dl_runtime_resolve_code.clear();
 }
 
+// https://documentation-service.arm.com/static/67581b3355451e3c38d97c22
+static bool is_aarch64_bti(uint32_t instruction) {
+  if ((instruction >> 12) == 0b11010101000000110010 && (instruction & 0x1f) == 0b11111) {
+    // Hint instruction.
+    uint32_t crm = (instruction >> 8) & ((1 << 4) - 1);
+    uint32_t op2 = (instruction >> 5) & ((1 << 3) - 1);
+    return crm == 0b0100 && (op2 & 1) == 0;
+  }
+  return false;
+}
+
+static bool is_aarch64_adrp(uint32_t instruction, remote_ptr<void> pc, remote_ptr<void>* address) {
+  if ((instruction >> 31) == 1 && ((instruction >> 24) & 0x1f) == 0b10000) {
+    uint64_t base = (pc.as_int() >> 12) << 12;
+    uint64_t imm =  ((instruction >> 29) & 0x3) +
+        (((instruction >> 5) & ((1 << 19) - 1)) << 2);
+    *address = remote_ptr<void>(base + (imm << 12));
+    return true;
+  }
+  return false;
+}
+
+static bool is_aarch64_ldrb(uint32_t instruction, uint32_t* offset) {
+  if ((instruction >> 22) == 0b0011100101) {
+    *offset = (instruction >> 10) & ((1 << 12) - 1);
+    return true;
+  }
+  return false;
+}
+
+/**
+ * Patch the __aarch64_have_lse_atomics variable to ensure that LSE atomics are
+ * always used even if init_lse_atomics
+ */
+void Monkeypatcher::patch_aarch64_have_lse_atomics(RecordTask* t, ElfReader& reader,
+                                                   uintptr_t ldadd4_addr,
+                                                   remote_ptr<void> map_start,
+                                                   size_t map_size,
+                                                   size_t map_offset) {
+  ASSERT(t, t->arch() == aarch64);
+  remote_ptr<void> addr =
+    resolve_address(reader, ldadd4_addr, map_start, map_size, map_offset);
+  if (!addr) {
+    return;
+  }
+
+  bool ok = true;
+  uint8_t instruction_bytes[12];
+  t->read_bytes_helper(addr, sizeof(instruction_bytes), instruction_bytes, &ok);
+  if (!ok) {
+    LOG(warn) << "Can't read ldadd4 instruction bytes at " << addr;
+    return;
+  }
+  uint32_t instructions[3];
+  memcpy(instructions, instruction_bytes, sizeof(instructions));
+  int index = 0;
+  if (is_aarch64_bti(instructions[0])) {
+    ++index;
+  }
+  remote_ptr<void> adrp_address;
+  if (!is_aarch64_adrp(instructions[index], addr + index*4, &adrp_address)) {
+    LOG(warn) << "Instruction 0x" << HEX(instructions[index]) << " is not ADRP";
+    return;
+  }
+  uint32_t ldrb_offset;
+  if (!is_aarch64_ldrb(instructions[index + 1], &ldrb_offset)) {
+    LOG(warn) << "Instruction 0x" << HEX(instructions[index + 1]) << " is not LDRB";
+    return;
+  }
+  remote_ptr<void> have_lse_atomics_addr = adrp_address + ldrb_offset;
+  uint8_t enable = 1;
+  write_and_record_mem(t, have_lse_atomics_addr.cast<uint8_t>(), &enable, 1);
+}
+
 static bool file_may_need_instrumentation(const AddressSpace::Mapping& map) {
   size_t file_part = map.map.fsname().rfind('/');
   if (file_part == string::npos) {
@@ -1603,74 +1683,90 @@ void Monkeypatcher::patch_after_mmap(RecordTask* t, remote_ptr<void> start,
                                      size_t size, size_t offset_bytes,
                                      int child_fd, MmapMode mode) {
   const auto& map = t->vm()->mapping_of(start);
-  if (file_may_need_instrumentation(map) &&
-      (t->arch() == x86 || t->arch() == x86_64)) {
-    ScopedFd open_fd;
-    if (child_fd >= 0) {
-      open_fd = t->open_fd(child_fd, O_RDONLY);
-      ASSERT(t, open_fd.is_open()) << "Failed to open child fd " << child_fd;
-    } else {
-      char buf[100];
-      sprintf(buf, "/proc/%d/map_files/%llx-%llx", t->tid,
-              (long long)start.as_int(), (long long)start.as_int() + size);
-      // Reading these directly requires CAP_SYS_ADMIN, so open the link target
-      // instead.
-      char link[PATH_MAX];
-      int ret = readlink(buf, link, sizeof(link) - 1);
-      if (ret < 0) {
-        return;
-      }
-      link[ret] = 0;
-      open_fd = ScopedFd(link, O_RDONLY);
-      if (!open_fd.is_open()) {
-        return;
-      }
+  if (!file_may_need_instrumentation(map)) {
+    return;
+  }
+  if (t->arch() == aarch64 && mode != MMAP_EXEC) {
+    return;
+  }
+  ScopedFd open_fd;
+  if (child_fd >= 0) {
+    open_fd = t->open_fd(child_fd, O_RDONLY);
+    ASSERT(t, open_fd.is_open()) << "Failed to open child fd " << child_fd;
+  } else {
+    char buf[100];
+    sprintf(buf, "/proc/%d/map_files/%llx-%llx", t->tid,
+            (long long)start.as_int(), (long long)start.as_int() + size);
+    // Reading these directly requires CAP_SYS_ADMIN, so open the link target
+    // instead.
+    char link[PATH_MAX];
+    int ret = readlink(buf, link, sizeof(link) - 1);
+    if (ret < 0) {
+      return;
     }
-    ElfFileReader reader(open_fd, t->arch());
-    // Check for symbols first in the library itself, regardless of whether
-    // there is a debuglink.  For example, on Fedora 26, the .symtab and
-    // .strtab sections are stripped from the debuginfo file for
-    // libpthread.so.
-    SymbolTable syms = reader.read_symbols(".symtab", ".strtab");
-    if (syms.size() == 0) {
-      ScopedFd debug_fd = reader.open_debug_file(map.map.fsname());
-      if (debug_fd.is_open()) {
-        ElfFileReader debug_reader(debug_fd, t->arch());
-        syms = debug_reader.read_symbols(".symtab", ".strtab");
-      }
+    link[ret] = 0;
+    open_fd = ScopedFd(link, O_RDONLY);
+    if (!open_fd.is_open()) {
+      return;
     }
-    for (size_t i = 0; i < syms.size(); ++i) {
-      if (syms.is_name(i, "__elision_aconf")) {
-        static const int zero = 0;
-        // Setting __elision_aconf.retry_try_xbegin to zero means that
-        // pthread rwlocks don't try to use elision at all. See ELIDE_LOCK
-        // in glibc's elide.h.
-        set_and_record_bytes(t, reader, syms.addr(i) + 8, &zero, sizeof(zero),
-                             start, size, offset_bytes);
-      }
-      if (syms.is_name(i, "elision_init")) {
-        // Make elision_init return without doing anything. This means
-        // the __elision_available and __pthread_force_elision flags will
-        // remain zero, disabling elision for mutexes. See glibc's
-        // elision-conf.c.
-        static const uint8_t ret = 0xC3;
-        set_and_record_bytes(t, reader, syms.addr(i), &ret, sizeof(ret), start,
-                             size, offset_bytes);
-      }
-      // The following operations can only be applied once because after the
-      // patch is applied the code no longer matches the expected template.
-      // For replaying a replay to work, we need to only apply these changes
-      // during a real exec, not during the mmap operations performed when rr
-      // replays an exec.
-      if (mode == MMAP_EXEC &&
-          (syms.is_name(i, "_dl_runtime_resolve_fxsave") ||
-           syms.is_name(i, "_dl_runtime_resolve_xsave") ||
-           syms.is_name(i, "_dl_runtime_resolve_xsavec"))) {
-        patch_dl_runtime_resolve(t, reader, syms.addr(i), start, size,
-                                 offset_bytes);
-      }
+  }
+  ElfFileReader reader(open_fd, t->arch());
+  // Check for symbols first in the library itself, regardless of whether
+  // there is a debuglink.  For example, on Fedora 26, the .symtab and
+  // .strtab sections are stripped from the debuginfo file for
+  // libpthread.so.
+  SymbolTable syms = reader.read_symbols(".symtab", ".strtab");
+  if (syms.size() == 0) {
+    ScopedFd debug_fd = reader.open_debug_file(map.map.fsname());
+    if (debug_fd.is_open()) {
+      ElfFileReader debug_reader(debug_fd, t->arch());
+      syms = debug_reader.read_symbols(".symtab", ".strtab");
     }
   }
+  switch (t->arch()) {
+    case x86:
+    case x86_64:
+      for (size_t i = 0; i < syms.size(); ++i) {
+        if (syms.is_name(i, "__elision_aconf")) {
+          static const int zero = 0;
+          // Setting __elision_aconf.retry_try_xbegin to zero means that
+          // pthread rwlocks don't try to use elision at all. See ELIDE_LOCK
+          // in glibc's elide.h.
+          set_and_record_bytes(t, reader, syms.addr(i) + 8, &zero, sizeof(zero),
+                               start, size, offset_bytes);
+        }
+        if (syms.is_name(i, "elision_init")) {
+          // Make elision_init return without doing anything. This means
+          // the __elision_available and __pthread_force_elision flags will
+          // remain zero, disabling elision for mutexes. See glibc's
+          // elision-conf.c.
+          static const uint8_t ret = 0xC3;
+          set_and_record_bytes(t, reader, syms.addr(i), &ret, sizeof(ret), start,
+                               size, offset_bytes);
+        }
+        // The following operations can only be applied once because after the
+        // patch is applied the code no longer matches the expected template.
+        // For replaying a replay to work, we need to only apply these changes
+        // during a real exec, not during the mmap operations performed when rr
+        // replays an exec.
+        if (mode == MMAP_EXEC &&
+            (syms.is_name(i, "_dl_runtime_resolve_fxsave") ||
+             syms.is_name(i, "_dl_runtime_resolve_xsave") ||
+             syms.is_name(i, "_dl_runtime_resolve_xsavec"))) {
+          patch_dl_runtime_resolve(t, reader, syms.addr(i), start, size,
+                                   offset_bytes);
+        }
+      }
+      break;
+    case aarch64:
+      for (size_t i = 0; i < syms.size(); ++i) {
+        if (syms.is_name(i, "__aarch64_ldadd4_relax")) {
+          patch_aarch64_have_lse_atomics(t, reader, syms.addr(i), start, size,
+                                         offset_bytes);
+        }
+      }
+      break;
+  }
 }
 
 } // namespace rr
