@@ -13,6 +13,7 @@
 #include <unistd.h>
 
 #include <algorithm>
+#include <filesystem>
 #include <limits>
 #include <map>
 #include <set>
@@ -51,9 +52,13 @@ protected:
 
 PackCommand PackCommand::singleton(
     "pack",
-    " rr pack [OPTION]... [<trace-dir>]\n"
+    " rr pack [OPTION]... [<trace-dirs>...]\n"
     "  --symlink                  Create symlinks to all mmapped files\n"
     "                             instead of copying them.\n"
+    "  --pack-dir=<path>          Specify a directory in which to pack common files.\n"
+    "                             This helps conserve space when packing multiple traces\n"
+    "                             with common files. Both the trace dir and pack dir\n"
+    "                             (at the same relative path) are required for replay\n"
     "\n"
     "Eliminates duplicate files in the trace directory, and copies files into\n"
     "the trace directory as necessary to ensure that all needed files are in\n"
@@ -67,6 +72,7 @@ struct PackFlags {
   /* If true, insert symlinks into the trace dir which point to the original
    * files, rather than copying the files themselves */
   bool symlink;
+  std::string pack_dir;
 
   PackFlags()
       : symlink(false) {}
@@ -99,6 +105,12 @@ bool operator<(const FsExtentsHash& h1, const FsExtentsHash& h2) {
   return memcmp(h1.bytes, h2.bytes, sizeof(h1)) < 0;
 }
 
+struct PackDir {
+  string dir;
+  map<FileHash, string> mapped_files;
+  PackDir(string dir) : dir(dir) {}
+};
+
 static bool name_comparator(const TraceReader::MappedData& d1,
                             const TraceReader::MappedData d2) {
   return d1.file_name < d2.file_name;
@@ -528,7 +540,8 @@ static map<string, string> compute_canonical_symlink_map(
  * for all files with that hash.
  */
 static map<string, string> compute_canonical_mmapped_files(
-    const string& trace_dir) {
+    const string& trace_dir,
+    PackDir &pack_dir) {
   map<string, FileHash> file_info = gather_file_info(trace_dir);
 
   map<FileHash, string> hash_to_name;
@@ -545,10 +558,24 @@ static map<string, string> compute_canonical_mmapped_files(
 
   int name_index = 0;
   for (auto& p : hash_to_name) {
+    // Check if this in our common pack directory
+    auto it = pack_dir.mapped_files.find(p.first);
+    if (it != pack_dir.mapped_files.end()) {
+      LOG(debug) << "Found in common pack dir";
+      p.second = symlink_into_trace(filesystem::relative(it->second, trace_dir), trace_dir, &name_index);
+      continue;
+    }
+
     // Copy hardlinked files into the trace to avoid the possibility of someone
     // overwriting the original file.
     if (is_hardlink(p.second) || !is_in_trace_dir(p.second, trace_dir)) {
-      p.second = copy_into_trace(p.second, trace_dir, &name_index);
+      if (pack_dir.dir != "") {
+        // If a pack dir is specified, first copy into pack dir, then symlink into trace.
+        auto path = pack_dir.mapped_files[p.first] = copy_into_trace(p.second, pack_dir.dir, &name_index);
+        p.second = symlink_into_trace(filesystem::relative(path, trace_dir), trace_dir, &name_index);
+      } else {
+        p.second = copy_into_trace(p.second, trace_dir, &name_index);
+      }
     }
   }
 
@@ -648,35 +675,38 @@ static void delete_unnecessary_files(const map<string, string>& file_map,
   }
 }
 
-static int pack(const string& trace_dir, const PackFlags& flags) {
-  string dir;
-  {
-    // validate trace and produce default trace directory if trace_dir is empty
-    TraceReader reader(trace_dir);
-    dir = reader.dir();
-  }
+static int pack(const vector<string>& trace_dirs, const PackFlags& flags) {
+  for (const string &trace_dir : trace_dirs) {
+    string dir;
+    {
+      // validate trace and produce default trace directory if trace_dir is empty
+      TraceReader reader(trace_dir);
+      dir = reader.dir();
+    }
 
-  char buf[PATH_MAX];
-  char* ret = realpath(dir.c_str(), buf);
-  if (!ret) {
-    FATAL() << "realpath failed on " << dir;
-  }
-  string abspath(buf);
+    PackDir pack_dir(flags.pack_dir);
+    char buf[PATH_MAX];
+    char* ret = realpath(dir.c_str(), buf);
+    if (!ret) {
+      FATAL() << "realpath failed on " << dir;
+    }
+    string abspath(buf);
 
-  if (flags.symlink) {
-    map<string, string> canonical_symlink_map =
-        compute_canonical_symlink_map(abspath);
-    rewrite_mmaps(canonical_symlink_map, abspath);
-    delete_unnecessary_files(canonical_symlink_map, abspath);
-  } else {
-    map<string, string> canonical_mmapped_files =
-        compute_canonical_mmapped_files(abspath);
-    rewrite_mmaps(canonical_mmapped_files, abspath);
-    delete_unnecessary_files(canonical_mmapped_files, abspath);
-  }
+    if (flags.symlink) {
+      map<string, string> canonical_symlink_map =
+          compute_canonical_symlink_map(abspath);
+      rewrite_mmaps(canonical_symlink_map, abspath);
+      delete_unnecessary_files(canonical_symlink_map, abspath);
+    } else {
+      map<string, string> canonical_mmapped_files =
+          compute_canonical_mmapped_files(abspath, pack_dir);
+      rewrite_mmaps(canonical_mmapped_files, abspath);
+      delete_unnecessary_files(canonical_mmapped_files, abspath);
+    }
 
-  if (!probably_not_interactive(STDOUT_FILENO)) {
-    printf("rr: Packed trace directory `%s'.\n", dir.c_str());
+    if (!probably_not_interactive(STDOUT_FILENO)) {
+      printf("rr: Packed trace directory `%s'.\n", dir.c_str());
+    }
   }
 
   return 0;
@@ -685,6 +715,7 @@ static int pack(const string& trace_dir, const PackFlags& flags) {
 static bool parse_pack_arg(vector<string>& args, PackFlags& flags) {
   static const OptionSpec options[] = {
     { 0, "symlink", NO_PARAMETER },
+    { 1, "pack-dir", HAS_PARAMETER },
   };
   ParsedOption opt;
   auto args_copy = args;
@@ -696,6 +727,9 @@ static bool parse_pack_arg(vector<string>& args, PackFlags& flags) {
     case 0:
       flags.symlink = true;
       break;
+    case 1:
+      flags.pack_dir = opt.value;
+      break;
     default:
       DEBUG_ASSERT(0 && "Unknown pack option");
   }
@@ -705,23 +739,27 @@ static bool parse_pack_arg(vector<string>& args, PackFlags& flags) {
 }
 
 int PackCommand::run(vector<string>& args) {
-  bool found_dir = false;
-  string trace_dir;
   PackFlags flags;
 
   while (parse_pack_arg(args, flags)) {
   }
 
+  vector<string> trace_dirs;
   while (!args.empty()) {
-    if (!found_dir && parse_optional_trace_dir(args, &trace_dir)) {
-      found_dir = true;
-      continue;
+    string trace_dir;
+    if (!parse_optional_trace_dir(args, &trace_dir)) {
+      print_help(stderr);
+      return 1;
     }
-    print_help(stderr);
-    return 1;
+    trace_dirs.push_back(trace_dir);
   }
 
-  return pack(trace_dir, flags);
+  // If no trace dirs were supplied, default to packing "", i.e. the latest trace.
+  if (trace_dirs.empty()) {
+    trace_dirs.push_back("");
+  }
+
+  return pack(trace_dirs, flags);
 }
 
 } // namespace rr
