@@ -16,12 +16,15 @@
 #include <utility>
 #include <vector>
 
+#include "PerfCounterBuffers.h"
 #include "ScopedFd.h"
 #include "Ticks.h"
 
 namespace rr {
 
+class Registers;
 class Task;
+class BpfAccelerator;
 
 enum TicksSemantics {
   TICKS_RETIRED_CONDITIONAL_BRANCHES,
@@ -68,10 +71,9 @@ public:
   struct PTState {
     PTData pt_data;
     ScopedFd pt_perf_event_fd;
-    volatile perf_event_mmap_page* mmap_header;
-    char* mmap_aux_buffer;
+    PerfCounterBuffers perf_buffers;
 
-    PTState() : mmap_header(nullptr), mmap_aux_buffer(nullptr) {}
+    PTState() {}
     ~PTState() { close(); }
 
     void open(pid_t tid);
@@ -175,6 +177,21 @@ public:
    */
   static void start_pt_copy_thread();
 
+  /**
+   * Try to use BPF to accelerate async signal processing
+   */
+#ifdef BPF
+  bool accelerate_async_signal(const Registers& regs);
+  uint64_t bpf_skips() const;
+#else
+  bool accelerate_async_signal(const Registers&) {
+    return false;
+  }
+  uint64_t bpf_skips() const {
+    return 0;
+  }
+#endif
+
 private:
   template <typename Arch> void reset_arch_extras();
 
@@ -212,6 +229,11 @@ private:
   // aarch64 specific counter to detect use of ll/sc instructions
   ScopedFd fd_strex_counter;
 
+  // BPF-enabled hardware breakpoint for fast async signal emulation.
+  ScopedFd fd_async_signal_accelerator;
+
+  std::shared_ptr<BpfAccelerator> bpf;
+
   std::unique_ptr<PTState> pt_state;
 
   TicksSemantics ticks_semantics_;
