@@ -7,9 +7,11 @@
 
 namespace rr {
 
+void force_close_record_session();
+
 class RecordCommand : public Command {
 public:
-  virtual int run(std::vector<std::string>& args);
+  virtual int run(std::vector<std::string>& args) override;
 
   static RecordCommand* get() { return &singleton; }
 
