@@ -11,6 +11,7 @@
 #include <sys/syscall.h>
 
 #include "AutoRemoteSyscalls.h"
+#include "ContextSwitchEvent.h"
 #include "PreserveFileMonitor.h"
 #include "RecordSession.h"
 #include "WaitManager.h"
@@ -199,20 +200,20 @@ RecordTask::RecordTask(RecordSession& session, pid_t _tid, uint32_t serial,
       did_record_robust_futex_changes(false),
       waiting_for_reap(false),
       waiting_for_ptrace_exit(false),
-      retry_syscall_patching(false),
       sent_shutdown_kill(false),
       did_execveat(false),
       tick_request_override((TicksRequest)0),
       schedule_frozen(false) {
   push_event(Event::sentinel());
   if (session.tasks().empty()) {
-    // Initial tracee. It inherited its state from this process, so set it up.
-    // The very first task we fork inherits the signal
-    // dispositions of the current OS process (which should all be
-    // default at this point, but ...).  From there on, new tasks
-    // will transitively inherit from this first task.
+    // Initial tracee.
     auto sh = Sighandlers::create();
     sh->init_from_current_process();
+    // Task::set_up_process resets all non-RT signals to
+    // default handling, so mimic that now.
+    for (int sig = 1; sig <= 31; ++sig) {
+      reset_handler(&sh->get(sig), arch());
+    }
     sighandlers.swap(sh);
     own_namespace_rec_tid = _tid;
   }
@@ -265,6 +266,14 @@ RecordTask::~RecordTask() {
   session().on_destroy_record_task(this);
 }
 
+void RecordTask::record_exit_trace_event(WaitStatus exit_status) {
+  session().trace_writer().write_task_event(
+      TraceTaskEvent::for_exit(tid, exit_status));
+  if (thread_group()->tgid == tid) {
+    thread_group()->exit_status = exit_status;
+  }
+}
+
 void RecordTask::record_exit_event(WriteChildTid write_child_tid) {
   // The kernel explicitly only clears the futex if the address space is shared.
   // If the address space has no other users then the futex will not be cleared
@@ -420,6 +429,12 @@ template <typename Arch> static void do_preload_init_arch(RecordTask* t) {
   auto cpu_binding_ptr = REMOTE_PTR_FIELD(params.globals.rptr(), cpu_binding);
   t->write_mem(cpu_binding_ptr, cpu_binding);
   t->record_local(cpu_binding_ptr, &cpu_binding);
+
+  auto context_switch_event_strategy = ContextSwitchEvent::strategy();
+  auto context_switch_event_strategy_ptr =
+      REMOTE_PTR_FIELD(params.globals.rptr(), context_switch_event_strategy);
+  t->write_mem(context_switch_event_strategy_ptr, context_switch_event_strategy);
+  t->record_local(context_switch_event_strategy_ptr, &context_switch_event_strategy);
 }
 
 void RecordTask::push_syscall_event(int syscallno) {
@@ -519,7 +534,11 @@ template <typename Arch> void RecordTask::init_buffers_arch() {
     desched_fd_child = args.desched_counter_fd;
     // Prevent the child from closing this fd
     fds->add_monitor(this, desched_fd_child, new PreserveFileMonitor());
-    desched_fd = remote.retrieve_fd(desched_fd_child);
+    if (!desched_fd.init(remote.retrieve_fd(desched_fd_child))) {
+      LOG(warn)
+          << "ContextSwitchEvent initialization with strategy "
+             "STRATEGY_RECORD_SWITCH failed. tracee died unexpectedly or killed ??";
+    }
 
     if (trace_writer().supports_file_data_cloning() &&
         session().use_read_cloning()) {
@@ -671,6 +690,9 @@ void RecordTask::will_resume_execution(ResumeRequest, WaitRequest,
       // We're injecting a signal, so make sure that signal is unblocked.
       sigset &= ~signal_bit(sig);
     }
+    LOG(debug) << "Set signal mask for " << tid << " to block all signals (bar "
+               << "SYSCALLBUF_DESCHED_SIGNAL/TIME_SLICE_SIGNAL) while we "
+               << "have a stashed signal";
     set_sigmask(sigset);
   }
 
@@ -709,6 +731,8 @@ void RecordTask::will_resume_execution(ResumeRequest, WaitRequest,
       }
     }
   }
+
+  desched_fd.drain_events();
 }
 
 vector<remote_code_ptr> RecordTask::syscallbuf_syscall_entry_breakpoints() {
@@ -897,9 +921,8 @@ void RecordTask::send_synthetic_SIGCHLD_if_necessary() {
         if (rchild->emulated_SIGCHLD_pending) {
           need_signal = true;
           // check to see if any thread in the ptracer process is in a waitpid
-          // that
-          // could read the status of 'tracee'. If it is, we should wake up that
-          // thread. Otherwise we send SIGCHLD to the ptracer thread.
+          // that could read the status of 'tracee'. If it is, we should wake
+          // up that thread. Otherwise we send SIGCHLD to the ptracer thread.
           for (Task* t : thread_group()->task_set()) {
             auto rt = static_cast<RecordTask*>(t);
             if (rt->is_waiting_for(rchild)) {
@@ -931,13 +954,16 @@ void RecordTask::send_synthetic_SIGCHLD_if_necessary() {
   si.si_value.sival_int = SIGCHLD_SYNTHETIC;
   int ret;
   if (wake_task) {
-    LOG(debug) << "Sending synthetic SIGCHLD to tid " << wake_task->tid;
+    LOG(debug) << "Sending synthetic SIGCHLD to waiting tid " << wake_task->tid;
     // We must use the raw SYS_rt_tgsigqueueinfo syscall here to ensure the
     // signal is sent to the correct thread by tid.
     ret = syscall(SYS_rt_tgsigqueueinfo, wake_task->tgid(), wake_task->tid,
                   SIGCHLD, &si);
     ASSERT(this, ret == 0);
-    if (wake_task->is_sig_blocked(SIGCHLD)) {
+    // `stashed_signals_blocking_more_signals` means that SIGCHLD can be
+    // blocked even when `is_sig_blocked` is false.
+    if (wake_task->is_sig_blocked(SIGCHLD) ||
+        wake_task->stashed_signals_blocking_more_signals) {
       LOG(debug) << "SIGCHLD is blocked, kicking it out of the syscall";
       // Just sending SIGCHLD won't wake it up. Send it a TIME_SLICE_SIGNAL
       // as well to make sure it exits a blocking syscall. We ensure those
@@ -963,55 +989,45 @@ static bool is_synthetic_SIGCHLD(const siginfo_t& si) {
   return si.si_signo == SIGCHLD && si.si_value.sival_int == SIGCHLD_SYNTHETIC;
 }
 
-bool RecordTask::set_siginfo_for_synthetic_SIGCHLD(siginfo_t* si) {
+void RecordTask::set_siginfo_for_synthetic_SIGCHLD(siginfo_t* si) {
   if (!is_synthetic_SIGCHLD(*si)) {
-    return true;
-  }
-
-  if (is_syscall_restart() && EV_SYSCALL_INTERRUPTION == ev().type()) {
-    int syscallno = regs().original_syscallno();
-    SupportedArch syscall_arch = ev().Syscall().arch();
-    if (is_waitpid_syscall(syscallno, syscall_arch) ||
-        is_waitid_syscall(syscallno, syscall_arch) ||
-        is_wait4_syscall(syscallno, syscall_arch)) {
-      // Wait-like syscalls always check for notifications from waited-for processes
-      // before they check for pending signals. So, if the tracee has a pending
-      // notification that also generated a signal, the wait syscall will return
-      // normally rather than returning with ERESTARTSYS etc. (The signal will
-      // be dequeued and any handler run on the return to userspace, however.)
-      // We need to emulate this by deferring our synthetic ptrace signal
-      // until after the wait syscall has returned.
-      LOG(debug) << "Deferring signal because we're in a wait";
-      // Return false to tell the caller to defer the signal and resume
-      // the syscall.
-      return false;
-    }
+    return;
   }
 
+  RecordTask* from_task = nullptr;
   for (RecordTask* tracee : emulated_ptrace_tracees) {
     if (tracee->emulated_ptrace_SIGCHLD_pending) {
-      tracee->emulated_ptrace_SIGCHLD_pending = false;
-      tracee->set_siginfo_for_waited_task<NativeArch>(
-          reinterpret_cast<NativeArch::siginfo_t*>(si));
-      si->si_value.sival_int = 0;
-      return true;
+      from_task = tracee;
+      from_task->emulated_ptrace_SIGCHLD_pending = false;
+      break;
     }
   }
 
-  for (ThreadGroup* child_tg : thread_group()->children()) {
-    for (Task* child : child_tg->task_set()) {
-      auto rchild = static_cast<RecordTask*>(child);
-      if (rchild->emulated_SIGCHLD_pending) {
-        rchild->emulated_SIGCHLD_pending = false;
-        rchild->set_siginfo_for_waited_task<NativeArch>(
-            reinterpret_cast<NativeArch::siginfo_t*>(si));
-        si->si_value.sival_int = 0;
-        return true;
+  if (!from_task) {
+    for (ThreadGroup* child_tg : thread_group()->children()) {
+      for (Task* child : child_tg->task_set()) {
+        auto rchild = static_cast<RecordTask*>(child);
+        if (rchild->emulated_SIGCHLD_pending) {
+          from_task = rchild;
+          from_task->emulated_SIGCHLD_pending = false;
+          break;
+        }
+      }
+      if (from_task) {
+        break;
       }
     }
+
+    if (!from_task) {
+      // Maybe the task died after the synthetic SIGCHLD was sent
+      LOG(warn) << "Can't find traced task that send synthetic SIGCHLD";
+      return;
+    }
   }
 
-  return true;
+  from_task->set_siginfo_for_waited_task<NativeArch>(
+      reinterpret_cast<NativeArch::siginfo_t*>(si));
+  si->si_value.sival_int = 0;
 }
 
 bool RecordTask::is_waiting_for_ptrace(RecordTask* t) {
@@ -1161,7 +1177,7 @@ void RecordTask::emulate_SIGCONT() {
   // All threads in the process are resumed.
   for (Task* t : thread_group()->task_set()) {
     auto rt = static_cast<RecordTask*>(t);
-    LOG(debug) << "setting " << tid << " to NOT_STOPPED due to SIGCONT";
+    LOG(debug) << "setting " << rt->tid << " to NOT_STOPPED due to SIGCONT";
     rt->clear_stashed_group_stop();
     rt->emulated_stop_pending = false;
     rt->emulated_stop_type = NOT_STOPPED;
@@ -1347,10 +1363,6 @@ void RecordTask::set_sigmask(sig_set_t mask) {
       return;
     }
     ASSERT(this, errno == EINVAL);
-  } else {
-    LOG(debug) << "Set signal mask to block all signals (bar "
-               << "SYSCALLBUF_DESCHED_SIGNAL/TIME_SLICE_SIGNAL) while we "
-               << " have a stashed signal";
   }
 }
 
@@ -1622,7 +1634,7 @@ bool RecordTask::is_in_syscallbuf() {
     bool ok = true;
     uint64_t addr;
     if (arch() == aarch64) {
-      addr = regs().xlr();
+      addr = regs().x(30);
     }
     else {
       ASSERT(this, is_x86ish(arch())) << "Unknown architecture";
@@ -1724,7 +1736,8 @@ bool RecordTask::record_remote_by_local_map(remote_ptr<void> addr,
   return false;
 }
 
-void RecordTask::record_remote(remote_ptr<void> addr, ssize_t num_bytes) {
+void RecordTask::record_remote(remote_ptr<void> addr, ssize_t num_bytes,
+                               MemWriteSizeValidation size_validation) {
   ASSERT(this, num_bytes >= 0);
 
   if (!addr) {
@@ -1750,11 +1763,11 @@ void RecordTask::record_remote(remote_ptr<void> addr, ssize_t num_bytes) {
     ASSERT(this, false) << "Should have recorded " << num_bytes << " bytes from "
                         << addr << ", but failed";
   }
-  trace_writer().write_raw(rec_tid, buf.data(), num_bytes, addr);
+  trace_writer().write_raw(rec_tid, buf.data(), num_bytes, addr, size_validation);
 }
 
-void RecordTask::record_remote_writable(remote_ptr<void> addr,
-                                        ssize_t num_bytes) {
+void RecordTask::record_remote_writable(remote_ptr<void> addr, ssize_t num_bytes,
+                                        MemWriteSizeValidation size_validation) {
   ASSERT(this, num_bytes >= 0);
 
   remote_ptr<void> p = addr;
@@ -1777,7 +1790,7 @@ void RecordTask::record_remote_writable(remote_ptr<void> addr,
   }
   num_bytes = min(num_bytes, p - addr);
 
-  record_remote(addr, num_bytes);
+  record_remote(addr, num_bytes, size_validation);
 }
 
 ssize_t RecordTask::record_remote_fallible(remote_ptr<void> addr,
@@ -2098,66 +2111,6 @@ bool RecordTask::is_fatal_signal(int sig,
 
 void RecordTask::record_current_event() { record_event(ev()); }
 
-pid_t RecordTask::find_newborn_thread() {
-  ASSERT(this, session().is_recording());
-  ASSERT(this, ptrace_event() == PTRACE_EVENT_CLONE);
-
-  pid_t hint = get_ptrace_eventmsg_pid();
-  char path[PATH_MAX];
-  if (hint >= 0) {
-    sprintf(path, "/proc/%d/task/%d", tid, hint);
-    struct stat stat_buf;
-    // This should always succeed, but may fail in old kernels due to
-    // a kernel bug. See RecordSession::handle_ptrace_event.
-    if (!session().find_task(hint) && 0 == stat(path, &stat_buf)) {
-      return hint;
-    }
-  }
-
-  sprintf(path, "/proc/%d/task", tid);
-  DIR* dir = opendir(path);
-  ASSERT(this, dir);
-  while (true) {
-    struct dirent* result = readdir(dir);
-    ASSERT(this, result);
-    char* end;
-    pid_t thread_tid = strtol(result->d_name, &end, 10);
-    if (*end == '\0' && !session().find_task(thread_tid)) {
-      closedir(dir);
-      return thread_tid;
-    }
-  }
-}
-
-pid_t RecordTask::find_newborn_process(pid_t child_parent) {
-  ASSERT(this, session().is_recording());
-  ASSERT(this,
-         ptrace_event() == PTRACE_EVENT_CLONE ||
-             ptrace_event() == PTRACE_EVENT_VFORK ||
-             ptrace_event() == PTRACE_EVENT_FORK);
-
-  pid_t hint = get_ptrace_eventmsg_pid();
-  // This should always succeed, but may fail in old kernels due to
-  // a kernel bug. See RecordSession::handle_ptrace_event.
-  if (hint >= 0 && !session().find_task(hint) && get_ppid(hint) == child_parent) {
-    return hint;
-  }
-
-  DIR* dir = opendir("/proc");
-  ASSERT(this, dir);
-  while (true) {
-    struct dirent* result = readdir(dir);
-    ASSERT(this, result);
-    char* end;
-    pid_t proc_tid = strtol(result->d_name, &end, 10);
-    if (*end == '\0' && !session().find_task(proc_tid) &&
-        get_ppid(proc_tid) == child_parent) {
-      closedir(dir);
-      return proc_tid;
-    }
-  }
-}
-
 void RecordTask::set_tid_addr(remote_ptr<int> tid_addr) {
   LOG(debug) << "updating cleartid futex to " << tid_addr;
   tid_futex = tid_addr;
@@ -2190,7 +2143,7 @@ void RecordTask::set_tid_and_update_serial(pid_t tid,
 
 bool RecordTask::may_reap() {
   if (emulated_stop_pending) {
-    LOG(debug) << "Declining to reap " << tid << "; emulated stop pending";
+    LOG(debug) << "Declining to reap " << tid << "; emulated stop pending " << emulated_stop_code;
     // Don't reap until the emulated ptrace stop has been processed.
     return false;
   }
