@@ -3,6 +3,7 @@
 #ifndef RR_RECORD_TASK_H_
 #define RR_RECORD_TASK_H_
 
+#include "ContextSwitchEvent.h"
 #include "Registers.h"
 #include "Task.h"
 #include "TraceFrame.h"
@@ -133,6 +134,8 @@ public:
    * If necessary, signal the ptracer that this task has exited.
    */
   void do_ptrace_exit_stop(WaitStatus exit_status);
+
+  void record_exit_trace_event(WaitStatus exit_status);
   /**
    * Return the exit event.
    * If write_child_tid is set, zero out child_tid now if applicable.
@@ -149,9 +152,8 @@ public:
    * ptraced task has had its SIGCHLD sent.
    * Note that we can't set the correct siginfo when we send the signal, because
    * it requires us to set information only the kernel has permission to set.
-   * Returns false if this signal should be deferred.
    */
-  bool set_siginfo_for_synthetic_SIGCHLD(siginfo_t* si);
+  void set_siginfo_for_synthetic_SIGCHLD(siginfo_t* si);
   /**
    * Sets up |si| as if we're delivering a SIGCHLD/waitid for this waited task.
    */
@@ -400,7 +402,8 @@ public:
   void record_local(remote_ptr<T> addr, const T* buf, size_t count = 1) {
     record_local(addr, sizeof(T) * count, buf);
   }
-  void record_remote(remote_ptr<void> addr, ssize_t num_bytes);
+  void record_remote(remote_ptr<void> addr, ssize_t num_bytes,
+                     MemWriteSizeValidation size_validation = MemWriteSizeValidation::EXACT);
   template <typename T> void record_remote(remote_ptr<T> addr) {
     record_remote(addr, sizeof(T));
   }
@@ -417,7 +420,8 @@ public:
   // Record as much as we can of the bytes in this range. Will record only
   // contiguous mapped-writable data starting at `addr`. rr mappings (e.g. syscallbuf)
   // are treated as non-contiguous with any other mapping.
-  void record_remote_writable(remote_ptr<void> addr, ssize_t num_bytes);
+  void record_remote_writable(remote_ptr<void> addr, ssize_t num_bytes,
+                              MemWriteSizeValidation size_validation = MemWriteSizeValidation::EXACT);
 
   // Simple helper that attempts to use the local mapping to record if one
   // exists
@@ -521,19 +525,6 @@ public:
 
   bool is_fatal_signal(int sig, SignalDeterministic deterministic) const;
 
-  /**
-   * Return the pid of the newborn thread created by this task.
-   * Called when this task has a PTRACE_CLONE_EVENT with CLONE_THREAD.
-   */
-  pid_t find_newborn_thread();
-  /**
-   * Return the pid of the newborn process (whose parent has pid `parent_pid`,
-   * which need not be the same as the current task's pid, due to CLONE_PARENT)
-   * created by this task. Called when this task has a PTRACE_CLONE_EVENT
-   * without CLONE_THREAD, or PTRACE_FORK_EVENT.
-   */
-  pid_t find_newborn_process(pid_t child_parent);
-
   /**
    * If the process looks alive, kill it.
    */
@@ -637,6 +628,10 @@ public:
 
   void set_stopped(bool stopped) override;
 
+  // Tries to extend an adjacent `MAP_GROWSDOWN` mapping to include the
+  // given address. Returns false if nothing was done.
+  bool try_grow_map(remote_ptr<void> addr);
+
 private:
   /* Retrieve the tid of this task from the tracee and store it */
   void update_own_namespace_tid();
@@ -719,6 +714,7 @@ public:
   // tracer attached via PTRACE_SEIZE
   bool emulated_ptrace_seized;
   WaitType in_wait_type;
+  int in_wait_options;
   pid_t in_wait_pid;
 
   // Signal handler state
@@ -746,7 +742,7 @@ public:
   // Syscallbuf state
 
   SyscallbufCodeLayout syscallbuf_code_layout;
-  ScopedFd desched_fd;
+  ContextSwitchEvent desched_fd;
   /* Value of hdr->num_rec_bytes when the buffer was flushed */
   uint32_t flushed_num_rec_bytes;
   /* Nonzero after the trace recorder has flushed the
@@ -791,6 +787,9 @@ public:
   // Stashed signal-delivery state, ready to be delivered at
   // next opportunity.
   std::deque<StashedSignal> stashed_signals;
+  // When true, we're blocking signals during a syscall to
+  // prevent new signals from being delivered. `blocked_sigs_dirty`
+  // is false and `blocked_sigs` contains the previous sigmask.
   bool stashed_signals_blocking_more_signals;
   bool stashed_group_stop;
   bool break_at_syscallbuf_traced_syscalls;
@@ -811,9 +810,6 @@ public:
   // be manually run.
   bool waiting_for_ptrace_exit;
 
-  // When exiting a syscall, we should call MonkeyPatcher::try_patch_syscall again.
-  bool retry_syscall_patching;
-
   // We've sent a SIGKILL during shutdown for this task.
   bool sent_shutdown_kill;
 
