@@ -10,7 +10,7 @@
 #include <string.h>
 #include <sys/user.h>
 
-#include "GdbRegister.h"
+#include "GdbServerRegister.h"
 #include "core.h"
 #include "kernel_abi.h"
 #include "remote_code_ptr.h"
@@ -55,7 +55,7 @@ class Registers {
 public:
   enum { MAX_SIZE = 16 };
 
-  Registers(SupportedArch a = SupportedArch(-1)) : arch_(a) {
+  Registers(SupportedArch a = x86) : arch_(a) {
     memset(&u, 0, sizeof(u));
   }
 
@@ -404,34 +404,14 @@ public:
     u.arm64regs.pstate = pstate;
   }
 
-  void set_x7(uintptr_t x7) {
-    DEBUG_ASSERT(arch() == aarch64);
-    u.arm64regs.x[7] = x7;
+  void set_x(unsigned regno, uintptr_t val) {
+    DEBUG_ASSERT(arch() == aarch64 && regno < 31);
+    u.arm64regs.x[regno] = val;
   }
 
-  void set_x15(uintptr_t x15) {
-    DEBUG_ASSERT(arch() == aarch64);
-    u.arm64regs.x[15] = x15;
-  }
-
-  void set_xlr(uintptr_t xlr) {
-    DEBUG_ASSERT(arch() == aarch64);
-    u.arm64regs.x[30] = xlr;
-  }
-
-  uintptr_t x1() const {
-    DEBUG_ASSERT(arch() == aarch64);
-    return u.arm64regs.x[1];
-  }
-
-  uintptr_t x7() const {
-    DEBUG_ASSERT(arch() == aarch64);
-    return u.arm64regs.x[7];
-  }
-
-  uintptr_t xlr() const {
-    DEBUG_ASSERT(arch() == aarch64);
-    return u.arm64regs.x[30];
+  uintptr_t x(unsigned regno) const {
+    DEBUG_ASSERT(arch() == aarch64 && regno < 31);
+    return u.arm64regs.x[regno];
   }
   // End of aarch64 specific accessors
 
@@ -492,8 +472,8 @@ public:
     return !result.mismatch_count;
   }
 
-  // TODO: refactor me to use the GdbRegisterValue helper from
-  // GdbConnection.h.
+  // TODO: refactor me to use the GdbServerRegisterValue helper from
+  // GdbServerConnection.h.
 
   /**
    * Write the value for register |regno| into |buf|, which should
@@ -501,7 +481,7 @@ public:
    * Return the size of the register in bytes and set |defined| to
    * indicate whether a useful value has been written to |buf|.
    */
-  size_t read_register(uint8_t* buf, GdbRegister regno, bool* defined) const;
+  size_t read_register(uint8_t* buf, GdbServerRegister regno, bool* defined) const;
 
   /**
    * Write the value for register |offset| into |buf|, which should
@@ -517,7 +497,7 @@ public:
    * Update the register named |reg_name| to |value| with
    * |value_size| number of bytes.
    */
-  bool write_register(GdbRegister reg_name, const void* value,
+  bool write_register(GdbServerRegister reg_name, const void* value,
                       size_t value_size);
 
   /**
@@ -576,7 +556,7 @@ private:
   void compare_internal(const Registers& other, Comparison& result) const;
 
   template <typename Arch>
-  size_t read_register_arch(uint8_t* buf, GdbRegister regno,
+  size_t read_register_arch(uint8_t* buf, GdbServerRegister regno,
                             bool* defined) const;
 
   template <typename Arch>
@@ -584,7 +564,7 @@ private:
                                            bool* defined) const;
 
   template <typename Arch>
-  bool write_register_arch(GdbRegister regno, const void* value,
+  bool write_register_arch(GdbServerRegister regno, const void* value,
                            size_t value_size);
 
   template <typename Arch>
