@@ -380,6 +380,8 @@ private:
   ReplaySession(const std::string& dir, const Flags& flags);
   ReplaySession(const ReplaySession& other);
 
+  void check_virtual_address_size() const;
+
   ReplayTask* revive_task_for_exec();
   ReplayTask* setup_replay_one_trace_frame(ReplayTask* t);
   void advance_to_next_trace_frame();
@@ -430,6 +432,7 @@ private:
   siginfo_t last_siginfo_;
   Flags flags_;
   FastForwardStatus fast_forward_status;
+  TaskUid last_task_tuid;
   bool skip_next_execution_event;
   bool replay_stops_at_first_execve_;
   bool detected_transient_error_;
