@@ -60,10 +60,12 @@ public:
   void validate_regs(uint32_t flags = 0);
   const TraceFrame& current_trace_frame();
   FrameTime current_frame_time();
-  /** Restore the next chunk of saved data from the trace to this. */
-  ssize_t set_data_from_trace();
+
+  /** Restore the next chunk of this frame's saved data from the trace to this. */
+  void apply_data_record_from_trace();
   /** Restore all remaining chunks of saved data for the current trace frame. */
   void apply_all_data_records_from_trace();
+
   /**
    * Set the syscall-return-value register of this to what was
    * saved in the current trace frame.
@@ -76,6 +78,9 @@ public:
    */
   void set_real_tid_and_update_serial(pid_t tid);
 
+  /** Return the extra registers of this. Asserts if the task died. */
+  const ExtraRegisters& extra_regs();
+
   void note_sched_in_syscallbuf_syscall_hook() {
     seen_sched_in_syscallbuf_syscall_hook = true;
   }
