@@ -8,6 +8,8 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include "log.h"
+
 namespace rr {
 
 /**
@@ -17,7 +19,7 @@ namespace rr {
 class ScopedFd {
 public:
   ScopedFd() : fd(-1) {}
-  ScopedFd(int fd) : fd(fd) {}
+  explicit ScopedFd(int fd) : fd(fd) {}
   ScopedFd(const char* pathname, int flags, mode_t mode = 0)
       : fd(open(pathname, flags, mode)) {}
   ScopedFd(ScopedFd&& other) : fd(other.fd) { other.fd = -1; }
@@ -41,7 +43,13 @@ public:
   bool is_open() const { return fd >= 0; }
   void close() {
     if (fd >= 0) {
-      ::close(fd);
+      int err = ::close(fd);
+      // With EINTR/EIO, it is unspecified whether fd will be
+      // closed, but on Linux, it is always removed from the FD
+      // table, so the close was successful for our purposes.
+      if (err != 0 && err != -EINTR && err != -EIO) {
+        FATAL() << "Unexpected error while closing fd " << fd;
+      }
     }
     fd = -1;
   }
