@@ -1,8 +1,10 @@
 /* -*- Mode: C++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
 #include <dirent.h>
+#include <spawn.h>
 #include <unistd.h>
 
+#include <algorithm>
 #include <set>
 #include <tuple>
 #include <unordered_map>
@@ -12,7 +14,9 @@
 
 #include "Command.h"
 #include "ElfReader.h"
-#include "RecordSession.h"
+#include "Flags.h"
+#include "ReplaySession.h"
+#include "StringVectorToCharArray.h"
 #include "TraceStream.h"
 #include "core.h"
 #include "cpp_supplement.h"
@@ -66,7 +70,10 @@ SourcesCommand SourcesCommand::singleton(
     "                             in the library's DW_AT_comp_dir property\n"
     "                             for all compilation units.\n"
     "                             LIBRARY is the basename of the original file name,\n"
-    "                             e.g. libc-2.32.so\n");
+    "                             e.g. libc-2.32.so\n"
+    "  --gdb-script=SCRIPT        Runs the provided gdb script in a (very basic)\n"
+    "                             emulator, and gives it a chance to change paths\n"
+    "                             for symbol loading/etc\n");
 
 class ExplicitSourcesCommand : public Command {
 public:
@@ -90,6 +97,11 @@ ExplicitSourcesCommand ExplicitSourcesCommand::singleton(
     "                             LIBRARY is the basename of the original file name,\n"
     "                             e.g. libc-2.32.so\n");
 
+struct SourcesFlags {
+  map<string, string> comp_dir_substitutions;
+  string gdb_script;
+};
+
 static void dir_name(string& s) {
   size_t p = s.rfind('/');
   if (p == string::npos || (p == 0 && s.size() == 1)) {
@@ -101,13 +113,6 @@ static void dir_name(string& s) {
   }
 }
 
-static void base_name(string& s) {
-  size_t p = s.rfind('/');
-  if (p != string::npos) {
-    s.erase(0, p + 1);
-  }
-}
-
 static bool is_absolute(const string& s) {
   return s[0] == '/';
 }
@@ -137,6 +142,186 @@ struct DirExistsCache {
   }
 };
 
+struct DebugDirs {
+  vector<string> debug_file_directories;
+  vector<string> source_directories;
+};
+
+/// Manages integration with rr-gdb-script-host.py to allow a gdb script to
+/// control which directories we search. If input_pipe_fd is open we have a
+/// python child process. If input_pipe_fd is closed then everything here
+/// becomes a no-op.
+class DebugDirManager {
+public:
+  DebugDirManager(const string& program, const string& gdb_script);
+  ~DebugDirManager();
+
+  DebugDirs initial_directories() {
+    return read_result();
+  }
+  DebugDirs process_one_binary(const string& binary_path);
+
+private:
+  DebugDirManager(const DebugDirManager&) = delete;
+  DebugDirManager& operator=(const DebugDirManager&) = delete;
+
+  DebugDirs read_result();
+
+  ScopedFd input_pipe_fd;
+  FILE* output_file;
+  pid_t pid;
+};
+
+DebugDirManager::~DebugDirManager() {
+  if (!input_pipe_fd.is_open()) {
+    return;
+  }
+
+  input_pipe_fd.close();
+  fclose(output_file);
+
+  int status;
+  if (waitpid(pid, &status, 0) == -1) {
+    FATAL() << "Failed to wait on gdb script host";
+  }
+}
+
+DebugDirManager::DebugDirManager(const string& program, const string& gdb_script)
+  : pid(-1)
+{
+  if (gdb_script.empty()) {
+    return;
+  }
+
+  int stdin_pipe_fds[2];
+  if (pipe(stdin_pipe_fds) == -1) {
+    FATAL();
+  }
+  int stdout_pipe_fds[2];
+  if (pipe(stdout_pipe_fds) == -1) {
+    FATAL();
+  }
+
+  posix_spawn_file_actions_t file_actions;
+  int ret = posix_spawn_file_actions_init(&file_actions);
+  if (ret != 0) {
+    FATAL() << "posix_spawn_file_actions_init failed with " << ret;
+  }
+
+  // Close unused write end in the child.
+  ret = posix_spawn_file_actions_addclose(&file_actions, stdin_pipe_fds[1]);
+  if (ret != 0) {
+    FATAL() << "posix_spawn_file_actions_addclose failed with " << ret;
+  }
+
+  // Close unused read end in the child.
+  ret = posix_spawn_file_actions_addclose(&file_actions, stdout_pipe_fds[0]);
+  if (ret != 0) {
+    FATAL() << "posix_spawn_file_actions_addclose failed with " << ret;
+  }
+
+  // Replace child's stdin with the read end.
+  ret = posix_spawn_file_actions_adddup2(&file_actions, stdin_pipe_fds[0], 0);
+  if (ret != 0) {
+    FATAL() << "posix_spawn_file_actions_adddup2 failed with " << ret;
+  }
+
+  // Replace child's stdout with the write end.
+  ret = posix_spawn_file_actions_adddup2(&file_actions, stdout_pipe_fds[1], 1);
+  if (ret != 0) {
+    FATAL() << "posix_spawn_file_actions_adddup2 failed with " << ret;
+  }
+
+  string gdb_script_host_path = resource_path() + "bin/rr-gdb-script-host.py";
+  pid_t pid;
+  vector<string> gdb_script_host_argv_vec = { gdb_script_host_path, gdb_script, program };
+  StringVectorToCharArray gdb_script_host_argv(gdb_script_host_argv_vec);
+  ret = posix_spawn(&pid, gdb_script_host_path.c_str(), &file_actions, nullptr,
+                    gdb_script_host_argv.get(), environ);
+  if (ret != 0) {
+    FATAL() << "posix_spawn failed with " << ret;
+  }
+
+  // Ignore the return values during cleanup.
+  posix_spawn_file_actions_destroy(&file_actions);
+
+  close(stdin_pipe_fds[0]);
+  close(stdout_pipe_fds[1]);
+
+  this->pid = pid;
+  this->input_pipe_fd = ScopedFd(stdin_pipe_fds[1]);
+  this->output_file = fdopen(stdout_pipe_fds[0], "r");
+  if (!this->output_file) {
+    FATAL() << "Failed to fdopen(stdout_pipe_fds[0])";
+  }
+}
+
+DebugDirs DebugDirManager::process_one_binary(const string& binary_path) {
+  if (!input_pipe_fd.is_open()) {
+    return DebugDirs();
+  }
+
+  auto len = binary_path.length();
+  size_t written = write(input_pipe_fd, binary_path.c_str(), len);
+  if (written != len) {
+    FATAL() << "Failed to write filename";
+  }
+  written = write(input_pipe_fd, "\n", 1);
+  if (written != 1) {
+    FATAL() << "Failed to write trailing newline";
+  }
+
+  return read_result();
+}
+
+DebugDirs DebugDirManager::read_result() {
+  char buf[4096];
+  DebugDirs result;
+  size_t index;
+  const char delimiter[2] = ":";
+
+  if (!input_pipe_fd.is_open()) {
+    return result;
+  }
+
+  if (!fgets(buf, sizeof(buf) - 1, output_file)) {
+    FATAL() << "Failed to read gdb script output";
+  }
+  index = strcspn(buf, "\n");
+  buf[index] = 0;
+
+  char* token = strtok(buf, delimiter);
+  while (token != nullptr) {
+    string s(token);
+    s = real_path(s);
+    result.debug_file_directories.push_back(s);
+    LOG(debug) << "gdb script added debug dir '" << s << "'";
+    token = strtok(nullptr, delimiter);
+  }
+
+  if (!fgets(buf, sizeof(buf) - 1, output_file)) {
+    FATAL() << "Failed to read gdb script output";
+  }
+  index = strcspn(buf, "\n");
+  buf[index] = 0;
+
+  token = strtok(buf, delimiter);
+  while (token != nullptr) {
+    char* buf = realpath(token, nullptr);
+    if (buf) {
+      auto s = string(buf);
+      result.source_directories.push_back(s);
+      LOG(debug) << "gdb script added source dir '" << s << "'";
+      free(buf);
+    } else {
+      LOG(debug) << "realpath(" << token << ") = " << strerror(errno);
+    }
+    token = strtok(nullptr, delimiter);
+  }
+
+  return result;
+}
+
 // Resolve a file name relative to a compilation directory and relative directory.
 // file_name cannot be null, but the others can be.
 // Takes into account the original file name as follows:
@@ -215,11 +400,19 @@ struct DwoInfo {
   uint64_t id;
 };
 
+struct OutputCompDirSubstitution {
+  string trace_relative_name;
+  string substitution;
+};
+
 static bool process_compilation_units(ElfFileReader& reader,
                                       ElfFileReader* sup_reader,
                                       const string& trace_relative_name,
                                       const string& original_file_name,
                                       const string& comp_dir_substitution,
+                                      vector<OutputCompDirSubstitution>& comp_dir_substitutions,
+                                      const string* debug_file_directory,
+                                      const string* debug_src_directory,
                                       set<string>* file_names, vector<DwoInfo>* dwos,
                                       DirExistsCache& dir_exists_cache) {
   string build_id = reader.read_buildid();
@@ -271,15 +464,27 @@ static bool process_compilation_units(ElfFileReader& reader,
     } else {
       cu.set_str_offsets_base(0);
     }
-    const char* original_comp_dir = cu.die().string_attr(cu, DW_AT_comp_dir, debug_strs, &ok);;
-    const char* comp_dir;
+    const char* original_comp_dir = cu.die().string_attr(cu, DW_AT_comp_dir, debug_strs, &ok);
+    string comp_dir;
     if (!comp_dir_substitution.empty()) {
-      comp_dir = comp_dir_substitution.c_str();
+      comp_dir = comp_dir_substitution;
     } else {
-      comp_dir = original_comp_dir;
       if (!ok) {
         continue;
       }
+      if (original_comp_dir) {
+        comp_dir = original_comp_dir;
+      }
+      if (debug_src_directory && !is_absolute(comp_dir)) {
+        prepend_path(debug_src_directory->c_str(), comp_dir);
+        if (std::find_if(comp_dir_substitutions.begin(), comp_dir_substitutions.end(), [trace_relative_name](OutputCompDirSubstitution& s) {
+          return s.trace_relative_name == trace_relative_name;
+        }) == comp_dir_substitutions.end()) {
+          comp_dir_substitutions.push_back({ trace_relative_name, comp_dir });
+        }
+      } else if (debug_file_directory) {
+        prepend_path(debug_file_directory->c_str(), comp_dir);
+      }
     }
     const char* dwo_name = cu.die().string_attr(cu, DW_AT_GNU_dwo_name, debug_strs, &ok);
     if (!ok || !dwo_name) {
@@ -297,16 +502,15 @@ static bool process_compilation_units(ElfFileReader& reader,
       if (!has_dwo_id) {
         dwo_id = cu.die().unsigned_attr(DW_AT_GNU_dwo_id, &has_dwo_id, &ok);
         if (!ok) {
+          LOG(warn) << "Have DWO name " << dwo_name << " but can't get DWO id";
           continue;
         }
       }
       if (has_dwo_id) {
         string full_name;
-        if (resolve_file_name(original_file_name.c_str(), comp_dir, original_comp_dir, comp_dir_substitution, nullptr, dwo_name, dir_exists_cache, full_name)) {
-          string c;
-          if (comp_dir) {
-            c = comp_dir;
-          }
+        LOG(debug) << "Have DWO name " << dwo_name << " id " << HEX(dwo_id);
+        if (resolve_file_name(original_file_name.c_str(), comp_dir.c_str(), original_comp_dir, comp_dir_substitution, nullptr, dwo_name, dir_exists_cache, full_name)) {
+          string c = comp_dir;
           dwos->push_back({ dwo_name, trace_relative_name, build_id, std::move(c), full_name, dwo_id });
         } else {
           FATAL() << "DWO missing due to relative path " << full_name;
@@ -321,7 +525,7 @@ static bool process_compilation_units(ElfFileReader& reader,
     }
     if (source_file_name) {
       string full_name;
-      if (resolve_file_name(original_file_name.c_str(), comp_dir, original_comp_dir, comp_dir_substitution, nullptr, source_file_name, dir_exists_cache, full_name)) {
+      if (resolve_file_name(original_file_name.c_str(), comp_dir.c_str(), original_comp_dir, comp_dir_substitution, nullptr, source_file_name, dir_exists_cache, full_name)) {
         file_names->insert(full_name);
       }
     }
@@ -340,7 +544,7 @@ static bool process_compilation_units(ElfFileReader& reader,
       }
       const char* dir = lines.directories()[f.directory_index];
       string full_name;
-      if (resolve_file_name(original_file_name.c_str(), comp_dir, original_comp_dir, comp_dir_substitution, dir, f.file_name, dir_exists_cache, full_name)) {
+      if (resolve_file_name(original_file_name.c_str(), comp_dir.c_str(), original_comp_dir, comp_dir_substitution, dir, f.file_name, dir_exists_cache, full_name)) {
         file_names->insert(full_name);
       }
     }
@@ -373,7 +577,8 @@ struct ExternalDebugInfo {
 static unique_ptr<ElfFileReader>
 find_auxiliary_file(const string& original_file_name,
                     const string& aux_file_name,
-                    string& full_file_name) {
+                    string& full_file_name,
+                    const vector<string>& dirs) {
   if (aux_file_name.empty()) {
     return nullptr;
   }
@@ -432,6 +637,17 @@ find_auxiliary_file(const string& original_file_name,
     }
     LOG(info) << "Can't find external debuginfo file " << full_file_name;
 
+    // Try in an appropriate subdirectory of the provided debug dirs
+    for (auto& d : dirs) {
+      full_file_name = d + original_file_dir + "/" + aux_file_name;
+      normalize_file_name(full_file_name);
+      fd = ScopedFd(full_file_name.c_str(), O_RDONLY);
+      if (fd.is_open()) {
+        goto found;
+      }
+      LOG(info) << "Can't find external debuginfo file " << full_file_name;
+    }
+
     // On Ubuntu 20.04 there's both a /lib/x86_64-linux-gnu/libc-2.31.so and a
     // /usr/lib/x86_64-linux-gnu/libc-2.31.so. They are hardlinked to the same inode,
     // and glibc debuginfo is present in the location corresponding to
@@ -463,7 +679,9 @@ found:
 }
 
 static unique_ptr<ElfFileReader>
-find_auxiliary_file_by_buildid(ElfFileReader& trace_file_reader, string& full_file_name) {
+find_auxiliary_file_by_buildid(ElfFileReader& trace_file_reader,
+                               string& full_file_name,
+                               const vector<string>& dirs) {
   string build_id = trace_file_reader.read_buildid();
   if (build_id.empty()) {
     LOG(warn) << "Main ELF binary has no build ID!";
@@ -474,10 +692,22 @@ find_auxiliary_file_by_buildid(ElfFileReader& trace_file_reader, string& full_fi
     return nullptr;
   }
 
-  string path = "/usr/lib/debug/.build-id/" + build_id.substr(0, 2) + "/" + build_id.substr(2) + ".debug";
+  string filename = build_id.substr(0, 2) + "/" + build_id.substr(2) + ".debug";
+  string path = "/usr/lib/debug/.build-id/" + filename;
   ScopedFd fd(path.c_str(), O_RDONLY);
   if (!fd.is_open()) {
     LOG(info) << "Can't find external debuginfo file " << path;
+    for (auto &d : dirs) {
+      path = d + "/.build-id/" + filename;
+      fd = ScopedFd(path.c_str(), O_RDONLY);
+      if (fd.is_open()) {
+        break;
+      }
+      LOG(info) << "Can't find external debuginfo file " << path;
+    }
+  }
+
+  if (!fd.is_open()) {
     return nullptr;
   }
 
@@ -500,7 +730,10 @@ static bool process_auxiliary_file(ElfFileReader& trace_file_reader,
                                    set<string>* file_names,
                                    const string& full_aux_file_name,
                                    const char* file_type,
-                                   const map<string, string>& comp_dir_substitutions,
+                                   map<string, string> comp_dir_substitutions,
+                                   vector<OutputCompDirSubstitution>& output_comp_dir_substitutions,
+                                   const string* chosen_debug_dir,
+                                   const string* chosen_src_dir,
                                    vector<DwoInfo>* dwos,
                                    set<ExternalDebugInfo>* external_debug_info,
                                    bool already_used_file,
@@ -519,12 +752,16 @@ static bool process_auxiliary_file(ElfFileReader& trace_file_reader,
     LOG(debug) << "\tFound comp_dir substitution " << it->second;
     did_work = process_compilation_units(aux_file_reader, alt_file_reader,
                                          trace_relative_name, original_file_name,
-                                         it->second, file_names, dwos, dir_exists_cache);
+                                         it->second, output_comp_dir_substitutions,
+                                         chosen_debug_dir, chosen_src_dir,
+                                         file_names, dwos, dir_exists_cache);
   } else {
     LOG(debug) << "\tNo comp_dir substitution found";
     did_work = process_compilation_units(aux_file_reader, alt_file_reader,
                                          trace_relative_name, original_file_name,
-                                         {}, file_names, dwos, dir_exists_cache);
+                                         {}, output_comp_dir_substitutions,
+                                         chosen_debug_dir, chosen_src_dir,
+                                         file_names, dwos, dir_exists_cache);
   }
 
   if (!did_work) {
@@ -544,40 +781,73 @@ static bool try_debuglink_file(ElfFileReader& trace_file_reader,
                                const string& trace_relative_name,
                                const string& original_file_name,
                                set<string>* file_names, const string& aux_file_name,
-                               const map<string, string>& comp_dir_substitutions,
+                               map<string, string>& comp_dir_substitutions,
+                               vector<OutputCompDirSubstitution>& output_comp_dir_substitutions,
+                               unique_ptr<DebugDirManager>& debug_dirs,
+                               DebugDirs& dd,
                                vector<DwoInfo>* dwos,
                                set<ExternalDebugInfo>* external_debug_info,
                                DirExistsCache& dir_exists_cache) {
   string full_file_name;
   auto reader = find_auxiliary_file(original_file_name, aux_file_name,
-                                    full_file_name);
+                                    full_file_name, dd.debug_file_directories);
   if (!reader) {
-    reader = find_auxiliary_file_by_buildid(trace_file_reader, full_file_name);
+    reader = find_auxiliary_file_by_buildid(trace_file_reader, full_file_name, dd.debug_file_directories);
     if (!reader) {
+      dd.debug_file_directories.clear();
+      dd.source_directories.clear();
       return false;
     }
   }
 
+  if (debug_dirs) {
+    dd = debug_dirs->process_one_binary(full_file_name);
+  }
+
   /* A debuglink file can have its own debugaltlink */
   string full_altfile_name;
   Debugaltlink debugaltlink = reader->read_debugaltlink();
   auto altlink_reader = find_auxiliary_file(original_file_name, debugaltlink.file_name,
-                                            full_altfile_name);
+                                            full_altfile_name, dd.debug_file_directories);
 
-  bool has_source_files = process_auxiliary_file(trace_file_reader, *reader, altlink_reader.get(),
-                                                 trace_relative_name, original_file_name,
-                                                 file_names, full_file_name, DEBUGLINK,
-                                                 comp_dir_substitutions,
-                                                 dwos, external_debug_info, false, dir_exists_cache);
-
-  if (altlink_reader) {
-    has_source_files |= process_auxiliary_file(trace_file_reader, *altlink_reader, nullptr,
-                                               trace_relative_name, original_file_name,
-                                               file_names, full_altfile_name, DEBUGALTLINK,
-                                               comp_dir_substitutions,
-                                               dwos, external_debug_info, has_source_files, dir_exists_cache);
+  // Notify a gdb script about the main binary before processing CUs.
+  if (debug_dirs) {
+    dd = debug_dirs->process_one_binary(original_file_name);
   }
-  return has_source_files;
+
+  vector<string*> debug_file_directories;
+  debug_file_directories.reserve(dd.debug_file_directories.size() + 1);
+  for (auto& dfd : dd.debug_file_directories) {
+    debug_file_directories.push_back(&dfd);
+  }
+  debug_file_directories.push_back(nullptr);
+
+  for (auto chosen_debug_dir : debug_file_directories) {
+    const string* chosen_src_dir = nullptr;
+    if (!dd.source_directories.empty()) {
+      chosen_src_dir = &dd.source_directories.back();
+    }
+
+    bool has_source_files = process_auxiliary_file(trace_file_reader, *reader, altlink_reader.get(),
+                                                   trace_relative_name, original_file_name,
+                                                   file_names, full_file_name, DEBUGLINK,
+                                                   comp_dir_substitutions, output_comp_dir_substitutions,
+                                                   chosen_debug_dir, chosen_src_dir,
+                                                   dwos, external_debug_info, false, dir_exists_cache);
+
+    if (altlink_reader) {
+      has_source_files |= process_auxiliary_file(trace_file_reader, *altlink_reader, nullptr,
+                                                 trace_relative_name, original_file_name,
+                                                 file_names, full_altfile_name, DEBUGALTLINK,
+                                                 comp_dir_substitutions, output_comp_dir_substitutions,
+                                                 chosen_debug_dir, chosen_src_dir,
+                                                 dwos, external_debug_info, has_source_files, dir_exists_cache);
+    }
+    if (has_source_files) {
+      return true;
+    }
+  }
+  return false;
 }
 
 struct Symlink {
@@ -758,12 +1028,11 @@ static bool starts_with(const string& s, const string& prefix) {
   return strncmp(s.c_str(), prefix.c_str(), prefix.size()) == 0;
 }
 
-struct OutputCompDirSubstitution {
-  string trace_relative_name;
-  string substitution;
-};
-
-static int sources(const map<string, string>& binary_file_names, const map<string, string>& comp_dir_substitutions, bool is_explicit) {
+template<class iterable>
+static int sources(const iterable& binary_file_names,
+                   map<string, string>& comp_dir_substitutions,
+                   unique_ptr<DebugDirManager>& debug_dirs,
+                   bool is_explicit) {
   vector<string> relevant_binary_names;
   // Must be absolute.
   set<string> file_names;
@@ -771,6 +1040,11 @@ static int sources(const map<string, string>& binary_file_names, const map<strin
   vector<DwoInfo> dwos;
   vector<OutputCompDirSubstitution> output_comp_dir_substitutions;
   DirExistsCache dir_exists_cache;
+  DebugDirs dd;
+  if (debug_dirs) {
+    dd = debug_dirs->initial_directories();
+  }
+
   for (auto& pair : binary_file_names) {
     string trace_relative_name = pair.first;
     string original_name = pair.second;
@@ -793,7 +1067,7 @@ static int sources(const map<string, string>& binary_file_names, const map<strin
 
     string full_altfile_name;
     auto altlink_reader = find_auxiliary_file(pair.second, debugaltlink.file_name,
-                                              full_altfile_name);
+                                              full_altfile_name, dd.debug_file_directories);
 
     bool has_source_files;
     auto dwo_count = dwos.size();
@@ -804,12 +1078,16 @@ static int sources(const map<string, string>& binary_file_names, const map<strin
       output_comp_dir_substitutions.push_back({ trace_relative_name, it->second });
       has_source_files = process_compilation_units(reader, altlink_reader.get(),
                                                    trace_relative_name, pair.second,
-                                                   it->second, &file_names, &dwos, dir_exists_cache);
+                                                   it->second, output_comp_dir_substitutions,
+                                                   nullptr, nullptr, &file_names, &dwos,
+                                                   dir_exists_cache);
     } else {
       LOG(debug) << "\tNo comp_dir substitution found";
       has_source_files = process_compilation_units(reader, altlink_reader.get(),
                                                    trace_relative_name, pair.second,
-                                                   {}, &file_names, &dwos, dir_exists_cache);
+                                                   {}, output_comp_dir_substitutions,
+                                                   nullptr, nullptr, &file_names, &dwos,
+                                                   dir_exists_cache);
     }
     /* If the original binary had source files, force the inclusion of any debugaltlink
      * file, even if it does not itself have compilation units (it may have relevant strings)
@@ -819,15 +1097,19 @@ static int sources(const map<string, string>& binary_file_names, const map<strin
     Debuglink debuglink = reader.read_debuglink();
     has_source_files |= try_debuglink_file(reader, trace_relative_name, pair.second,
                                            &file_names, debuglink.file_name,
-                                           comp_dir_substitutions, &dwos,
+                                           comp_dir_substitutions, output_comp_dir_substitutions, debug_dirs, dd, &dwos,
                                            &external_debug_info, dir_exists_cache);
 
+    if (dd.debug_file_directories.empty() && debug_dirs) {
+      dd = debug_dirs->process_one_binary(pair.first);
+    }
+
     if (altlink_reader) {
       has_source_files |= process_auxiliary_file(reader, *altlink_reader, nullptr,
                                                  trace_relative_name, pair.second,
                                                  &file_names, full_altfile_name,
-                                                 DEBUGALTLINK, comp_dir_substitutions,
-                                                 &dwos, &external_debug_info,
+                                                 DEBUGALTLINK, comp_dir_substitutions, output_comp_dir_substitutions,
+                                                 nullptr, nullptr, &dwos, &external_debug_info,
                                                  original_had_source_files, dir_exists_cache);
     }
 
@@ -969,13 +1251,14 @@ static int sources(const map<string, string>& binary_file_names, const map<strin
   return 0;
 }
 
-static bool parse_sources_option(vector<string>& args, map<string, string>& comp_dir_substitutions) {
+static bool parse_sources_option(vector<string>& args, SourcesFlags& flags) {
   if (parse_global_option(args)) {
     return true;
   }
 
   static const OptionSpec options[] = {
-    { 0, "substitute", HAS_PARAMETER }
+    { 0, "substitute", HAS_PARAMETER },
+    { 1, "gdb-script", HAS_PARAMETER }
   };
 
   ParsedOption opt;
@@ -989,18 +1272,26 @@ static bool parse_sources_option(vector<string>& args, map<string, string>& comp
       if (pos != string::npos) {
         auto k = opt.value.substr(0, pos);
         auto v = opt.value.substr(pos+1);
-        comp_dir_substitutions.insert(std::pair<string, string>(k, v));
+        flags.comp_dir_substitutions.insert(std::pair<string, string>(k, v));
       }
       break;
     }
+    case 1: {
+      flags.gdb_script = opt.value;
+      break;
+    }
   }
 
   return true;
 }
 
 int SourcesCommand::run(vector<string>& args) {
-  map<string, string> comp_dir_substitutions;
-  while (parse_sources_option(args, comp_dir_substitutions)) {
+  // Various "cannot replay safely..." warnings cannot affect us since
+  // we only replay to the first execve.
+  Flags::get_for_init().suppress_environment_warnings = true;
+
+  SourcesFlags flags;
+  while (parse_sources_option(args, flags)) {
   }
 
   // (Trace file name, original file name) pairs
@@ -1031,12 +1322,46 @@ int SourcesCommand::run(vector<string>& args) {
     }
   }
 
-  return sources(binary_file_names, comp_dir_substitutions, false);
+  string program;
+  {
+    ReplaySession::Flags flags;
+    flags.redirect_stdio = false;
+    flags.share_private_mappings = false;
+    flags.replay_stops_at_first_execve = true;
+    flags.cpu_unbound = true;
+
+    ReplaySession::shr_ptr replay_session = ReplaySession::create(trace_dir, flags);
+    while (true) {
+      auto result = replay_session->replay_step(RUN_CONTINUE);
+      if (replay_session->done_initial_exec()) {
+        program = replay_session->vms()[0]->exe_image();
+        break;
+      }
+
+      if (result.status == REPLAY_EXITED) {
+        break;
+      }
+    }
+  }
+
+  std::vector<std::pair<string, string>> binary_file_names_ordered;
+  auto i = binary_file_names.find(program);
+  if (i != binary_file_names.end()) {
+    binary_file_names_ordered.push_back(*i);
+    binary_file_names.erase(program);
+  }
+  std::copy(binary_file_names.begin(), binary_file_names.end(), std::back_inserter(binary_file_names_ordered));
+  auto debug_dirs = make_unique<DebugDirManager>(program, flags.gdb_script);
+  return sources(binary_file_names_ordered, flags.comp_dir_substitutions, debug_dirs, false);
 }
 
 int ExplicitSourcesCommand::run(vector<string>& args) {
-  map<string, string> comp_dir_substitutions;
-  while (parse_sources_option(args, comp_dir_substitutions)) {
+  // Various "cannot replay safely..." warnings cannot affect us since
+  // we only replay to the first execve.
+  Flags::get_for_init().suppress_environment_warnings = true;
+
+  SourcesFlags flags;
+  while (parse_sources_option(args, flags)) {
   }
 
   // (Trace file name, original file name) pairs
@@ -1066,7 +1391,8 @@ int ExplicitSourcesCommand::run(vector<string>& args) {
     binary_file_names.insert(make_pair(std::move(buildid), arg));
   }
 
-  return sources(binary_file_names, comp_dir_substitutions, true);
+  unique_ptr<DebugDirManager> debug_dirs;
+  return sources(binary_file_names, flags.comp_dir_substitutions, debug_dirs, true);
 }
 
 } // namespace rr
