@@ -528,6 +528,16 @@ public:
    */
   void set_x86_debug_status(uintptr_t status);
 
+  /**
+   * Read the (architecture-specific) pointer authentication keys of the current task
+   */
+  std::vector<uint8_t> pac_keys(bool *ok = nullptr);
+
+  /**
+   * Set the (architecture-specific) pointer authentication keys for the current task
+   */
+  bool set_pac_keys(const std::vector<uint8_t> &data);
+
   /**
    * Determine why a SIGTRAP occurred. On x86, uses x86_debug_status() but doesn't
    * consume it.
@@ -806,7 +816,7 @@ public:
   /**
    * |flags| is bits from WriteFlags.
    */
-  void write_bytes_helper(remote_ptr<void> addr, ssize_t buf_size,
+  ssize_t write_bytes_helper(remote_ptr<void> addr, ssize_t buf_size,
                           const void* buf, bool* ok = nullptr,
                           uint32_t flags = 0);
   /**
@@ -817,6 +827,11 @@ public:
                                               const void* buf, bool* ok = nullptr,
                                               uint32_t flags = 0);
 
+  /**
+   * This task has been selected to run next.
+   */
+  void will_schedule();
+
   SupportedArch detect_syscall_arch();
 
   /**
@@ -1127,6 +1142,13 @@ public:
   // (used to avoid double recording on unexpected exit)
   bool last_syscall_entry_recorded;
 
+  /*
+   * Called before the scheduler resumes a task to check if the task's address
+   * space has any leftover syscallbufs from dead processes which shared the
+   * address space
+   */
+  void unmap_dead_syscallbufs_if_required();
+
 protected:
   Task(Session& session, pid_t tid, pid_t rec_tid, uint32_t serial,
        SupportedArch a);
@@ -1295,7 +1317,7 @@ protected:
   // Task.cc
   uint64_t last_resume_orig_cx;
   // The instruction type we're singlestepping through.
-  TrappedInstruction singlestepping_instruction;
+  SpecialInst singlestepping_instruction;
   // True if we set a breakpoint after a singlestepped CPUID instruction.
   // We need this in addition to `singlestepping_instruction` because that
   // might be CPUID but we failed to set the breakpoint.
