@@ -422,6 +422,7 @@ void TraceWriter::write_frame(RecordTask* t, const Event& ev,
     w.setTid(r.rec_tid);
     w.setAddr(r.addr.as_int());
     w.setSize(r.size);
+    w.setSizeIsConservative(r.size_validation == MemWriteSizeValidation::CONSERVATIVE);
     auto holes = w.initHoles(r.holes.size());
     for (size_t j = 0; j < r.holes.size(); ++j) {
       holes[j].setOffset(r.holes[j].offset);
@@ -573,7 +574,8 @@ TraceFrame TraceReader::read_frame(FrameTime skip_before) {
       const auto& hole = holes[j];
       h[j] = { hole.getOffset(), hole.getSize() };
     }
-    raw_recs[i] = { w.getAddr(), (size_t)w.getSize(), i32_to_tid(w.getTid()), h };
+    raw_recs[i] = { w.getAddr(), (size_t)w.getSize(), i32_to_tid(w.getTid()), h,
+                    w.getSizeIsConservative() ? MemWriteSizeValidation::CONSERVATIVE : MemWriteSizeValidation::EXACT };
   }
 
   if (ret.global_time < skip_before) {
@@ -778,6 +780,9 @@ void TraceWriter::write_task_event(const TraceTaskEvent& event) {
       exec.setExeBase(event.exe_base().as_int());
       exec.setInterpBase(event.interp_base().as_int());
       exec.setInterpName(str_to_data(event.interp_name()));;
+      auto pac_data = exec.initPacData();
+      std::vector<uint8_t> pac_data_vec = event.pac_data();
+      pac_data.setRaw(Data::Reader(pac_data_vec.data(), pac_data_vec.size()));
       break;
     }
     case TraceTaskEvent::EXIT:
@@ -840,6 +845,8 @@ TraceTaskEvent TraceReader::read_task_event(FrameTime* time) {
       r.exe_base_ = exec.getExeBase();
       r.interp_base_ = exec.getInterpBase();
       r.interp_name_ = data_to_str(exec.getInterpName());
+      auto pac_data = exec.getPacData().getRaw();
+      r.pac_data_ = std::vector<uint8_t>(pac_data.begin(), pac_data.end());
       break;
     }
     case trace::TaskEvent::Which::EXIT:
@@ -1232,8 +1239,9 @@ KernelMapping TraceReader::read_mapped_region(MappedData* data, bool* found,
 
 void TraceWriter::write_raw_header(pid_t rec_tid, size_t total_len,
                                    remote_ptr<void> addr,
-                                   const std::vector<WriteHole>& holes = std::vector<WriteHole>()) {
-  raw_recs.push_back({ addr, total_len, rec_tid, holes });
+                                   const std::vector<WriteHole>& holes,
+                                   MemWriteSizeValidation size_validation) {
+  raw_recs.push_back({ addr, total_len, rec_tid, holes, size_validation });
 }
 
 void TraceWriter::write_raw_data(const void* d, size_t len) {
@@ -1248,6 +1256,7 @@ bool TraceReader::read_raw_data_for_frame(RawData& d) {
   auto& rec = raw_recs[raw_recs.size() - 1];
   d.rec_tid = rec.rec_tid;
   d.addr = rec.addr;
+  d.size_validation = rec.size_validation;
 
   d.data.resize(rec.size);
   auto hole_iter = rec.holes.begin();
@@ -1280,6 +1289,7 @@ bool TraceReader::read_raw_data_for_frame_with_holes(RawDataWithHoles& d) {
   d.addr = rec.addr;
   d.holes = std::move(rec.holes);
   size_t data_size = rec.size;
+  d.size_validation = rec.size_validation;
   for (auto& h : d.holes) {
     data_size -= h.size;
   }
@@ -1356,13 +1366,14 @@ TraceWriter::TraceWriter(const std::string& file_name,
                   1),
       ticks_semantics_(ticks_semantics_),
       mmap_count(0),
+      max_virtual_address_size(0),
       has_cpuid_faulting_(false),
       xsave_fip_fdp_quirk_(false),
       fdp_exception_only_quirk_(false),
       clear_fip_fdp_(false),
       supports_file_data_cloning_(false),
       chaos_mode(false)
-       {
+{
   this->ticks_semantics_ = ticks_semantics_;
 
   for (Substream s = SUBSTREAM_FIRST; s < SUBSTREAM_COUNT; ++s) {
@@ -1480,6 +1491,7 @@ void TraceWriter::close(CloseStatus status, const TraceUuid* uuid) {
   header.setExclusionRangeEnd(exclusion_range.end().as_int());
   header.setRuntimePageSize(page_size());
   header.setPreloadLibraryPageSize(PRELOAD_LIBRARY_PAGE_SIZE);
+  header.setMaxVirtualAddressSize(max_virtual_address_size);
 
   {
     struct utsname uname_buf;
@@ -1641,6 +1653,7 @@ TraceReader::TraceReader(const string& dir)
   preload_thread_locals_recorded_ = header.getPreloadThreadLocalsRecorded();
   ticks_semantics_ = from_trace_ticks_semantics(header.getTicksSemantics());
   rrcall_base_ = header.getRrcallBase();
+  max_virtual_address_size_ = header.getMaxVirtualAddressSize();
   syscallbuf_fds_disabled_size_ = header.getSyscallbufFdsDisabledSize();
   syscallbuf_hdr_size_ = header.getSyscallbufHdrSize();
   required_forward_compatibility_version_ = header.getRequiredForwardCompatibilityVersion();
@@ -1735,6 +1748,7 @@ TraceReader::TraceReader(const TraceReader& other)
   xcr0_ = other.xcr0_;
   preload_thread_locals_recorded_ = other.preload_thread_locals_recorded_;
   rrcall_base_ = other.rrcall_base_;
+  max_virtual_address_size_ = other.max_virtual_address_size_;
   arch_ = other.arch_;
   chaos_mode_ = other.chaos_mode_;
   chaos_mode_known_ = other.chaos_mode_known_;
