@@ -15,6 +15,25 @@ using namespace std;
 
 namespace rr {
 
+WaitStatus::WaitStatus(const siginfo_t &info) : status(0)
+{
+  if (info.si_code == CLD_EXITED) {
+    status = (info.si_status & 0x7f) << 8;
+    return;
+  }
+
+  if (info.si_code == CLD_KILLED || info.si_code == CLD_DUMPED) {
+    status = (info.si_status & 0x7f);
+    if (info.si_code == CLD_DUMPED) {
+      status |= 0x80;
+    }
+    return;
+  }
+
+  DEBUG_ASSERT(info.si_code == CLD_STOPPED || info.si_code == CLD_TRAPPED);
+  status = info.si_status << 8 | 0x7f;
+}
+
 WaitStatus::Type WaitStatus::type() const {
   if (exit_code() >= 0) {
     return EXIT;
@@ -46,11 +65,26 @@ int WaitStatus::fatal_sig() const {
   return WIFSIGNALED(status) ? WTERMSIG(status) : 0;
 }
 
+bool WaitStatus::stopped() const {
+  // N.B.: infop distinguishes between ptrace and child stops,
+  // but regular waitpid does not.
+  // (info.si_code == CLD_TRAPPED || info.si_code == CLD_TRAPPED)
+  return WIFSTOPPED(status);
+}
+
+int WaitStatus::ptrace_event_code() const {
+  return (status >> 16) & 0xff;
+}
+
+int WaitStatus::stop_sig_code() const {
+  return WSTOPSIG(status);
+}
+
 int WaitStatus::stop_sig() const {
-  if (!WIFSTOPPED(status) || ((status >> 16) & 0xff)) {
+  if (!stopped() || ptrace_event_code()) {
     return 0;
   }
-  int sig = WSTOPSIG(status);
+  int sig = stop_sig_code();
   if (sig == (SIGTRAP | 0x80)) {
     return 0;
   }
@@ -59,28 +93,30 @@ int WaitStatus::stop_sig() const {
 }
 
 int WaitStatus::group_stop() const {
-  if (!WIFSTOPPED(status) || ((status >> 16) & 0xff) != PTRACE_EVENT_STOP) {
+  if (!stopped() || ptrace_event_code() != PTRACE_EVENT_STOP) {
     return 0;
   }
-  int sig = WSTOPSIG(status);
+  int sig = stop_sig_code();
   sig &= ~0x80;
   return sig ? sig : SIGSTOP;
 }
 
 bool WaitStatus::is_syscall() const {
-  if (!WIFSTOPPED(status) || ptrace_event()) {
+  if (!stopped() || ptrace_event_code()) {
     return 0;
   }
-  return WSTOPSIG(status) == (SIGTRAP | 0x80);
+  return stop_sig_code() == (SIGTRAP | 0x80);
 }
 
 int WaitStatus::ptrace_event() const {
-  int event = (status >> 16) & 0xff;
+  if (!stopped())
+    return 0;
+  int event = ptrace_event_code();
   return event == PTRACE_EVENT_STOP ? 0 : event;
 }
 
 int WaitStatus::ptrace_signal() const {
-  return WIFSTOPPED(status) ? (WSTOPSIG(status) & 0x7f) : 0;
+  return stopped() ? (stop_sig_code() & 0x7f) : 0;
 }
 
 WaitStatus WaitStatus::for_exit_code(int code) {
@@ -120,6 +156,47 @@ WaitStatus WaitStatus::for_ptrace_event(int ptrace_event) {
   return WaitStatus((ptrace_event << 16) | (SIGTRAP << 8) | 0x7f);
 }
 
+template <typename Arch>
+void WaitStatus::fill_siginfo(typename Arch::siginfo_t *si, bool ptracer, unsigned ptrace_options)
+{
+  if (exit_code() >= 0) {
+    si->si_code = CLD_EXITED;
+    si->_sifields._sigchld.si_status_ = exit_code();
+    return;
+  }
+
+  if (fatal_sig()) {
+    // `waitpid`'s status can't distinguish between CLD_KILLED and CLD_DUMPED.
+    // Always use CLD_KILLED for now.
+    si->si_code = is_coredumping_signal(fatal_sig()) ? CLD_DUMPED : CLD_KILLED;
+    si->_sifields._sigchld.si_status_ = fatal_sig();
+    return;
+  }
+
+  if (!ptracer) {
+    DEBUG_ASSERT(!ptrace_event());
+    si->si_code = CLD_STOPPED;
+    si->_sifields._sigchld.si_status_ = stop_sig();
+    return;
+  }
+
+  si->si_code = CLD_TRAPPED;
+  if (is_syscall()) {
+    if (ptrace_options & PTRACE_O_TRACESYSGOOD) {
+      si->_sifields._sigchld.si_status_ = 0x80 | SIGTRAP;
+    } else {
+      si->_sifields._sigchld.si_status_ = SIGTRAP;
+    }
+    return;
+  }
+
+  si->_sifields._sigchld.si_status_ = status >> 8;
+}
+
+template void WaitStatus::fill_siginfo<X86Arch>(X86Arch::siginfo_t*, bool, unsigned);
+template void WaitStatus::fill_siginfo<X64Arch>(X64Arch::siginfo_t*, bool, unsigned);
+template void WaitStatus::fill_siginfo<ARM64Arch>(ARM64Arch::siginfo_t*, bool, unsigned);
+
 ostream& operator<<(ostream& stream, WaitStatus status) {
   stream << HEX(status.get());
   switch (status.type()) {
