@@ -7,12 +7,13 @@
 
 int main(__attribute__((unused)) int argc, char** argv) {
   int cores = atoi(argv[1]);
+  int sysconf_onln = sysconf(_SC_NPROCESSORS_ONLN);
 
-  if (sysconf(_SC_NPROCESSORS_ONLN) == cores &&
-      sysconf(_SC_NPROCESSORS_CONF) == cores) {
+  if (sysconf_onln == cores) {
     caught_test_failure("got core count: %d", cores);
   }
 
+  atomic_printf("Found %d cores online\n", sysconf_onln);
   atomic_puts("EXIT-SUCCESS");
   return 0;
 }
