@@ -56,6 +56,7 @@ enum CpuMicroarch {
   IntelSkylake,
   IntelSilvermont,
   IntelGoldmont,
+  IntelTremont,
   IntelKabylake,
   IntelCometlake,
   IntelIcelake,
@@ -64,9 +65,12 @@ enum CpuMicroarch {
   IntelAlderlake,
   IntelRaptorlake,
   IntelSapphireRapid,
-  LastIntel = IntelSapphireRapid,
+  IntelEmeraldRapid,
+  IntelMeteorLake,
+  IntelArrowLake,
+  LastIntel = IntelArrowLake,
   FirstAMD,
-  AMDF15R30 = FirstAMD,
+  AMDF15 = FirstAMD,
   AMDZen,
   LastAMD = AMDZen,
   FirstARM,
@@ -135,7 +139,10 @@ struct PmuConfig {
 // See Intel 64 and IA32 Architectures Performance Monitoring Events.
 // See check_events from libpfm4.
 static const PmuConfig pmu_configs[] = {
+  { IntelEmeraldRapid, "Intel EmeraldRapid", 0x5111c4, 0, 0, 125, PMU_TICKS_RCB },
   { IntelSapphireRapid, "Intel SapphireRapid", 0x5111c4, 0, 0, 125, PMU_TICKS_RCB },
+  { IntelArrowLake, "Intel Arrowlake", 0x5111c4, 0, 0, 125, PMU_TICKS_RCB },
+  { IntelMeteorLake, "Intel Meteorlake", 0x5111c4, 0, 0, 125, PMU_TICKS_RCB },
   { IntelRaptorlake, "Intel Raptorlake", 0x5111c4, 0, 0, 125, PMU_TICKS_RCB },
   { IntelAlderlake, "Intel Alderlake", 0x5111c4, 0, 0, 125, PMU_TICKS_RCB },
   { IntelRocketlake, "Intel Rocketlake", 0x5111c4, 0, 0, 100, PMU_TICKS_RCB },
@@ -145,6 +152,7 @@ static const PmuConfig pmu_configs[] = {
   { IntelKabylake, "Intel Kabylake", 0x5101c4, 0, 0, 100, PMU_TICKS_RCB },
   { IntelSilvermont, "Intel Silvermont", 0x517ec4, 0, 0, 100, PMU_TICKS_RCB },
   { IntelGoldmont, "Intel Goldmont", 0x517ec4, 0, 0, 100, PMU_TICKS_RCB },
+  { IntelTremont, "Intel Tremont", 0x517ec4, 0, 0, 100, PMU_TICKS_RCB },
   { IntelSkylake, "Intel Skylake", 0x5101c4, 0, 0, 100, PMU_TICKS_RCB },
   { IntelBroadwell, "Intel Broadwell", 0x5101c4, 0, 0, 100, PMU_TICKS_RCB },
   { IntelHaswell, "Intel Haswell", 0x5101c4, 0, 0, 100, PMU_TICKS_RCB },
@@ -154,7 +162,7 @@ static const PmuConfig pmu_configs[] = {
   { IntelWestmere, "Intel Westmere", 0x5101c4, 0, 0, 100, PMU_TICKS_RCB },
   { IntelPenryn, "Intel Penryn", 0, 0, 0, 100, 0 },
   { IntelMerom, "Intel Merom", 0, 0, 0, 100, 0 },
-  { AMDF15R30, "AMD Family 15h Revision 30h", 0xc4, 0xc6, 0, 250, PMU_TICKS_TAKEN_BRANCHES },
+  { AMDF15, "AMD Family 15h", 0xc4, 0xc6, 0, 250, PMU_TICKS_TAKEN_BRANCHES },
   // 0xd1 == RETIRED_CONDITIONAL_BRANCH_INSTRUCTIONS - Number of retired conditional branch instructions
   // 0x2c == INTERRUPT_TAKEN - Counts the number of interrupts taken
   // Both counters are available on Zen, Zen+ and Zen2.
@@ -333,8 +341,15 @@ static CpuMicroarch compute_cpu_microarch(void) {
       return IntelRaptorlake;
     case 0x806f0:
       return IntelSapphireRapid;
+    case 0xc06f0:
+      return IntelEmeraldRapid;
+    case 0xa06a0:
+      return IntelMeteorLake;
+    case 0xc0660:
+      return IntelArrowLake;
+    case 0xf20:
     case 0x30f00:
-      return AMDF15R30;
+      return AMDF15;
     case 0x00f10: // Naples, Whitehaven, Summit Ridge, Snowy Owl (Zen), Milan (Zen 3) (UNTESTED)
     case 0x10f10: // Raven Ridge, Great Horned Owl (Zen) (UNTESTED)
     case 0x10f80: // Banded Kestrel (Zen), Picasso (Zen+) (UNTESTED)
@@ -348,13 +363,15 @@ static CpuMicroarch compute_cpu_microarch(void) {
       if (ext_family == 8 || ext_family == 0xa) {
         return AMDZen;
       } else if (ext_family == 3) {
-        return AMDF15R30;
+        return AMDF15;
       }
       break;
     case 0x20f10: // Vermeer (Zen 3)
     case 0x50f00: // Cezanne (Zen 3)
     case 0x40f40: // Rembrandt (Zen 3+)
     case 0x60f10: // Raphael (Zen 4)
+    case 0x70f40: // Phoenix (Zen 4)
+    case 0x70f50: // Hawk Point (Zen 4)
       if (ext_family == 0xa) {
         return AMDZen;
       }
@@ -819,8 +836,8 @@ int main(int argc, char** argv) {
   }
   CHECK(pmu);
 
-  do_test_ticks_basic = pmu->flags & PMU_TICKS_RCB != 0;
-  do_test_ticks_syscalls = pmu->flags & PMU_TICKS_RCB != 0;
+  do_test_ticks_basic = (pmu->flags & PMU_TICKS_RCB) != 0;
+  do_test_ticks_syscalls = (pmu->flags & PMU_TICKS_RCB) != 0;
 
   CHECK(0 == pipe(parent_to_child_fds));
   CHECK(0 == pipe(child_to_parent_fds));
