@@ -78,15 +78,18 @@ static void restore_signal_state(RecordTask* t, int sig,
 static bool try_handle_trapped_instruction(RecordTask* t, siginfo_t* si) {
   ASSERT(t, si->si_signo == SIGSEGV);
 
-  auto trapped_instruction = trapped_instruction_at(t, t->ip());
-  switch (trapped_instruction) {
-    case TrappedInstruction::RDTSC:
-    case TrappedInstruction::RDTSCP:
+  auto special_instruction = special_instruction_at(t, t->ip());
+  switch (special_instruction.opcode) {
+    case SpecialInstOpcode::ARM_MRS_CNTFRQ_EL0:
+    case SpecialInstOpcode::ARM_MRS_CNTVCT_EL0:
+    case SpecialInstOpcode::ARM_MRS_CNTVCTSS_EL0:
+    case SpecialInstOpcode::X86_RDTSC:
+    case SpecialInstOpcode::X86_RDTSCP:
       if (t->tsc_mode == PR_TSC_SIGSEGV) {
         return false;
       }
       break;
-    case TrappedInstruction::CPUID:
+    case SpecialInstOpcode::X86_CPUID:
       if (t->cpuid_mode == 0) {
         return false;
       }
@@ -95,14 +98,24 @@ static bool try_handle_trapped_instruction(RecordTask* t, siginfo_t* si) {
       return false;
   }
 
-  size_t len = trapped_instruction_len(trapped_instruction);
+  size_t len = special_instruction_len(special_instruction.opcode);
   ASSERT(t, len > 0);
 
   Registers r = t->regs();
-  if (trapped_instruction == TrappedInstruction::RDTSC ||
-      trapped_instruction == TrappedInstruction::RDTSCP) {
-    if (trapped_instruction == TrappedInstruction::RDTSC &&
-        t->vm()->monkeypatcher().try_patch_trapping_instruction(t, len, true)) {
+  bool should_retry_patch = false;
+  if (special_instruction.opcode == SpecialInstOpcode::ARM_MRS_CNTVCT_EL0 ||
+      special_instruction.opcode == SpecialInstOpcode::ARM_MRS_CNTVCTSS_EL0) {
+    if (special_instruction.regno != 31) {
+      r.set_x(special_instruction.regno, cntvct());
+    }
+  } else if (special_instruction.opcode == SpecialInstOpcode::ARM_MRS_CNTFRQ_EL0) {
+    if (special_instruction.regno != 31) {
+      r.set_x(special_instruction.regno, cntfrq());
+    }
+  } else if (special_instruction.opcode == SpecialInstOpcode::X86_RDTSC ||
+             special_instruction.opcode == SpecialInstOpcode::X86_RDTSCP) {
+    if (special_instruction.opcode == SpecialInstOpcode::X86_RDTSC &&
+        t->vm()->monkeypatcher().try_patch_trapping_instruction(t, len, true, should_retry_patch)) {
       Event ev = Event::patch_syscall();
       ev.PatchSyscall().patch_trapping_instruction = true;
       t->record_event(ev);
@@ -114,7 +127,7 @@ static bool try_handle_trapped_instruction(RecordTask* t, siginfo_t* si) {
     r.set_rdtsc_output(current_time);
 
     LOG(debug) << " trapped for rdtsc: returning " << current_time;
-  } else if (trapped_instruction == TrappedInstruction::CPUID) {
+  } else if (special_instruction.opcode == SpecialInstOpcode::X86_CPUID) {
     auto eax = r.syscallno();
     auto ecx = r.cx();
     auto cpuid_data = cpuid(eax, ecx);
@@ -129,15 +142,16 @@ static bool try_handle_trapped_instruction(RecordTask* t, siginfo_t* si) {
   t->set_regs(r);
   t->record_event(Event::instruction_trap());
 
-  if (t->retry_syscall_patching) {
+  if (should_retry_patch) {
     LOG(debug) << "Retrying deferred syscall patching";
-    t->retry_syscall_patching = false;
-    if (t->vm()->monkeypatcher().try_patch_trapping_instruction(t, len, false)) {
+    should_retry_patch = false;
+    if (t->vm()->monkeypatcher().try_patch_trapping_instruction(t, len, false, should_retry_patch)) {
       // Instruction was patched. Emit event.
       auto ev = Event::patch_syscall();
       ev.PatchSyscall().patch_after_syscall = true;
       t->record_event(ev);
     }
+    ASSERT(t, !should_retry_patch);
   }
 
   t->push_event(Event::noop());
@@ -154,96 +168,41 @@ static bool try_grow_map(RecordTask* t, siginfo_t* si) {
   // Use kernel_abi to avoid odd inconsistencies between distros
   auto arch_si = reinterpret_cast<NativeArch::siginfo_t*>(si);
   auto addr = arch_si->_sifields._sigfault.si_addr_.rptr();
-
-  if (t->vm()->has_mapping(addr)) {
-    LOG(debug) << "try_grow_map " << addr << ": address already mapped";
-    return false;
+  if (t->try_grow_map(addr)) {
+    t->push_event(Event::noop());
+    return true;
   }
-  auto maps = t->vm()->maps_starting_at(floor_page_size(addr));
-  auto it = maps.begin();
-  if (it == maps.end()) {
-    LOG(debug) << "try_grow_map " << addr << ": no later map to grow downward";
-    return false;
-  }
-  if (!(it->map.flags() & MAP_GROWSDOWN)) {
-    LOG(debug) << "try_grow_map " << addr << ": map is not MAP_GROWSDOWN ("
-               << it->map << ")";
-    return false;
-  }
-  if (addr >= page_size() && t->vm()->has_mapping(addr - page_size())) {
-    LOG(debug) << "try_grow_map " << addr << ": address would be in guard page";
-    return false;
-  }
-  remote_ptr<void> limit_bottom;
-#if defined (__i386__)
-  struct rlimit stack_limit;
-  int ret = prlimit(t->tid, RLIMIT_STACK, NULL, &stack_limit);
-#else
-  struct rlimit64 stack_limit;
-  int ret = syscall(__NR_prlimit64, t->tid, RLIMIT_STACK, (void*)0, &stack_limit);
-#endif
-  if (ret >= 0 && stack_limit.rlim_cur != RLIM_INFINITY) {
-    limit_bottom = ceil_page_size(it->map.end() - stack_limit.rlim_cur);
-    if (limit_bottom > addr) {
-      LOG(debug) << "try_grow_map " << addr << ": RLIMIT_STACK exceeded";
-      return false;
-    }
-  }
-
-  // Try to grow by 64K at a time to reduce signal frequency.
-  auto new_start = floor_page_size(addr);
-  static const uintptr_t grow_size = 0x10000;
-  if (it->map.start().as_int() >= grow_size) {
-    auto possible_new_start = std::max(
-        limit_bottom, std::min(new_start, it->map.start() - grow_size));
-    // Ensure that no mapping exists between possible_new_start - page_size()
-    // and new_start. If there is, possible_new_start is not valid, in which
-    // case we just abandon the optimization.
-    if (possible_new_start >= page_size() &&
-        !t->vm()->has_mapping(possible_new_start - page_size()) &&
-        t->vm()->maps_starting_at(possible_new_start - page_size())
-                .begin()
-                ->map.start() == it->map.start()) {
-      new_start = possible_new_start;
-    }
-  }
-  LOG(debug) << "try_grow_map " << addr << ": trying to grow map " << it->map;
-
-  {
-    AutoRemoteSyscalls remote(t, AutoRemoteSyscalls::DISABLE_MEMORY_PARAMS);
-    remote.infallible_mmap_syscall_if_alive(
-        new_start, it->map.start() - new_start, it->map.prot(),
-        (it->map.flags() & ~MAP_GROWSDOWN) | MAP_ANONYMOUS | MAP_FIXED, -1, 0);
-  }
-
-  KernelMapping km =
-      t->vm()->map(t, new_start, it->map.start() - new_start, it->map.prot(),
-                   it->map.flags() | MAP_ANONYMOUS, 0, string(),
-                   KernelMapping::NO_DEVICE, KernelMapping::NO_INODE);
-  t->trace_writer().write_mapped_region(t, km, km.fake_stat(), km.fsname(), vector<TraceRemoteFd>());
-  // No need to flush syscallbuf here. It's safe to map these pages "early"
-  // before they're really needed.
-  t->record_event(Event::grow_map(), RecordTask::DONT_FLUSH_SYSCALLBUF);
-  t->push_event(Event::noop());
-  LOG(debug) << "try_grow_map " << addr << ": extended map "
-             << t->vm()->mapping_of(addr).map;
-  return true;
+  return false;
 }
 
 void disarm_desched_event(RecordTask* t) {
-  if (t->desched_fd.is_open() &&
-      ioctl(t->desched_fd, PERF_EVENT_IOC_DISABLE, 0)) {
+  ScopedFd& fd = t->desched_fd.tracee_fd();
+  if (fd.is_open() && ioctl(fd, PERF_EVENT_IOC_DISABLE, 0)) {
     FATAL() << "Failed to disarm desched event";
   }
 }
 
 void arm_desched_event(RecordTask* t) {
-  if (t->desched_fd.is_open() &&
-      ioctl(t->desched_fd, PERF_EVENT_IOC_ENABLE, 0)) {
+  ScopedFd& fd = t->desched_fd.tracee_fd();
+  if (fd.is_open() && ioctl(fd, PERF_EVENT_IOC_ENABLE, 0)) {
     FATAL() << "Failed to arm desched event";
   }
 }
 
+bool desched_event_armed(RecordTask *t) {
+  if (t->syscallbuf_child == nullptr) {
+    return false;
+  }
+  bool ok = true;
+  bool is_armed = t->read_mem(
+    REMOTE_PTR_FIELD(t->syscallbuf_child, desched_signal_may_be_relevant), &ok);
+  if (!ok) {
+    // If we can't read this (perhaps syscallbuf isn't actually mapped), it's not armed
+    return false;
+  }
+  return is_armed;
+}
+
 template <typename Arch>
 static remote_code_ptr get_stub_scratch_1_arch(RecordTask* t) {
   auto remote_locals = AddressSpace::preload_thread_locals_start()
@@ -293,8 +252,8 @@ bool handle_syscallbuf_breakpoint(RecordTask* t) {
       uint64_t x15_x30[2];
       get_stub_scratch_2(t, x15_x30, 16);
       Registers r = t->regs();
-      r.set_x15(x15_x30[0]);
-      r.set_xlr(x15_x30[1]);
+      r.set_x(15, x15_x30[0]);
+      r.set_x(30, x15_x30[1]);
       t->set_regs(r);
       t->count_direct_jump();
     }
@@ -343,8 +302,7 @@ bool handle_syscallbuf_breakpoint(RecordTask* t) {
   // We're at an untraced-syscall entry point.
   // To allow an AutoRemoteSyscall, we need to make sure desched signals are
   // disarmed (and rearmed afterward).
-  bool armed_desched_event = t->read_mem(
-      REMOTE_PTR_FIELD(t->syscallbuf_child, desched_signal_may_be_relevant));
+  bool armed_desched_event = desched_event_armed(t);
   if (armed_desched_event) {
     disarm_desched_event(t);
   }
@@ -396,9 +354,7 @@ static void handle_desched_event(RecordTask* t) {
    * the desched_signal_may_be_relevant was set by the outermost syscallbuf
    * invocation.
    */
-  if (!t->read_mem(REMOTE_PTR_FIELD(t->syscallbuf_child,
-                                    desched_signal_may_be_relevant)) ||
-      t->running_inside_desched()) {
+  if (!desched_event_armed(t) || t->running_inside_desched()) {
     LOG(debug) << "  (not entering may-block syscall; resuming)";
     /* We have to disarm the event just in case the tracee
      * has cleared the relevancy flag, but not yet
@@ -761,9 +717,7 @@ SignalHandled handle_signal(RecordTask* t, siginfo_t* si,
       return DEFER_SIGNAL;
     }
 
-    if (!t->set_siginfo_for_synthetic_SIGCHLD(si)) {
-      return DEFER_SIGNAL;
-    }
+    t->set_siginfo_for_synthetic_SIGCHLD(si);
 
     if (sig == PerfCounters::TIME_SLICE_SIGNAL) {
       t->push_event(Event::sched());
