@@ -568,7 +568,7 @@ ioperm = EmulatedSyscall(x86=101, x64=173)
 # through to the appropriate call.
 socketcall = IrregularEmulatedSyscall(x86=102)
 
-syslog = UnsupportedSyscall(x86=103, x64=103, generic=116)
+syslog = IrregularEmulatedSyscall(x86=103, x64=103, generic=116)
 
 #  int setitimer(int which, const struct itimerval *new_value, struct itimerval
 #*old_value);
@@ -1722,11 +1722,11 @@ pidfd_send_signal = EmulatedSyscall(all=424)
 io_uring_setup = IrregularEmulatedSyscall(all=425)
 io_uring_enter = UnsupportedSyscall(all=426)
 io_uring_register = UnsupportedSyscall(all=427)
-open_tree = UnsupportedSyscall(all=428)
-move_mount = UnsupportedSyscall(all=429)
-fsopen = UnsupportedSyscall(all=430)
-fsconfig = UnsupportedSyscall(all=431)
-fsmount = UnsupportedSyscall(all=432)
+open_tree = EmulatedSyscall(all=428)
+move_mount = EmulatedSyscall(all=429)
+fsopen = EmulatedSyscall(all=430)
+fsconfig = EmulatedSyscall(all=431)
+fsmount = EmulatedSyscall(all=432)
 fspick = UnsupportedSyscall(all=433)
 pidfd_open = EmulatedSyscall(all=434)
 clone3 = IrregularEmulatedSyscall(all=435)
@@ -1734,7 +1734,7 @@ openat2 = IrregularEmulatedSyscall(all=437)
 pidfd_getfd = EmulatedSyscall(all=438)
 process_madvise = UnsupportedSyscall(all=440)
 epoll_pwait2 = IrregularEmulatedSyscall(all=441)
-mount_setattr = UnsupportedSyscall(all=442)
+mount_setattr = EmulatedSyscall(all=442)
 quotactl_fd = UnsupportedSyscall(all=443)
 landlock_create_ruleset = EmulatedSyscall(all=444)
 landlock_add_rule = EmulatedSyscall(all=445)
@@ -1744,11 +1744,21 @@ process_mrelease = UnsupportedSyscall(all=448)
 futex_waitv = UnsupportedSyscall(all=449)
 set_mempolicy_home_node = UnsupportedSyscall(all=450)
 cachestat = UnsupportedSyscall(all=451)
-fchmodat2 = UnsupportedSyscall(all=452)
+fchmodat2 = EmulatedSyscall(all=452)
 map_shadow_stack = UnsupportedSyscall(all=453)
 futex_wake = UnsupportedSyscall(all=454)
 futex_wait = UnsupportedSyscall(all=455)
 futex_requeue = UnsupportedSyscall(all=456)
+statmount = UnsupportedSyscall(all=457)
+listmount = UnsupportedSyscall(all=458)
+lsm_get_self_attr = UnsupportedSyscall(all=459)
+lsm_set_self_attr = UnsupportedSyscall(all=460)
+lsm_list_modules = UnsupportedSyscall(all=461)
+mseal = UnsupportedSyscall(all=462)
+setxattrat = UnsupportedSyscall(all=463)
+getxattrat = UnsupportedSyscall(all=464)
+listxattrat = UnsupportedSyscall(all=465)
+removexattrat = UnsupportedSyscall(all=466)
 
 # restart_syscall is a little special.
 restart_syscall = RestartSyscall(x86=0, x64=219, generic=128)
