@@ -3,6 +3,7 @@
 #include "util.h"
 
 static pthread_key_t exit_key;
+static uint64_t exit_key_value;
 
 static void thread_exit(__attribute__((unused)) void* data) {
   atomic_puts("thread exit");
@@ -10,7 +11,7 @@ static void thread_exit(__attribute__((unused)) void* data) {
 
 static void* thread(__attribute__((unused)) void* unused) {
   pthread_key_create(&exit_key, thread_exit);
-  pthread_setspecific(exit_key, (void*)0x1);
+  pthread_setspecific(exit_key, (void*)&exit_key_value);
   pthread_exit(NULL);
 }
 
