@@ -11,12 +11,11 @@ int main(void) {
   test_assert(0 == access(file_path, R_OK));
   test_assert(0 == fchmod(fd, 0200));
   test_assert(0 == access(file_path, W_OK));
-  test_assert(0 == fchmodat(AT_FDCWD, file_path, 0400, 0));
+  test_assert(0 == syscall(RR_fchmodat, AT_FDCWD, file_path, 0400));
+  test_assert(0 == syscall(RR_fchmodat2, AT_FDCWD, file_path, 0400, 0) || errno == ENOSYS);
   test_assert(0 == access(file_path, R_OK));
   test_assert(0 == faccessat(AT_FDCWD, file_path, R_OK, AT_SYMLINK_NOFOLLOW) || errno == ENOSYS);
-#ifdef SYS_faccessat2
-  test_assert(0 == syscall(SYS_faccessat2, AT_FDCWD, file_path, R_OK, AT_SYMLINK_NOFOLLOW) || errno == ENOSYS);
-#endif
+  test_assert(0 == syscall(RR_faccessat2, AT_FDCWD, file_path, R_OK, AT_SYMLINK_NOFOLLOW) || errno == ENOSYS);
 
   atomic_puts("EXIT-SUCCESS");
   return 0;
