@@ -7,7 +7,7 @@ void lib_exit_success(void);
 int main(void) {
   struct timespec ts = { 1, 0 };
   /* try patching clock_nanosleep, which a library thread is in */
-  clock_nanosleep(CLOCK_MONOTONIC, 0, NULL, NULL);
+  clock_nanosleep(CLOCK_MONOTONIC, 0, &ts, NULL);
   nanosleep(&ts, NULL);
 
   lib_exit_success();
