@@ -5,6 +5,6 @@ send_gdb('c')
 expect_gdb('EXIT-SUCCESS')
 expect_gdb('SIGKILL')
 send_gdb('reverse-continue')
-expect_gdb('stopped')
+expect_history_end()
 
 ok()
