@@ -8,7 +8,7 @@ static void handle_sig(__attribute__((unused)) int sig) {
 
 int main(void) {
   int pipe_fd[2];
-  int epfd;
+  int epfd = epoll_create(1 /*num events*/);
   struct timespec ts = { 5, 0 };
   struct epoll_event ev;
   sigset_t sigmask;
@@ -17,8 +17,8 @@ int main(void) {
 
   signal(SIGALRM, handle_sig);
 
+  test_assert(epfd >= 0);
   test_assert(0 == pipe(pipe_fd));
-  test_assert(0 <= (epfd = epoll_create(1 /*num events*/)));
 
   ev.events = EPOLLIN;
   ev.data.fd = pipe_fd[0];
@@ -27,7 +27,7 @@ int main(void) {
   // Make sure something will wake us from the epoll_pwait2.
   alarm(1);
   // But also use the epoll_pwait to modify the signal mask.
-  syscall(RR_epoll_pwait2, epfd, &ev, 1, &ts, &sigmask, 8);
+  syscall(RR_epoll_pwait2, epfd, &ev, 1, &ts, &sigmask, (long)8);
   test_assert(errno == EINTR || errno == ENOSYS);
 
   atomic_puts("EXIT-SUCCESS");
