@@ -1,12 +1,13 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
-static void breakpoint(void) {
-  __asm__("int $3");
-  /* NB: the above instruction *must* be at line 3 in this file.
-   * Tests rely on that. */
-}
 
 #include "util.h"
 
+static void breakpoint(void) {
+  debug_trap();
+  /* NB: the above instruction *must* be at line 6 in this file.
+   * Tests rely on that. */
+}
+
 static void handle_sigtrap(__attribute__((unused)) int sig) {
   atomic_puts("EXIT-SUCCESS");
   _exit(0);
