@@ -28,6 +28,12 @@ int main(void) {
   attr.config = PERF_COUNT_SW_CONTEXT_SWITCHES;
 
   counter_fd = sys_perf_event_open(&attr, 0 /*self*/, -1 /*any cpu*/, -1, 0);
+  if (counter_fd < 0) {
+    test_assert(errno == EACCES);
+    atomic_puts("Skipping test because counter is not available");
+    atomic_puts("EXIT-SUCCESS");
+    return 0;
+  }
   test_assert(0 <= counter_fd);
 
   atomic_printf("num descheds: %" PRIu64 "\n", get_desched());
