@@ -1,17 +1,20 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
 #include "util.h"
-
-#if defined(__i386__)
-#define IP eip
-#elif defined(__x86_64__)
-#define IP rip
-#else
-#error unknown architecture
-#endif
+#include "ptrace_util.h"
 
 static void breakpoint(void) {}
 
+#if defined(__i386__) || defined(__x86_64__)
+char breakpoint_instruction[] = { 0xcc };
+int ip_after_breakpoint = 1;
+#elif defined(__aarch64__)
+char breakpoint_instruction[] = { 0x0, 0x0, 0x20, 0xd4 };
+int ip_after_breakpoint = 0;
+#else
+#error Unknown architecture
+#endif
+
 int main(void) {
   pid_t child;
   int status;
@@ -19,8 +22,8 @@ int main(void) {
   int pipe_fds[2];
   int mem_fd;
   char buf[1024];
-  char breakpoint_instruction = 0xcc;
-  char saved_byte;
+  char saved_bytes[sizeof(breakpoint_instruction)];
+  ssize_t bkpt_size = sizeof(breakpoint_instruction);
 
   test_assert(0 == pipe(pipe_fds));
 
@@ -39,20 +42,23 @@ int main(void) {
   test_assert(child == waitpid(child, &status, 0));
   test_assert(status == ((SIGSTOP << 8) | 0x7f));
 
-  test_assert(1 == pread(mem_fd, &saved_byte, 1, (off_t)breakpoint));
-  test_assert(1 ==
-              pwrite(mem_fd, &breakpoint_instruction, 1, (off_t)breakpoint));
+  test_assert(bkpt_size ==
+    pread(mem_fd, saved_bytes, bkpt_size, (off_t)breakpoint));
+  test_assert(bkpt_size ==
+    pwrite(mem_fd, breakpoint_instruction, bkpt_size, (off_t)breakpoint));
 
   test_assert(1 == write(pipe_fds[1], "x", 1));
   test_assert(0 == ptrace(PTRACE_CONT, child, NULL, (void*)0));
   test_assert(child == waitpid(child, &status, 0));
   test_assert(status == ((SIGTRAP << 8) | 0x7f));
-  test_assert(0 == ptrace(PTRACE_GETREGS, child, NULL, &regs));
-  test_assert((char*)regs.IP == (char*)breakpoint + 1);
+  ptrace_getregs(child, &regs);
+  test_assert((char*)regs.IP == (char*)breakpoint + ip_after_breakpoint ? bkpt_size : 0);
 
-  test_assert(1 == pwrite(mem_fd, &saved_byte, 1, (off_t)breakpoint));
-  --regs.IP;
-  test_assert(0 == ptrace(PTRACE_SETREGS, child, NULL, &regs));
+  test_assert(bkpt_size == pwrite(mem_fd, saved_bytes, bkpt_size, (off_t)breakpoint));
+  if (ip_after_breakpoint) {
+    regs.IP -= bkpt_size;
+  }
+  ptrace_setregs(child, &regs);
   test_assert(0 == ptrace(PTRACE_CONT, child, NULL, (void*)0));
 
   test_assert(child == waitpid(child, &status, 0));
