@@ -1,14 +1,7 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
 #include "util.h"
-
-#if defined(__i386__)
-#define SYSCALL_RESULT eax
-#elif defined(__x86_64__)
-#define SYSCALL_RESULT rax
-#else
-#error unknown architecture
-#endif
+#include "ptrace_util.h"
 
 #ifdef SYS_geteuid32
 #define SYSCALLNO SYS_geteuid32
@@ -16,33 +9,8 @@
 #define SYSCALLNO SYS_geteuid
 #endif
 
-#ifndef PTRACE_SYSEMU_SINGLESTEP
-#define PTRACE_SYSEMU_SINGLESTEP 32
-#endif
-
 extern char syscall_addr;
 
-/* Make a syscallbuf-patchable syscall */
-static uid_t my_geteuid(void) {
-  int r;
-#ifdef __i386__
-  __asm__ __volatile__("syscall_addr: int $0x80\n\t"
-                       "nop\n\t"
-                       "nop\n\t"
-                       "nop\n\t"
-                       : "=a"(r)
-                       : "a"(SYS_geteuid32));
-#elif defined(__x86_64__)
-  __asm__ __volatile__("syscall_addr: syscall\n\t"
-                       "nop\n\t"
-                       "nop\n\t"
-                       "nop\n\t"
-                       : "=a"(r)
-                       : "a"(SYS_geteuid));
-#endif
-  return r;
-}
-
 int main(void) {
   pid_t child;
   int status;
@@ -74,9 +42,9 @@ int main(void) {
     test_assert(WIFSTOPPED(status));
     if (WSTOPSIG(status) == (SIGTRAP | 0x80)) {
       /* Syscall stop, change the return value */
-      test_assert(0 == ptrace(PTRACE_GETREGS, child, NULL, &regs));
+      ptrace_getregs(child, &regs);
       regs.SYSCALL_RESULT = uid + 1;
-      test_assert(0 == ptrace(PTRACE_SETREGS, child, NULL, &regs));
+      ptrace_setregs(child, &regs);
       break;
     }
     test_assert(WSTOPSIG(status) == SIGTRAP);
