@@ -1,38 +1,7 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
 #include "util.h"
-
-#if defined(__i386__)
-#define ORIG_SYSCALLNO orig_eax
-#elif defined(__x86_64__)
-#define ORIG_SYSCALLNO orig_rax
-#else
-#error unknown architecture
-#endif
-
-#if defined(__i386__)
-#define SYSCALL_RESULT eax
-#elif defined(__x86_64__)
-#define SYSCALL_RESULT rax
-#else
-#error unknown architecture
-#endif
-
-#if defined(__i386__)
-#define SYSCALL_ARG1 ebx
-#elif defined(__x86_64__)
-#define SYSCALL_ARG1 rdi
-#else
-#error unknown architecture
-#endif
-
-#if defined(__i386__)
-#define IP eip
-#elif defined(__x86_64__)
-#define IP rip
-#else
-#error unknown architecture
-#endif
+#include "ptrace_util.h"
 
 static void* do_thread(__attribute__((unused)) void* p) { return NULL; }
 
@@ -60,18 +29,20 @@ int main(void) {
     test_assert(0 == ptrace(PTRACE_SYSCALL, child, NULL, (void*)0));
     test_assert(child == waitpid(child, &status, 0));
     test_assert(status == (((0x80 | SIGTRAP) << 8) | 0x7f));
-    test_assert(0 == ptrace(PTRACE_GETREGS, child, NULL, &regs));
+    ptrace_getregs(child, &regs);
   } while (SYS_clone != regs.ORIG_SYSCALLNO);
 
   // Make sure CLONE_UNTRACED is honored.
   regs.SYSCALL_ARG1 |= CLONE_UNTRACED;
-  test_assert(0 == ptrace(PTRACE_SETREGS, child, NULL, &regs));
+  ptrace_setregs(child, &regs);
 
   test_assert(0 == ptrace(PTRACE_SYSCALL, child, NULL, (void*)0));
   test_assert(child == waitpid(child, &status, 0));
   test_assert(status == (((0x80 | SIGTRAP) << 8) | 0x7f));
-  test_assert(0 == ptrace(PTRACE_GETREGS, child, NULL, &regs2));
+  ptrace_getregs(child, &regs2);
+#if !defined(__aarch64__)
   test_assert(SYS_clone == regs2.ORIG_SYSCALLNO);
+#endif
   test_assert(regs.IP == regs2.IP);
 
   test_assert(0 == ptrace(PTRACE_CONT, child, NULL, (void*)0));
