@@ -10,6 +10,6 @@ send_gdb('f 1')
 expect_gdb('(rr)')
 
 send_gdb('p big')
-expect_gdb("bytes = 'Z'")
+expect_gdb('bytes = \'Z\'')
 
 ok()
