@@ -20,35 +20,35 @@ int main(void) {
 
   addr.sun_family = AF_UNIX;
   strcpy(addr.sun_path, sock_name);
-  test_assert(0 == bind(dest, &addr, sizeof(addr)));
+  test_assert(0 == bind(dest, (struct sockaddr*)&addr, sizeof(addr)));
 
   src = socket(AF_UNIX, SOCK_DGRAM, 0);
   test_assert(src >= 0);
 
-  test_assert(36 == sendto(src, buf, 36, 0, &addr, sizeof(addr)));
+  test_assert(36 == sendto(src, buf, 36, 0, (struct sockaddr*)&addr, sizeof(addr)));
   out_addr_len = sizeof(out_addr);
   test_assert(36 ==
-              recvfrom(dest, out, sizeof(out), 0, &out_addr, &out_addr_len));
+              recvfrom(dest, out, sizeof(out), 0, (struct sockaddr*)&out_addr, &out_addr_len));
   test_assert(0 == memcmp(buf, out, 36));
   test_assert(0 == out_addr_len);
 
   addr2.sun_family = AF_UNIX;
   strcpy(addr2.sun_path, sock_name2);
-  test_assert(0 == bind(src, &addr2, sizeof(addr2)));
+  test_assert(0 == bind(src, (struct sockaddr*)&addr2, sizeof(addr2)));
 
-  test_assert(36 == sendto(src, buf, 36, 0, &addr, sizeof(addr)));
+  test_assert(36 == sendto(src, buf, 36, 0, (struct sockaddr*)&addr, sizeof(addr)));
   out_addr_len = sizeof(out_addr);
   test_assert(36 ==
-              recvfrom(dest, out, sizeof(out), 0, &out_addr, &out_addr_len));
+              recvfrom(dest, out, sizeof(out), 0, (struct sockaddr*)&out_addr, &out_addr_len));
   test_assert(0 == memcmp(buf, out, 36));
   test_assert(out_addr_len == 2 + sizeof(sock_name2));
   test_assert(0 == memcmp(out_addr.sun_path, sock_name2, sizeof(sock_name2)));
 
-  test_assert(36 == sendto(src, buf, 36, 0, &addr, sizeof(addr)));
+  test_assert(36 == sendto(src, buf, 36, 0, (struct sockaddr*)&addr, sizeof(addr)));
   out_addr_len = 8;
   out_addr.sun_path[9] = 'x';
   test_assert(36 ==
-              recvfrom(dest, out, sizeof(out), 0, &out_addr, &out_addr_len));
+              recvfrom(dest, out, sizeof(out), 0, (struct sockaddr*)&out_addr, &out_addr_len));
   test_assert(0 == memcmp(buf, out, 36));
   test_assert(out_addr_len == 2 + sizeof(sock_name2));
   test_assert(0 == memcmp(out_addr.sun_path, sock_name2, 8));
