@@ -2,21 +2,6 @@
 
 #include "util.h"
 
-struct rseq {
-  uint32_t cpu_id_start;
-  uint32_t cpu_id;
-  uint64_t rseq_cs;
-  uint32_t flags;
-} __attribute__((aligned(32)));
-
-struct rseq_cs {
-  uint32_t version;
-  uint32_t flags;
-  uint64_t start_ip;
-  uint64_t post_commit_offset;
-  uint64_t abort_ip;
-} __attribute__((aligned(32)));
-
 static struct rseq* rs_ptr;
 static struct rseq_cs rs_cs;
 
@@ -34,7 +19,7 @@ static uint64_t jump_aborts;
 static volatile uint32_t dummy;
 
 static void do_section(void) {
-  int did_abort = 0;
+  long did_abort = 0;
 
   rs_ptr->rseq_cs = (uint64_t)(uintptr_t)&rs_cs;
 #if defined(__x86_64__) || defined(__i386__)
@@ -52,7 +37,7 @@ static void do_section(void) {
     "1:\n\t"
     : : "m"(dummy), "m"(did_abort));
 #elif defined(__aarch64__)
-  int dummy2;
+  long dummy2;
   __asm__ __volatile__ (
     "start_ip:\n\t"
     "mov %1, 1234\n\t"
