@@ -3,7 +3,7 @@
 #include "util.h"
 
 int main(void) {
-  int ret = syscall(RR_seccomp, SECCOMP_SET_MODE_FILTER, 0, NULL);
+  int ret = syscall(RR_seccomp, SECCOMP_SET_MODE_FILTER, 0, (void*)0);
   if (ret == -1 && errno == ENOSYS) {
     ret = prctl(PR_SET_SECCOMP, SECCOMP_MODE_FILTER, NULL);
   }
