@@ -12,7 +12,7 @@ expect_gdb('Checkpoint 1 at')
 send_gdb('c')
 expect_gdb('Breakpoint 1, sighandler')
 
-send_gdb("restart 1");
+send_gdb('restart 1');
 send_gdb('c')
 expect_gdb('Breakpoint 1, sighandler')
 
