@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
-#include "rrutil.h"
+#include "util.h"
 
 static void sighandler(__attribute__((unused)) int sig) {
   /* Must be a syscall we've already executed, otherwise patching gets in the
