@@ -11,6 +11,12 @@ int main(void) {
     return 0;
   }
   int fd = open(tty, O_RDWR);
+  if (fd < 0 && errno == EACCES) {
+    atomic_puts("tty access denied (su in progress?), skipping test");
+    fputs("EXIT-SUCCESS", stderr);
+    return 0;
+  }
+  test_assert(fd >= 0);
   test_assert(13 == write(fd, "EXIT-SUCCESS\n", 13));
   return 0;
 }
