@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: nil; -*- */
 
-#include "rrutil.h"
+#include "util.h"
 
 static void handle_usr1(__attribute__((unused)) int sig) {
   test_assert("Shouldn't have caught SIGUSR1" && 0);
