@@ -120,18 +120,10 @@
 #include <x86intrin.h>
 #endif
 
-#if defined(__i386__)
-#include "SyscallEnumsForTestsX86.generated"
-#elif defined(__x86_64__)
-#include "SyscallEnumsForTestsX64.generated"
-#elif defined(__aarch64__)
-#include "SyscallEnumsForTestsGeneric.generated"
-#else
-#error Unknown architecture
-#endif
-
 #include <rr/rr.h>
 
+#include "util_syscall.h"
+
 typedef unsigned char uint8_t;
 
 #define ALEN(_a) (sizeof(_a) / sizeof(_a[0]))
@@ -228,6 +220,16 @@ inline static void check_data(void* buf, size_t len) {
 inline static uint64_t rdtsc(void) { return __rdtsc(); }
 #endif
 
+inline static void trigger_timer_counter_trap(void) {
+#if defined(__i386__) || defined(__x86_64)
+  rdtsc();
+#elif defined(__aarch64__)
+  __asm__ __volatile__("mrs xzr, cntvct_el0");
+#else
+#error "Unknown architecture"
+#endif
+}
+
 /**
  * Perform some syscall that writes an event, i.e. is not syscall-buffered.
  */
@@ -400,42 +402,6 @@ inline static SyscallWrapper get_spurious_desched_syscall(void) {
   return ret ? ret : default_syscall_wrapper;
 }
 
-static inline uintptr_t unbufferable_syscall(uintptr_t syscall, uintptr_t arg1,
-                                             uintptr_t arg2,
-                                             uintptr_t arg3) {
-  uintptr_t ret;
-#ifdef __x86_64__
-  __asm__ volatile("syscall\n\t"
-                   /* Make sure we don't patch this syscall for syscall buffering */
-                   "cmp $0x77,%%rax\n\t"
-                   : "=a"(ret)
-                   : "a"(syscall), "D"(arg1), "S"(arg2), "d"(arg3)
-                   : "flags");
-#elif defined(__i386__)
-  __asm__ volatile("xchg %%esi,%%edi\n\t"
-                   "int $0x80\n\t"
-                   "xchg %%esi,%%edi\n\t"
-                   : "=a"(ret)
-                   : "a"(syscall), "b"(arg1), "c"(arg2), "d"(arg3));
-#elif defined(__aarch64__)
-  register long x8 __asm__("x8") = syscall;
-  register long x0 __asm__("x0") = (long)arg1;
-  register long x1 __asm__("x1") = (long)arg2;
-  register long x2 __asm__("x2") = (long)arg3;
-  __asm__ volatile("b 1f\n\t"
-                   "mov x8, 0xdc\n"
-                   "1:\n\t"
-                   "svc #0\n\t"
-                   : "+r"(x0)
-                   : "r"(x1), "r"(x2), "r"(x8));
-  ret = x0;
-#else
-#error define syscall here
-#endif
-  return ret;
-}
-
-
 /* Old systems don't have these functions, re-define using the syscall */
 #define tgkill(tgid, tid, sig) \
   syscall(SYS_tgkill, (int)(tgid), (int)(tid), (int)(sig))
@@ -464,6 +430,14 @@ static inline uintptr_t unbufferable_syscall(uintptr_t syscall, uintptr_t arg1,
 #define RR_KCMP_FILE 0
 #define RR_KCMP_FILES 2
 
+/* Old systems don't have linux/openat2.h */
+struct open_how {
+	__u64 flags;
+	__u64 mode;
+	__u64 resolve;
+};
+#define RESOLVE_BENEATH	0x08
+
 /* Old systems don't have these */
 #ifndef TIOCGPKT
 #define TIOCGPKT _IOR('T', 0x38, int)
@@ -577,4 +551,59 @@ static inline uintptr_t unbufferable_syscall(uintptr_t syscall, uintptr_t arg1,
 #define MREMAP_DONTUNMAP 4
 #endif
 
+struct rseq {
+  uint32_t cpu_id_start;
+  uint32_t cpu_id;
+  uint64_t rseq_cs;
+  uint32_t flags;
+} __attribute__((aligned(32)));
+
+struct rseq_cs {
+  uint32_t version;
+  uint32_t flags;
+  uint64_t start_ip;
+  uint64_t post_commit_offset;
+  uint64_t abort_ip;
+} __attribute__((aligned(32)));
+
+enum {
+  RR_BPF_MAP_CREATE,
+  RR_BPF_MAP_LOOKUP_ELEM,
+  RR_BPF_MAP_UPDATE_ELEM,
+  RR_BPF_MAP_DELETE_ELEM,
+  RR_BPF_MAP_GET_NEXT_KEY,
+  RR_BPF_PROG_LOAD,
+  RR_BPF_OBJ_PIN,
+  RR_BPF_OBJ_GET,
+  RR_BPF_PROG_ATTACH,
+  RR_BPF_PROG_DETACH,
+  RR_BPF_PROG_TEST_RUN,
+  RR_BPF_PROG_GET_NEXT_ID,
+  RR_BPF_MAP_GET_NEXT_ID,
+  RR_BPF_PROG_GET_FD_BY_ID,
+  RR_BPF_MAP_GET_FD_BY_ID,
+  RR_BPF_OBJ_GET_INFO_BY_FD,
+  RR_BPF_PROG_QUERY,
+  RR_BPF_RAW_TRACEPOINT_OPEN,
+  RR_BPF_BTF_LOAD,
+  RR_BPF_BTF_GET_FD_BY_ID,
+  RR_BPF_TASK_FD_QUERY,
+  RR_BPF_MAP_LOOKUP_AND_DELETE_ELEM,
+  RR_BPF_MAP_FREEZE,
+  RR_BPF_BTF_GET_NEXT_ID,
+  RR_BPF_MAP_LOOKUP_BATCH,
+  RR_BPF_MAP_LOOKUP_AND_DELETE_BATCH,
+  RR_BPF_MAP_UPDATE_BATCH,
+  RR_BPF_MAP_DELETE_BATCH,
+  RR_BPF_LINK_CREATE,
+  RR_BPF_LINK_UPDATE,
+  RR_BPF_LINK_GET_FD_BY_ID,
+  RR_BPF_LINK_GET_NEXT_ID,
+  RR_BPF_ENABLE_STATS,
+  RR_BPF_ITER_CREATE,
+  RR_BPF_LINK_DETACH,
+  RR_BPF_PROG_BIND_MAP,
+  RR_BPF_TOKEN_CREATE,
+};
+
 #endif /* RRUTIL_H */
