@@ -87,7 +87,8 @@ function onexit {
             rm -rf $nontmp_workdir
         fi
     else
-        echo Test $TESTNAME failed, leaving behind $tmp_workdir and $nontmp_workdir
+        echo -n Test $TESTNAME failed, leaving behind $tmp_workdir
+        (rmdir "$nontmp_workdir" 2>/dev/null && echo) || echo " and $nontmp_workdir"
         echo To replay the failed test, run
         echo " " _RR_TRACE_DIR="$workdir" rr replay
         exit 1
@@ -152,7 +153,7 @@ test_passed=y
 nonce=
 
 # Set up the environment and working directory.
-TESTDIR="${SRCDIR}/src/test"
+export TESTDIR="${SRCDIR}/src/test"
 
 # Make rr treat temp files as durable. This saves copying all test
 # binaries into the trace.
@@ -207,7 +208,7 @@ nontmp_workdir=`mktemp -p $PWD -dt rr-test-$TESTNAME-XXXXXXXXX`
 cd $workdir
 # NB: the testsuite should run on any system with different settings,
 #     so create a reasonable default for all tests
-export LC_ALL=en_US.UTF-8
+export LC_ALL=C
 
 # XXX technically the trailing -XXXXXXXXXX isn't unique, since there
 # could be "foo-123456789" and "bar-123456789", but if that happens,
@@ -318,7 +319,8 @@ function record_async_signal { sig=$1; delay_secs=$2; exe=$3; exeargs=$4;
 
 function replay { replayflags=$1
     _RR_TRACE_DIR="$workdir" test-monitor $TIMEOUT replay.err \
-        $RR_EXE $GLOBAL_OPTIONS replay -a $replayflags 1> replay.out 2> replay.err
+        $RR_EXE $GLOBAL_OPTIONS replay --retry-transient-errors -a \
+        $replayflags 1> replay.out 2> replay.err
 }
 
 function rerun { rerunflags=$1
@@ -326,9 +328,9 @@ function rerun { rerunflags=$1
         $RR_EXE $GLOBAL_OPTIONS rerun $rerunflags 1> rerun.out 2> rerun.err
 }
 
-function pack { packflags=$1
+function pack {
     _RR_TRACE_DIR="$workdir" test-monitor $TIMEOUT pack.err \
-        $RR_EXE $GLOBAL_OPTIONS pack $packflags 1> pack.out 2> pack.err
+        $RR_EXE $GLOBAL_OPTIONS pack $@ 1> pack.out 2> pack.err
 }
 
 function do_ps { psflags=$1
@@ -337,22 +339,47 @@ function do_ps { psflags=$1
 }
 
 #  debug <expect-script-name> [replay-args]
+function debug {
+    debug_gdb_only $TEST_PREFIX$TESTNAME_NO_BITNESS
+    if [[ "$test_passed" == "y" ]]; then
+        debug_lldb_only $TEST_PREFIX$TESTNAME_NO_BITNESS
+    fi
+}
+
+#  debug_lldb_only <expect-script-name> [replay-args]
 #
 # Load the "expect" script to drive replay of the recording of |exe|.
-function debug { expectscript=$1; replayargs=$2
-    _RR_TRACE_DIR="$workdir" test-monitor $TIMEOUT debug.err \
+# Only LLDB is tested.
+function debug_lldb_only { expectscript=$1; replayargs=$2
+    RR_LOG_FILE=rr.log _RR_TRACE_DIR="$workdir" test-monitor $TIMEOUT test-monitor.output \
+        python3 $TESTDIR/$expectscript.py --lldb \
+        $RR_EXE $GLOBAL_OPTIONS replay $replayargs
+    if [[ $? == 0 ]]; then
+        passed_msg lldb
+    else
+        failed "debug script failed (lldb); see `pwd`/lldb_rr.log and `pwd`/test-monitor.output"
+        echo "--------------------------------------------------"
+        echo "rr.log:"
+        cat rr.log
+        echo "--------------------------------------------------"
+    fi
+}
+
+#  debug_gdb_only <expect-script-name> [replay-args]
+#
+# Load the "expect" script to drive replay of the recording of |exe|.
+# Only GDB is tested.
+function debug_gdb_only { expectscript=$1; replayargs=$2
+    RR_LOG_FILE=rr.log _RR_TRACE_DIR="$workdir" test-monitor $TIMEOUT test-monitor.output \
         python3 $TESTDIR/$expectscript.py \
         $RR_EXE $GLOBAL_OPTIONS replay -o-n -o-ix -o$TESTDIR/test_setup.gdb $replayargs
     if [[ $? == 0 ]]; then
-        passed
+        passed_msg gdb
     else
-        failed "debug script failed"
+        failed "debug script failed (gdb); see `pwd`/gdb_rr.log and `pwd`/test-monitor.output"
         echo "--------------------------------------------------"
-        echo "gdb_rr.log:"
-        cat gdb_rr.log
-        echo "--------------------------------------------------"
-        echo "debug.err:"
-        cat debug.err
+        echo "rr.log:"
+        cat rr.log
         echo "--------------------------------------------------"
     fi
 }
@@ -366,6 +393,10 @@ function passed {
     echo "Test '$TESTNAME' PASSED"
 }
 
+function passed_msg { msg=$1
+    echo "Test '$TESTNAME' PASSED ($msg)"
+}
+
 function just_check_replay_err {
     if [[ $(cat replay.err) != "" ]]; then
         failed ": error during replay:"
@@ -476,7 +507,17 @@ function compare_test { token=$1; replayflags=$2;
 # computing test pass/fail.
 function debug_test {
     record $TESTNAME
-    debug $TEST_PREFIX$TESTNAME_NO_BITNESS
+    debug
+}
+
+#  debug_test_gdb_only
+#
+# Record the test name passed to |util.sh|, then replay the recording
+# using the "expect" script $test-name.py, which is responsible for
+# computing test pass/fail. Only GDB is tested.
+function debug_test_gdb_only {
+    record $TESTNAME
+    debug_gdb_only $TEST_PREFIX$TESTNAME_NO_BITNESS
 }
 
 #  rerun_singlestep_test
@@ -488,9 +529,14 @@ function rerun_singlestep_test {
     rerun "--singlestep=rip,gp_x16,flags"
 }
 
+# Return an rr dump result of the most recent local recording.
+function get_events {
+    $RR_EXE $GLOBAL_OPTIONS dump "$@" latest-trace
+}
+
 # Return the number of events in the most recent local recording.
 function count_events {
-    local events=$($RR_EXE $GLOBAL_OPTIONS dump -r latest-trace | wc -l)
+    local events=$(get_events -r | wc -l)
     # The |simple| test is just about the simplest possible C program,
     # and has around 180 events (when recorded on a particular
     # developer's machine).  If we count a number of events
@@ -532,7 +578,7 @@ function checkpoint_test { exe=$1; min=$2; max=$3;
     stride=$(rand_range $min $max)
     for i in $(seq 1 $stride $num_events); do
         echo Checkpointing at event $i ...
-        debug restart_finish "-g $i"
+        debug_gdb_only restart_finish "-g $i"
         if [[ "$test_passed" != "y" ]]; then
             break
         fi
@@ -541,8 +587,25 @@ function checkpoint_test { exe=$1; min=$2; max=$3;
 
 function wait_for_complete {
     local record_dir=${1:-"${workdir}/latest-trace"}
-    for ((i = 0; i < TIMEOUT * 10; i++)); do
+    for ((i = 0; i < $TIMEOUT * 10; i++)); do
         [[ -f "$record_dir/incomplete" ]] || break
         sleep 0.1
     done
 }
+
+# If not given by user, try to find out if C.UTF-8 or en_US.UTF-8 is available, if not use any UTF-8 locale.
+function set_utf_locale {
+    unset LC_ALL
+    if [ -z "$(which locale)" ]; then
+        if [ -z "$LC_ALL" ]; then export LC_ALL=en_US.UTF-8; fi
+    else
+        if [ -z "$LC_ALL" ]; then export LC_ALL=$(locale -a | grep -i -E "C\.utf.*8" | head -n1); fi
+        if [ -z "$LC_ALL" ]; then export LC_ALL=$(locale -a | grep -i -E "en_US\.utf.*8" | head -n1); fi
+        if [ -z "$LC_ALL" ]; then export LC_ALL=$(locale -a | grep -i -E ".*\.utf.*8" | head -n1); fi
+        if [ -z "$LC_ALL" ]; then
+            echo "Warning: no UTF-8 locale found."
+        else
+            echo "Using LC_ALL=$LC_ALL"
+        fi
+    fi
+}
