@@ -2,6 +2,11 @@
 
 #include "util.h"
 
+static void breakpoint(void) {
+  int break_here = 1;
+  (void)break_here;
+}
+
 int main(int argc, char* argv[]) {
   const char* exe;
   pid_t child;
@@ -20,6 +25,8 @@ int main(int argc, char* argv[]) {
   test_assert(child == waitpid(child, &status, 0));
   test_assert(WIFEXITED(status) && 0 == WEXITSTATUS(status));
 
+  breakpoint();
+
   atomic_puts("vforker-EXIT-SUCCESS");
   return 0;
 }
