@@ -1052,6 +1052,9 @@ bool cpuid_faulting_works() {
   static bool did_check_cpuid_faulting = false;
   static bool cpuid_faulting_ok = false;
 
+#if !(defined(__i386__) || defined(__x86_64__))
+  did_check_cpuid_faulting = true;
+#endif
   if (did_check_cpuid_faulting) {
     return cpuid_faulting_ok;
   }
@@ -1758,9 +1761,6 @@ OpenedSocket open_socket(const string& host, unsigned short port,
 }
 
 void notifying_abort() {
-  flush_log_buffer();
-  dump_rr_stack();
-
   char* test_monitor_pid = getenv("RUNNING_UNDER_TEST_MONITOR");
   if (test_monitor_pid) {
     pid_t pid = atoi(test_monitor_pid);
@@ -1773,19 +1773,19 @@ void notifying_abort() {
   abort();
 }
 
-void dump_rr_stack() {
+void dump_rr_stack(ScopedFd& fd) {
   static const char msg[] = "=== Start rr backtrace:\n";
-  write_all(STDERR_FILENO, msg, sizeof(msg) - 1);
+  write_all(fd, msg, sizeof(msg) - 1);
 #ifdef EXECINFO_BACKTRACE
   void* buffer[1024];
   int count = backtrace(buffer, 1024);
-  backtrace_symbols_fd(buffer, count, STDERR_FILENO);
+  backtrace_symbols_fd(buffer, count, fd);
 #else
   static const char msg_fallback[] = "<rr backtraces not available on this system>\n";
-  write_all(STDERR_FILENO, msg_fallback, sizeof(msg_fallback) - 1);
+  write_all(fd, msg_fallback, sizeof(msg_fallback) - 1);
 #endif
   static const char msg2[] = "=== End rr backtrace\n";
-  write_all(STDERR_FILENO, msg2, sizeof(msg2) - 1);
+  write_all(fd, msg2, sizeof(msg2) - 1);
 }
 
 void check_for_leaks() {
@@ -1947,54 +1947,78 @@ static const uint8_t pushf_insn[] = { 0x9c };
 static const uint8_t pushf16_insn[] = { 0x66, 0x9c };
 
 // XXX this probably needs to be extended to decode ignored prefixes
-TrappedInstruction trapped_instruction_at(Task* t, remote_code_ptr ip) {
-  uint8_t insn[sizeof(rdtscp_insn)];
-  ssize_t ret =
-      t->read_bytes_fallible(ip.to_data_ptr<uint8_t>(), sizeof(insn), insn);
-  if (ret < 0) {
-    return TrappedInstruction::NONE;
+SpecialInst special_instruction_at(Task* t, remote_code_ptr ip) {
+  if (is_x86ish(t->arch())) {
+    uint8_t insn[sizeof(rdtscp_insn)];
+    ssize_t ret =
+        t->read_bytes_fallible(ip.to_data_ptr<uint8_t>(), sizeof(insn), insn);
+    if (ret < 0) {
+      return {SpecialInstOpcode::NONE};
+    }
+    size_t len = ret;
+    if (len >= sizeof(rdtsc_insn) &&
+        !memcmp(insn, rdtsc_insn, sizeof(rdtsc_insn))) {
+      return {SpecialInstOpcode::X86_RDTSC};
+    }
+    if (len >= sizeof(rdtscp_insn) &&
+        !memcmp(insn, rdtscp_insn, sizeof(rdtscp_insn))) {
+      return {SpecialInstOpcode::X86_RDTSCP};
+    }
+    if (len >= sizeof(cpuid_insn) &&
+        !memcmp(insn, cpuid_insn, sizeof(cpuid_insn))) {
+      return {SpecialInstOpcode::X86_CPUID};
+    }
+    if (len >= sizeof(int3_insn) &&
+        !memcmp(insn, int3_insn, sizeof(int3_insn))) {
+      return {SpecialInstOpcode::X86_INT3};
+    }
+    if (len >= sizeof(pushf_insn) &&
+        !memcmp(insn, pushf_insn, sizeof(pushf_insn))) {
+      return {SpecialInstOpcode::X86_PUSHF};
+    }
+    if (len >= sizeof(pushf16_insn) &&
+        !memcmp(insn, pushf16_insn, sizeof(pushf16_insn))) {
+      return {SpecialInstOpcode::X86_PUSHF16};
+    }
+  } else if (t->arch() == aarch64) {
+    uint8_t insn[4];
+    ssize_t ret =
+        t->read_bytes_fallible(ip.to_data_ptr<uint8_t>(), sizeof(insn), insn);
+    if (ret < 0) {
+      return {SpecialInstOpcode::NONE};
+    }
+    uint32_t insn_word =
+        insn[0] | (insn[1] << 8) | (insn[2] << 16) | (insn[3] << 24);
+    if ((insn_word & 0xffffffe0) == 0xd53be000) {
+      return {SpecialInstOpcode::ARM_MRS_CNTFRQ_EL0, insn_word & 31};
+    }
+    if ((insn_word & 0xffffffe0) == 0xd53be040) {
+      return {SpecialInstOpcode::ARM_MRS_CNTVCT_EL0, insn_word & 31};
+    }
+    if ((insn_word & 0xffffffe0) == 0xd53be0c0) {
+      return {SpecialInstOpcode::ARM_MRS_CNTVCTSS_EL0, insn_word & 31};
+    }
   }
-  size_t len = ret;
-  if (len >= sizeof(rdtsc_insn) &&
-      !memcmp(insn, rdtsc_insn, sizeof(rdtsc_insn))) {
-    return TrappedInstruction::RDTSC;
-  }
-  if (len >= sizeof(rdtscp_insn) &&
-      !memcmp(insn, rdtscp_insn, sizeof(rdtscp_insn))) {
-    return TrappedInstruction::RDTSCP;
-  }
-  if (len >= sizeof(cpuid_insn) &&
-      !memcmp(insn, cpuid_insn, sizeof(cpuid_insn))) {
-    return TrappedInstruction::CPUID;
-  }
-  if (len >= sizeof(int3_insn) &&
-      !memcmp(insn, int3_insn, sizeof(int3_insn))) {
-    return TrappedInstruction::INT3;
-  }
-  if (len >= sizeof(pushf_insn) &&
-      !memcmp(insn, pushf_insn, sizeof(pushf_insn))) {
-    return TrappedInstruction::PUSHF;
-  }
-  if (len >= sizeof(pushf16_insn) &&
-      !memcmp(insn, pushf16_insn, sizeof(pushf16_insn))) {
-    return TrappedInstruction::PUSHF16;
-  }
-  return TrappedInstruction::NONE;
+  return {SpecialInstOpcode::NONE};
 }
 
-size_t trapped_instruction_len(TrappedInstruction insn) {
-  if (insn == TrappedInstruction::RDTSC) {
+size_t special_instruction_len(SpecialInstOpcode insn) {
+  if (insn == SpecialInstOpcode::X86_RDTSC) {
     return sizeof(rdtsc_insn);
-  } else if (insn == TrappedInstruction::RDTSCP) {
+  } else if (insn == SpecialInstOpcode::X86_RDTSCP) {
     return sizeof(rdtscp_insn);
-  } else if (insn == TrappedInstruction::CPUID) {
+  } else if (insn == SpecialInstOpcode::X86_CPUID) {
     return sizeof(cpuid_insn);
-  } else if (insn == TrappedInstruction::INT3) {
+  } else if (insn == SpecialInstOpcode::X86_INT3) {
     return sizeof(int3_insn);
-  } else if (insn == TrappedInstruction::PUSHF) {
+  } else if (insn == SpecialInstOpcode::X86_PUSHF) {
     return sizeof(pushf_insn);
-  } else if (insn == TrappedInstruction::PUSHF16) {
+  } else if (insn == SpecialInstOpcode::X86_PUSHF16) {
     return sizeof(pushf16_insn);
+  } else if (insn == SpecialInstOpcode::ARM_MRS_CNTFRQ_EL0 ||
+             insn == SpecialInstOpcode::ARM_MRS_CNTVCT_EL0 ||
+             insn == SpecialInstOpcode::ARM_MRS_CNTVCTSS_EL0) {
+    return 4;
   } else {
     return 0;
   }
@@ -2264,6 +2288,10 @@ bool remove_latest_trace_symlink() {
   return true;
 }
 
+static bool ends_with(std::string_view str, std::string_view suffix) {
+  return str.size() >= suffix.size() && str.compare(str.size()-suffix.size(), suffix.size(), suffix) == 0;
+}
+
 bool is_valid_trace_name(const string& entry, std::string* reason) {
   // filename corresponds to dirname
   const string name = filename(entry.c_str());
@@ -2286,9 +2314,9 @@ bool is_valid_trace_name(const string& entry, std::string* reason) {
     }
     return false;
   }
-  if (name == "cpu_lock") {
+  if (name == "cpu_lock" || name == "src" || ends_with(name, ".xml")) {
     if (reason) {
-      *reason = "Name cpu_lock is reserved";
+      *reason = "Name " + name + " is reserved";
     }
     return false;
   }
@@ -2314,30 +2342,48 @@ ssize_t read_to_end(const ScopedFd& fd, size_t offset, void* buf, size_t size) {
   return ret;
 }
 
-static struct rlimit initial_fd_limit;
+static struct rlimit raise_resource_limit(int resource, rlim_t max_value) {
+  struct rlimit initial;
 
-void raise_resource_limits() {
-  if (getrlimit(RLIMIT_NOFILE, &initial_fd_limit) < 0) {
-    FATAL() << "Can't get RLIMIT_NOFILE";
+  if (getrlimit(resource, &initial) < 0) {
+    FATAL() << "Can't get rlimit " << rlimit_resource_name(resource);
   }
 
-  struct rlimit new_limit = initial_fd_limit;
-  // Try raising fd limit to 65536
-  new_limit.rlim_cur = max<rlim_t>(new_limit.rlim_cur, 65536);
+  struct rlimit new_limit = initial;
+  new_limit.rlim_cur = max<rlim_t>(new_limit.rlim_cur, max_value);
   if (new_limit.rlim_max != RLIM_INFINITY) {
     new_limit.rlim_cur = min<rlim_t>(new_limit.rlim_cur, new_limit.rlim_max);
   }
-  if (new_limit.rlim_cur != initial_fd_limit.rlim_cur) {
-    if (setrlimit(RLIMIT_NOFILE, &new_limit) < 0) {
-      LOG(warn) << "Failed to raise file descriptor limit";
+  if (new_limit.rlim_cur != initial.rlim_cur) {
+    if (setrlimit(resource, &new_limit) < 0) {
+      LOG(warn) << "Failed to raise rlimit " << rlimit_resource_name(resource)
+          << " to " << new_limit.rlim_cur;
     }
   }
+
+  return initial;
+}
+
+static void restore_resource_limit(int resource, const struct rlimit& old_limit) {
+  if (setrlimit(resource, &old_limit) < 0) {
+    LOG(warn) << "Failed to reset rlimit " << rlimit_resource_name(resource);
+  }
+}
+
+static const int MAX_TRACEE_TASKS = 65536;
+static struct rlimit initial_fd_limit;
+static struct rlimit initial_memlock_limit;
+
+void raise_resource_limits() {
+  // We need up to 7 perf event counters per tracee task
+  initial_fd_limit = raise_resource_limit(RLIMIT_NOFILE, 1024 + 7 * MAX_TRACEE_TASKS);
+  // We may need one page of locked memory per tracee task
+  initial_memlock_limit = raise_resource_limit(RLIMIT_MEMLOCK, page_size() * 1 * MAX_TRACEE_TASKS);
 }
 
 void restore_initial_resource_limits() {
-  if (setrlimit(RLIMIT_NOFILE, &initial_fd_limit) < 0) {
-    LOG(warn) << "Failed to reset file descriptor limit";
-  }
+  restore_resource_limit(RLIMIT_NOFILE, initial_fd_limit);
+  restore_resource_limit(RLIMIT_MEMLOCK, initial_memlock_limit);
 }
 
 template <typename Arch> static size_t word_size_arch() {
@@ -2585,4 +2631,63 @@ void base_name(string& s) {
   }
 }
 
+static optional<int> init_read_perf_event_paranoid() {
+  ScopedFd fd("/proc/sys/kernel/perf_event_paranoid", O_RDONLY);
+  if (fd.is_open()) {
+    char buf[100];
+    ssize_t size = read(fd, buf, sizeof(buf) - 1);
+    if (size >= 0) {
+      buf[size] = 0;
+      return atoi(buf);
+    }
+  }
+
+  return nullopt;
+}
+
+optional<int> read_perf_event_paranoid() {
+  static optional<int> value = init_read_perf_event_paranoid();
+  return value;
+}
+
+bool virtual_address_size_supported(uint8_t bits) {
+  DEBUG_ASSERT(bits < 64);
+
+  // The easy, statically-determinable case.
+  // Legacy traces with bits == 0 will take this offramp too.
+  if (default_virtual_address_size(NativeArch::arch()) >= bits) {
+    return true;
+  }
+
+  // The architecture may have extensions (like five level paging) that allow
+  // higher addresses to be used. See if they're available.
+  size_t num_bytes = page_size();
+  void* ptr = (void*)(1ULL << (bits - 1));
+  void* map = mmap(ptr, num_bytes, PROT_NONE,
+                   MAP_PRIVATE | MAP_ANONYMOUS | MAP_FIXED_NOREPLACE, -1, 0);
+  if (map == ptr) {
+    // If we successfully mapped at a high address, the extensions work.
+    munmap(map, num_bytes);
+    return true;
+  }
+
+  if (map == MAP_FAILED) {
+    if (errno == EEXIST) {
+      // If we failed to map because something else was already at the high
+      // address, the extensions work.
+      return true;
+    }
+    // If we failed for some other reason, assume the extensions do not work.
+    return false;
+  }
+
+  // If we got back a different address, assume the extensions do not work.
+  // NB: We could potentially be on a kernel that supports five level paging (so
+  // >= 4.14) but not MAP_FIXED_NOREPLACE (so < 4.17) *and* have something
+  // already using the address we tried. Handling that situation correctly
+  // adds substantial complexity, so we just give a false negative there.
+  munmap(map, num_bytes);
+  return false;
+}
+
 } // namespace rr
