@@ -9,6 +9,7 @@
 
 #include <array>
 #include <map>
+#include <optional>
 #include <set>
 #include <string>
 #include <vector>
@@ -17,6 +18,7 @@
 #include <x86intrin.h>
 #endif
 
+#include "MemoryRange.h"
 #include "ScopedFd.h"
 #include "TraceFrame.h"
 #include "remote_ptr.h"
@@ -28,10 +30,6 @@
  * so we can't use it. */
 #define SYSCALLBUF_DEFAULT_DESCHED_SIGNAL SIGPWR
 
-#ifndef SOL_NETLINK
-#define SOL_NETLINK 270
-#endif
-
 #define UNUSED(expr)     \
   do {                   \
     if (expr) {          \
@@ -54,6 +52,9 @@ class KernelMapping;
 class Task;
 class TraceFrame;
 class RecordTask;
+class ReplayTask;
+
+typedef int BindCPU;
 
 enum Completion { COMPLETE, INCOMPLETE };
 
@@ -93,14 +94,13 @@ void dump_binary_data(const char* filename, const char* label,
 /**
  * Format a suitable filename within the trace directory for dumping
  * information about |t| at the current global time, to a file that
- * contains |tag|.  The constructed filename is returned through
- * |filename|.  For example, a filengit logame for a task with tid 12345 at
+ * contains |tag|.  For example, a file for a task with tid 12345 at
  * time 111, for a file tagged "foo", would be something like
- * "trace_0/12345_111_foo".  The returned name is not guaranteed to be
+ * "trace_0/111_12345_foo".  The returned name is not guaranteed to be
  * unique, caveat emptor.
  */
-void format_dump_filename(Task* t, FrameTime global_time, const char* tag,
-                          char* filename, size_t filename_size);
+std::string format_dump_filename(Task* t, FrameTime global_time,
+                                 const char* tag);
 
 /**
  * Return true if the user requested memory be dumped at this event/time.
@@ -122,12 +122,23 @@ bool should_checksum(const Event& event, FrameTime time);
  * special log, where it can be read by |validate_process_memory()|
  * during replay.
  */
-void checksum_process_memory(Task* t, FrameTime global_time);
+void checksum_process_memory(RecordTask* t, FrameTime global_time);
 /**
  * Validate the checksum of |t|'s address space that was written
  * during recording.
  */
-void validate_process_memory(Task* t, FrameTime global_time);
+void validate_process_memory(ReplayTask* t, FrameTime global_time);
+
+/**
+ * Write raw PT data to a file in the trace dir.
+ */
+void write_pt_data(Task* t, FrameTime global_time,
+                   const std::vector<std::vector<uint8_t>>& data);
+
+/**
+ * Read raw PT data to a file in the trace dir. Returns an empty vector if none found.
+ */
+std::vector<uint8_t> read_pt_data(Task* t, FrameTime global_time);
 
 /**
  * Return nonzero if the rr session is probably not interactive (that
@@ -399,18 +410,34 @@ inline bool is_kernel_trap(int si_code) {
 
 enum ProbePort { DONT_PROBE = 0, PROBE_PORT };
 
-ScopedFd open_socket(const char* address, unsigned short* port,
-                     ProbePort probe);
+struct OpenedSocket {
+  ScopedFd fd;
+  int domain;
+  std::string host;
+  unsigned short port;
+};
+
+// Open a socket bound to the given address and port.
+// If PROBE_PORT is set, probes for a usable port and sets it
+// in *port.
+// If `host` is empty, binds to localhost.
+// Returns the actual bound address, socket domain, and port.
+// Selects IPv4 or IPv6 automatically depending on what's in the
+// host address and what's available.
+OpenedSocket open_socket(const std::string& host, unsigned short port,
+                         ProbePort probe);
 
 /**
  * Like `abort`, but tries to wake up test-monitor for a snapshot if possible.
+ * We try not to allocate.
  */
 void notifying_abort();
 
 /**
- * Dump the current rr stack
+ * Dump the current rr stack to the given file.
+ * We try not to allocate.
  */
-void dump_rr_stack();
+void dump_rr_stack(ScopedFd& fd);
 
 /**
  * Check for leaked mappings etc
@@ -459,23 +486,31 @@ std::vector<std::string> current_env();
  */
 int get_num_cpus();
 
-enum class TrappedInstruction {
-  NONE = 0,
-  RDTSC = 1,
-  RDTSCP = 2,
-  CPUID = 3,
-  INT3 = 4,
-  PUSHF = 5,
-  PUSHF16 = 6,
+enum class SpecialInstOpcode {
+  NONE,
+  ARM_MRS_CNTFRQ_EL0,
+  ARM_MRS_CNTVCT_EL0,
+  ARM_MRS_CNTVCTSS_EL0,
+  X86_RDTSC,
+  X86_RDTSCP,
+  X86_CPUID,
+  X86_INT3,
+  X86_PUSHF,
+  X86_PUSHF16,
+};
+
+struct SpecialInst {
+  SpecialInstOpcode opcode;
+  unsigned regno = 0;
 };
 
 /* If |t->ip()| points at a decoded instruction, return the instruction */
-TrappedInstruction trapped_instruction_at(Task* t, remote_code_ptr ip);
+SpecialInst special_instruction_at(Task* t, remote_code_ptr ip);
 
 extern const uint8_t rdtsc_insn[2];
 
 /* Return the length of the TrappedInstruction */
-size_t trapped_instruction_len(TrappedInstruction insn);
+size_t special_instruction_len(SpecialInstOpcode insn);
 
 /**
  * Certain instructions generate deterministic signals but also advance pc.
@@ -488,7 +523,7 @@ bool is_advanced_pc_and_signaled_instruction(Task* t, remote_code_ptr ip);
  * UNBOUND_CPU means not binding to a particular CPU.
  * A non-negative value means binding to the specific CPU number.
  */
-enum BindCPU { BIND_CPU = -2, UNBOUND_CPU = -1 };
+enum { BIND_CPU = -2, UNBOUND_CPU = -1 };
 
 /* Get the path of the cpu lock file */
 std::string get_cpu_lock_file();
@@ -619,17 +654,53 @@ inline unsigned long long rdtsc(void) {
 #endif
 }
 
-inline unsigned long long dczid_el0_block_size(void) {
+inline unsigned long long cntfrq(void) {
 #if defined(__aarch64__)
   unsigned long long val;
-  asm volatile("mrs %0, DCZID_EL0" : "=r" (val));
-  return 1ULL << (val & 0xF);
+  asm volatile("mrs %0, CNTFRQ_EL0" : "=r" (val));
+  return val;
 #else
   FATAL() << "Reached AArch64-only code path on non-AArch64 architecture";
   return 0;
 #endif
 }
 
+inline unsigned long long cntvct(void) {
+#if defined(__aarch64__)
+  unsigned long long val;
+  asm volatile("mrs %0, CNTVCT_EL0" : "=r" (val));
+  return val;
+#else
+  FATAL() << "Reached AArch64-only code path on non-AArch64 architecture";
+  return 0;
+#endif
+}
+
+inline unsigned long long dczid_el0_block_size(void) {
+#if defined(__aarch64__)
+  unsigned long long val;
+  asm volatile("mrs %0, DCZID_EL0" : "=r" (val));
+  return 4ULL << (val & 0xF);
+#else
+  FATAL() << "Reached AArch64-only code path on non-AArch64 architecture";
+  return 0;
+#endif
+}
+
+/**
+ * If `src` overlaps `dst`, replace the bytes in `dst_data` from the range `dst`
+ * with the corresponding bytes in `src_data` from the range `src`.
+ */
+void replace_in_buffer(MemoryRange src, const uint8_t* src_data,
+                       MemoryRange dst, uint8_t* dst_data);
+
+// Strip any directory part from the filename `s`
+void base_name(std::string& s);
+
+std::optional<int> read_perf_event_paranoid();
+
+bool virtual_address_size_supported(uint8_t bit_size);
+
 } // namespace rr
 
 #endif /* RR_UTIL_H_ */
